@@ -23,8 +23,12 @@ extern "C" {
 }
 #endif /* TAU_PAPI */
 
+#ifdef BGL_TIMERS
+#include <bglpersonality.h>
+#include <rts.h>
+#endif
 
-#define SIZE_OF_INIT_ARRAY 12 //!!Change this if functions are added to the system!!
+#define SIZE_OF_INIT_ARRAY 13 //!!Change this if functions are added to the system!!
 
 
 //Some useful typedefs
@@ -59,6 +63,9 @@ class MultipleCounterLayer
   static bool linuxTimerMCLInit(int functionPosition);
   static void linuxTimerMCL(int tid, double values[]);
 
+  static bool bglTimersMCLInit(int functionPosition);
+  static void bglTimersMCL(int tid, double values[]);
+
   static bool sgiTimersMCLInit(int functionPosition);
   static void sgiTimersMCL(int tid, double values[]);
 
@@ -103,6 +110,11 @@ class MultipleCounterLayer
   static int linuxTimerMCL_FP;
 #endif //TAU_LINUX_TIMERS
 
+#ifdef BGL_TIMERS
+  static int bglTimersMCL_CP[1];
+  static int bglTimersMCL_FP;
+#endif // BGL_TIMERS
+
 #ifdef SGI_TIMERS
   static int sgiTimersMCL_CP[1];
   static int sgiTimersMCL_FP;
@@ -140,10 +152,6 @@ class MultipleCounterLayer
   static int papiMCL_FP; 
   static int papiWallClockMCL_FP;
   static int papiVirtualMCL_FP;
-  //Data specific to the papiMCL function.
-  static int numberOfPapiHWCounters;
-  static int PAPI_CounterCodeList[MAX_TAU_COUNTERS];
-  static ThreadValue * ThreadList[TAU_MAX_THREADS];
 #endif//TAU_PAPI
 
 #ifdef TAU_PCL
