@@ -32,6 +32,40 @@
 #pragma warning( disable : 4786 )
 #endif /* TAU_WINDOWS */
 
+#if (!defined(TAU_LIBRARY_SOURCE) && !defined(TAU_WINDOWS))
+
+#include <sys/types.h>
+#include <unistd.h>
+
+#undef fork
+#define fork() \
+        tau_fork()
+
+#ifdef __cplusplus
+extern "C" 
+#endif
+pid_t tau_fork (void);
+
+#ifdef PTHREADS
+/* pthread_create wrapper */
+
+#include <pthread.h>
+#undef pthread_create
+#define pthread_create(thread, attr, function, arg) \
+        tau_pthread_create(thread, attr, function, arg)
+
+#ifdef __cplusplus
+extern "C" 
+#endif
+int tau_pthread_create (pthread_t *threadp,
+			const pthread_attr_t *attr,
+			void *(*start_routine) (void *),
+			void *arg);
+
+#endif /* PTHREADS */
+#endif /* TAU_LIBRARY_SOURCE */
+
+
 #if (defined (__cplusplus ) && !defined (TAU_USE_C_API))
 
 #include <Profile/ProfileGroups.h>
@@ -57,6 +91,8 @@
 #define TAU_MAX_THREADS 1
 #endif /* PTHREADS || TULIPTHREADS || JAVA || TAU_WINDOWS || OPENMP || SPROC */
 
+
+
 /* We should throttle if number n > a && percall < b .a and b are given below */
 #define TAU_THROTTLE_NUMCALLS_DEFAULT 100000
 #define TAU_THROTTLE_PERCALL_DEFAULT  10
@@ -89,12 +125,22 @@
 
 #include <Profile/WindowsThreadLayer.h>
 
+#include <Profile/TauMemory.h>
+
 #include <Profile/TauMuse.h>
 
 #include <Profile/TauCompensate.h>
 
 #include <Profile/TauHandler.h>
 
+#if defined(TAUKTAU)
+class KtauProfiler;
+#ifdef TAUKTAU_MERGE
+#include <Profile/KtauMergeInfo.h>
+#endif /* defined(TAUKTAU_MERGE) */
+#endif /* defined(TAUKTAU) */
+
+
 //////////////////////////////////////////////////////////////////////
 //
 // class Profiler
@@ -127,6 +173,15 @@ public:
         void CallPathStop(double totaltime, int tid);
 #endif // TAU_MULTIPLE_COUNTERS 
 #endif // TAU_CALLPATH 
+
+#ifdef TAU_PROFILEPARAM
+#ifdef TAU_MULTIPLE_COUNTERS
+        void ProfileParamStop(double* totaltime, int tid);
+#else  // TAU_MULTIPLE_COUNTERS 
+        void ProfileParamStop(double totaltime, int tid);
+#endif // TAU_MULTIPLE_COUNTERS 
+#endif // TAU_PROFILEPARAM 
+
   	static void ProfileExit(const char *message=0, 
 	  int tid = RtsLayer::myThread());
 	static bool createDirectories();
@@ -134,6 +189,8 @@ public:
 	static int DumpData(bool increment = false,
 			    int tid = RtsLayer::myThread(), char *prefix = "dump"); 
 	static void PurgeData(int tid = RtsLayer::myThread());
+	static int Snapshot(char *name, bool finalize = false,
+			    int tid = RtsLayer::myThread()); 
 	static void theFunctionList(const char ***inPtr, int *numOfFunctions,
 				    bool addName = false, const char * inString = NULL);
 	static void dumpFunctionNames();
@@ -181,6 +238,11 @@ public:
 	FunctionInfo * CallPathFunction;
 	bool 	       AddInclCallPathFlag; 
 #endif // TAU_CALLPATH
+#ifdef TAU_PROFILEPARAM
+	FunctionInfo * ProfileParamFunction; 
+	bool 	       AddInclProfileParamFlag; 
+	static void AddProfileParamData(long key, string& keyname);
+#endif // TAU_PROFILEPARAM
 #ifdef TAU_COMPENSATE
 	/* Compensate for instrumentation overhead based on total number of 
 	child calls executed under the given timer */
@@ -213,6 +275,13 @@ public:
 	double& TheTauThrottlePerCall(void);
 	double& TheTauThrottleNumCalls(void);
 
+#if defined(TAUKTAU)
+	KtauProfiler* ThisKtauProfiler;
+#if defined(TAUKTAU_MERGE)
+	KtauMergeInfo ThisKtauMergeInfo;
+#endif /* TAUKTAU_MERGE */
+#endif /* TAUKTAU */
+
 private:
 	TauGroup_t MyProfileGroup_;
 	bool	StartStopUsed_;
@@ -242,6 +311,6 @@ private:
 #endif /* PROFILER_H */
 /***************************************************************************
  * $RCSfile: Profiler.h,v $   $Author: amorris $
- * $Revision: 1.62 $   $Date: 2005/11/11 03:46:47 $
- * POOMA_VERSION_ID: $Id: Profiler.h,v 1.62 2005/11/11 03:46:47 amorris Exp $ 
+ * $Revision: 1.73 $   $Date: 2006/12/27 02:50:21 $
+ * POOMA_VERSION_ID: $Id: Profiler.h,v 1.73 2006/12/27 02:50:21 amorris Exp $ 
  ***************************************************************************/
