@@ -42,6 +42,10 @@ extern "C" void Tau_create_top_level_timer_if_necessary(void);
 extern "C" void Tau_stop_top_level_timer_if_necessary(void);
 extern "C" char * Tau_phase_enable(const char *group);
 
+extern "C" void Tau_profile_snapshot(char *name);
+extern "C" void Tau_profile_snapshot_1l(char *name, int number);
+
+
 #define TAU_TYPE_STRING(profileString, str) static string profileString(str);
 
 #if (TAU_MAX_THREADS == 1)
@@ -94,6 +98,16 @@ extern "C" char * Tau_phase_enable(const char *group);
 #define TAU_PHASE_STOP  TAU_PROFILE_STOP
 #endif /* TAU_PROFILEPHASE */
 
+/* The macros below refer to TAU's FunctionInfo object. This object is created with
+a new call for multi-threaded applications and with a static constructor when 
+a single thread of execution is used. Correspondingly we either use tauFI.method() 
+or tauFI->method();
+*/
+#define TAU_PROFILE_SET_GROUP_NAME(newname) tauFI.SetPrimaryGroupName(newname);
+#define TAU_PROFILE_TIMER_SET_GROUP_NAME(t, newname) t##fi.SetPrimaryGroupName(newname);
+#define TAU_PROFILE_TIMER_SET_NAME(t, newname)	t##fi.SetName(newname);
+#define TAU_PROFILE_TIMER_SET_TYPE(t, newname)  t##fi.SetType(newname);
+#define TAU_PROFILE_TIMER_SET_GROUP(t, id) t##fi.SetProfileGroup(id); 
 
 
 #else  /* TAU_MAX_THREADS */
@@ -166,8 +180,24 @@ extern "C" char * Tau_phase_enable(const char *group);
 #else
 #define TAU_PHASE_STOP TAU_PROFILE_STOP
 #endif /* TAU_PROFILEPHASE */
+
+/* The macros below refer to TAU's FunctionInfo object. This object is created with
+a new call for multi-threaded applications and with a static constructor when 
+a single thread of execution is used. Correspondingly we either use tauFI.method() 
+or tauFI->method();
+*/
+#define TAU_PROFILE_SET_GROUP_NAME(newname) tauFI->SetPrimaryGroupName(newname);
+#define TAU_PROFILE_TIMER_SET_GROUP_NAME(t, newname) t##fi->SetPrimaryGroupName(newname);
+#define TAU_PROFILE_TIMER_SET_NAME(t, newname)	t##fi->SetName(newname);
+#define TAU_PROFILE_TIMER_SET_TYPE(t, newname)  t##fi->SetType(newname);
+#define TAU_PROFILE_TIMER_SET_GROUP(t, id) t##fi->SetProfileGroup(id); 
 #endif  /* TAU_MAX_THREADS */
 
+#ifdef TAU_PROFILEPARAM
+#define TAU_PROFILE_PARAM1L(b,c)  	Profiler::AddProfileParamData(b,c)
+#else   /* TAU_PROFILEPARAM */
+#define TAU_PROFILE_PARAM1L(b,c)  
+#endif /* TAU_PROFILEPARAM */
 
 
 // Construct a Profiler obj and a FunctionInfo obj with an extended name
@@ -183,11 +213,6 @@ extern "C" char * Tau_phase_enable(const char *group);
 #define TAU_INIT(argc, argv) RtsLayer::ProfileInit(*argc, *argv);
 #define TAU_PROFILE_SET_NODE(node) RtsLayer::setMyNode(node);
 #define TAU_PROFILE_SET_CONTEXT(context) RtsLayer::setMyContext(context);
-#define TAU_PROFILE_SET_GROUP_NAME(newname) tauFI.SetPrimaryGroupName(newname);
-#define TAU_PROFILE_TIMER_SET_GROUP_NAME(t, newname) t##fi.SetPrimaryGroupName(newname);
-#define TAU_PROFILE_TIMER_SET_NAME(t, newname)	t##fi.SetName(newname);
-#define TAU_PROFILE_TIMER_SET_TYPE(t, newname)  t##fi.SetType(newname);
-#define TAU_PROFILE_TIMER_SET_GROUP(t, id) t##fi.SetProfileGroup(id); 
 
 #define TAU_GLOBAL_TIMER(timer, name, type, group) FunctionInfo& timer (void) { \
 	static FunctionInfo *timer##fi = NULL; \
@@ -303,7 +328,11 @@ extern "C" char * Tau_phase_enable(const char *group);
 #define TAU_DISABLE_TRACKING_MUSE_EVENTS()	TauDisableTrackingMuseEvents()
 #define TAU_TRACK_MUSE_EVENTS()			TauTrackMuseEvents()
 #define TAU_SET_INTERRUPT_INTERVAL(value)	TauSetInterruptInterval(value)
+#define TAU_NEW(expr, size) 			Tau_new(__FILE__, __LINE__, size, expr)
+#define TAU_DELETE(expr, variable) 		Tau_track_memory_deallocation(__FILE__, __LINE__, variable) , expr
 
+#define TAU_PROFILE_SNAPSHOT(name)              Tau_profile_snapshot(name);
+#define TAU_PROFILE_SNAPSHOT_1L(name, expr)     Tau_profile_snapshot_1l(name, expr);
 
 #ifdef NO_RTTI
 /* #define CT(obj) string(#obj) */
@@ -391,8 +420,13 @@ extern "C" char * Tau_phase_enable(const char *group);
 #define TAU_CONTEXT_EVENT(event, data)
 #define TAU_DISABLE_CONTEXT_EVENT(event)
 #define TAU_ENABLE_CONTEXT_EVENT(event)
+#define TAU_PROFILE_PARAM1L(b,c)  
+#define TAU_TRACK_NEW(expr, size) 		expr
+#define TAU_TRACK_DELETE(expr, variable) 	expr
 
 
+#define TAU_PROFILE_SNAPSHOT(name)
+#define TAU_PROFILE_SNAPSHOT_1L(name, expr)
 
 #define CT(obj)
 
@@ -419,6 +453,6 @@ extern "C" char * Tau_phase_enable(const char *group);
 #endif /* _TAU_API_H_ */
 /***************************************************************************
  * $RCSfile: TauAPI.h,v $   $Author: amorris $
- * $Revision: 1.53 $   $Date: 2005/11/11 03:46:47 $
- * POOMA_VERSION_ID: $Id: TauAPI.h,v 1.53 2005/11/11 03:46:47 amorris Exp $ 
+ * $Revision: 1.59 $   $Date: 2007/01/04 02:36:57 $
+ * POOMA_VERSION_ID: $Id: TauAPI.h,v 1.59 2007/01/04 02:36:57 amorris Exp $ 
  ***************************************************************************/
