@@ -181,6 +181,17 @@ extern "C" {
 
 #define TAU_GLOBAL_TIMER_EXTERNAL(timer)  extern void* TauGlobal##timer(void);
 
+#ifdef TAU_PROFILEPARAM
+#define TAU_PROFILE_PARAM1L(b,c)  	Tau_profile_param1l(b,c)
+#else  /* TAU_PROFILEPARAM */
+#define TAU_PROFILE_PARAM1L(b,c)  	
+#endif /* TAU_PROFILEPARAM */
+
+#define TAU_PROFILE_SNAPSHOT(name)              Tau_profile_snapshot(name);
+#define TAU_PROFILE_SNAPSHOT_1L(name, expr)     Tau_profile_snapshot(name, expr);
+
+
+extern void Tau_specify_mapping_data1(long data, const char *name);
 extern void Tau_bcast_data(int data);
 extern void Tau_reduce_data(int data);
 extern void Tau_alltoall_data(int data);
@@ -246,6 +257,8 @@ extern void Tau_disable_instrumentation(void);
 extern void Tau_global_stop(void);
 extern char * Tau_phase_enable_once(const char *group, void **ptr);
 
+extern void Tau_profile_snapshot(char *name);
+extern void Tau_profile_snapshot_1l(char *name, int number);
 
 
 
@@ -328,6 +341,8 @@ extern char * Tau_phase_enable_once(const char *group, void **ptr);
 #define TAU_GLOBAL_TIMER_EXTERNAL(timer)
 #define TAU_GLOBAL_TIMER_START(timer)
 #define TAU_GLOBAL_TIMER_STOP()
+#define TAU_PROFILE_PARAM1L(b,c)  	
+
 
 
 #endif /* PROFILING_ON */
@@ -355,7 +370,7 @@ extern char * Tau_phase_enable_once(const char *group, void **ptr);
 
 /***************************************************************************
  * $RCSfile: TauCAPI.h,v $   $Author: amorris $
- * $Revision: 1.43 $   $Date: 2005/11/11 03:46:47 $
- * POOMA_VERSION_ID: $Id: TauCAPI.h,v 1.43 2005/11/11 03:46:47 amorris Exp $
+ * $Revision: 1.45 $   $Date: 2007/01/04 02:36:57 $
+ * POOMA_VERSION_ID: $Id: TauCAPI.h,v 1.45 2007/01/04 02:36:57 amorris Exp $
  ***************************************************************************/
 
