@@ -120,6 +120,12 @@
 #define TAU_GLOBAL_TIMER_EXTERNAL(timer)
 #define TAU_GLOBAL_TIMER_START(timer)
 #define TAU_GLOBAL_TIMER_STOP()
+#define TAU_PROFILE_PARAM1L(a,b)
+#define TAU_NEW(expr, size) 		expr
+#define TAU_DELETE(expr, variable) 	expr
+
+#define TAU_PROFILE_SNAPSHOT(name)
+#define TAU_PROFILE_SNAPSHOT_1L(name, expr)
 
 #endif /* PROFILING_ON || TRACING_ON */
 
