@@ -60,15 +60,34 @@ using namespace std;
 #endif //TAU_WINDOWS
 
 #ifdef TRACING_ON
+#ifdef TAU_VAMPIRTRACE 
+#include <Profile/TauVampirTrace.h>
+#else /* TAU_VAMPIRTRACE */
 #ifdef TAU_EPILOG
 #include "elg_trc.h"
 #else /* TAU_EPILOG */
 #define PCXX_EVENT_SRC
 #include "Profile/pcxx_events.h"
 #endif /* TAU_EPILOG */
+#endif /* TAU_VAMPIRTRACE */
 #endif // TRACING_ON 
 
 
+//////////////////////////////////////////////////////////////////////
+// The purpose of this subclass of vector is to give us a chance to execute
+// some code when TheFunctionDB is destroyed.  For Dyninst, this is necessary
+// when running with fortran programs
+//////////////////////////////////////////////////////////////////////
+class FIvector : public vector<FunctionInfo*> {
+public: 
+  ~FIvector() {
+    //printf ("FIvector destructor!\n");
+    if (TheUsingDyninst() && TheSafeToDumpData()) {
+      TAU_PROFILE_EXIT("FunctionDB destructor");
+      TheSafeToDumpData() = 0;
+    }
+  }
+};
 
 //////////////////////////////////////////////////////////////////////
 // Instead of using a global var., use static inside a function  to
@@ -77,7 +96,10 @@ using namespace std;
 //////////////////////////////////////////////////////////////////////
 vector<FunctionInfo*>& TheFunctionDB(void)
 { // FunctionDB contains pointers to each FunctionInfo static object
-  static vector<FunctionInfo*> FunctionDB;
+
+  // we now use the above FIvector, which subclasses vector
+  //static vector<FunctionInfo*> FunctionDB;
+  static FIvector FunctionDB;
 
   return FunctionDB;
 }
@@ -93,6 +115,27 @@ int& TheSafeToDumpData()
   return SafeToDumpData;
 }
 
+//////////////////////////////////////////////////////////////////////
+// Set when uning Dyninst
+//////////////////////////////////////////////////////////////////////
+int& TheUsingDyninst()
+{ 
+  static int UsingDyninst=0;
+  return UsingDyninst;
+}
+
+#ifdef TAU_VAMPIRTRACE
+//////////////////////////////////////////////////////////////////////
+// Initialize VampirTrace Tracing package
+//////////////////////////////////////////////////////////////////////
+int TauInitVampirTrace(void)
+{
+  DEBUGPROFMSG("Calling vt_open"<<endl;);
+  vt_open();
+  return 1;
+}
+#endif /* TAU_VAMPIRTRACE */
+
 #ifdef TAU_EPILOG 
 //////////////////////////////////////////////////////////////////////
 // Initialize EPILOG Tracing package
@@ -173,6 +216,13 @@ void FunctionInfo::FunctionInfoInit(TauGroup_t ProfileGroup,
 	// Important in the presence of concurrent threads.
 	TheFunctionDB().push_back(this);
 #ifdef TRACING_ON
+#ifdef TAU_VAMPIRTRACE
+        static int tau_vt_init=TauInitVampirTrace();
+        string tau_vt_name(Name+" "+Type);
+	FunctionId = vt_def_region(tau_vt_name.c_str(), VT_NO_ID, VT_NO_LNO,
+		VT_NO_LNO, GroupName.c_str(), VT_FUNCTION);
+	DEBUGPROFMSG("vt_def_region: "<<tau_vt_name<<": returns "<<FunctionId<<endl;);
+#else /* TAU_VAMPIRTRACE */
 #ifdef TAU_EPILOG
         static int tau_elg_init=TauInitEpilog();
 	string tau_elg_name(Name+" "+Type);
@@ -191,21 +241,38 @@ void FunctionInfo::FunctionInfoInit(TauGroup_t ProfileGroup,
 	FunctionId = RtsLayer::GenerateUniqueId();
 	SetFlushEvents(tid);
 #endif /* TAU_EPILOG */
+#endif /* TAU_VAMPIRTRACE */
 #endif //TRACING_ON
 	RtsLayer::UnLockDB();
 		
         DEBUGPROFMSG("nct "<< RtsLayer::myNode() <<"," 
 	  << RtsLayer::myContext() << ", " << tid 
           << " FunctionInfo::FunctionInfo(n,t) : Name : "<< GetName() 
-	  << " Group :  " << MyProfileGroup_ 
+          << " Group :  " << GetProfileGroup()
 	  << " Type : " << GetType() << endl;);
 
 #ifdef TAU_PROFILEMEMORY
 	MemoryEvent = new TauUserEvent(string(Name+" "+Type+" - Heap Memory Used (KB)").c_str());
 #endif /* TAU_PROFILEMEMORY */
+
 #ifdef TAU_PROFILEHEADROOM
 	HeadroomEvent = new TauUserEvent(string(Name+" "+Type+" - Memory Headroom Available (MB)").c_str());
 #endif /* TAU_PROFILEHEADROOM */
+
+#ifdef RENCI_STFF
+#ifdef TAU_MULTIPLE_COUNTERS
+    for (int t=0; t < TAU_MAX_THREADS; t++) {
+        for (int i=0; i < MAX_TAU_COUNTERS; i++) {
+            Signatures[t][i] = NULL;
+        }
+    }
+ #else // TAU_MULTIPLE_COUNTERS
+    for (int t=0; t < TAU_MAX_THREADS; t++) {
+        Signatures[t] = NULL;
+    }
+#endif // TAU_MULTIPLE_COUNTERS
+#endif //RENCI_STFF
+
 	return;
 }
 //////////////////////////////////////////////////////////////////////
@@ -431,6 +498,6 @@ void tauCreateFI(FunctionInfo **ptr, const string& name, const string& type,
 }
 /***************************************************************************
  * $RCSfile: FunctionInfo.cpp,v $   $Author: sameer $
- * $Revision: 1.41 $   $Date: 2005/11/11 18:27:00 $
- * POOMA_VERSION_ID: $Id: FunctionInfo.cpp,v 1.41 2005/11/11 18:27:00 sameer Exp $ 
+ * $Revision: 1.47 $   $Date: 2007/02/09 21:00:07 $
+ * POOMA_VERSION_ID: $Id: FunctionInfo.cpp,v 1.47 2007/02/09 21:00:07 sameer Exp $ 
  ***************************************************************************/
