@@ -50,11 +50,27 @@
  #include "tauarch.h"
 #endif
 
+#if (defined(__QK_USER__) || defined(__LIBCATAMOUNT__ ))
+#ifndef TAU_CATAMOUNT
+#define TAU_CATAMOUNT 
+#endif /* TAU_CATAMOUNT */
+#include <catamount/dclock.h>
+#endif /* __QK_USER__ || __LIBCATAMOUNT__ */
+
 #ifdef CRAY_TIMERS
+#ifndef TAU_CATAMOUNT
+/* These header files are for Cray X1 */
 #include <intrinsics.h>
 #include <sys/param.h>
+#endif /* TAU_CATAMOUNT */
 #endif // CRAY_TIMERS
 
+#ifdef BGL_TIMERS
+/* header files for BlueGene/L */
+#include <bglpersonality.h>
+#include <rts.h>
+#endif // BGL_TIMERS
+
 #ifdef TAU_XLC
 #define strcasecmp strcmp
 #define strncasecmp strncmp 
@@ -112,6 +128,13 @@ using namespace std;
 #include "Profile/pcxx_events.h"
 #endif // TRACING_ON 
 
+#ifdef TAUKTAU
+#include <Profile/ktau_timer.h>
+#include <sys/time.h>
+#include <time.h>
+#include <unistd.h>
+#include <asm/unistd.h>
+#endif /* TAUKTAU */
 
 /////////////////////////////////////////////////////////////////////////
 // Member Function Definitions For class RtsLayer
@@ -282,11 +305,15 @@ int RtsLayer::setMyNode(int NodeId, int tid) {
   TheNode() = NodeId;
 // At this stage, we should create the trace file because we know the node id
 #ifdef TRACING_ON
+#ifdef TAU_VAMPIRTRACE
+// Vampirtrace specific function not needed here 
+#else
 #ifdef TAU_EPILOG
 // EPILOG specific function not needed here. 
 #else /* TAU_EPILOG */
   TraceEvInit(tid);
 #endif /* TAU_EPILOG */
+#endif /* TAU_VAMPIRTRACE */
 #endif // TRACING_ON
   return TheNode();
 }
@@ -399,24 +426,48 @@ double getUserTimeInSec(void)
 #ifdef TAU_LINUX_TIMERS
 
 ///////////////////////////////////////////////////////////////////////////
-inline double TauGetMHzRatings(void)
+int TauReadFullLine(char ** line, FILE *fp)
 {
-  FILE *f;
-  double rating;
-  char *cmd = "cat /proc/cpuinfo | egrep -i '^cpu MHz' | head -1 | sed 's/^.*: //'";
-  char buf[BUFSIZ];
-  if ((f = popen(cmd,"r"))!=NULL)
-  while (fgets(buf, BUFSIZ, f) != NULL)
+  int ch, i;
+  i = 0; 
+  while ( (ch = fgetc(fp)) && ch != EOF && ch != (int) '\n')
   {
-    rating = atof(buf);
+    line[0][i++] = (unsigned char) ch;
   }
-  pclose(f);
-#ifdef DEBUG_PROF
-  printf("Rating = %g Mhz\n", rating);
-#endif /* DEBUG_PROF */
-  return rating;
+  line[0][i] = '\0'; 
+  return i; 
 }
 
+///////////////////////////////////////////////////////////////////////////
+double TauGetMHzRatings(void)
+{
+  float ret;
+  char *line;
+
+  FILE *fp;
+  ret = 0;
+  line = new char[1024]; 
+  fp = fopen("/proc/cpuinfo", "r");
+
+  if (fp)
+  {
+    while(TauReadFullLine(&line, fp))
+    {
+      if (strncmp(line, "cpu MHz", 7) == 0)
+      {
+        sscanf(line,"cpu MHz         : %f", &ret);
+        delete line; 
+        return (double) ret; 
+      }
+    }
+  }
+  else
+  {
+    perror("/proc/cpuinfo file not found:");
+  }
+  delete line; 
+  return (double) ret;
+}
   
 ///////////////////////////////////////////////////////////////////////////
 inline double TauGetMHz(void)
@@ -424,13 +475,26 @@ inline double TauGetMHz(void)
   static double ratings = TauGetMHzRatings();
   return ratings;
 }
-
 ///////////////////////////////////////////////////////////////////////////
 extern "C" unsigned long long getLinuxHighResolutionTscCounter(void);
 // Moved to TauLinuxTimers.c 
 
 #endif /* TAU_LINUX_TIMERS */
 
+#if defined(TAUKTAU) || defined(TAUKTAU_MERGE)
+///////////////////////////////////////////////////////////////////////////
+double KTauGetMHz(void)
+{
+#ifdef KTAU_WALLCLOCK
+  static double ktau_ratings = 1; //(microsec resolution from kernel)
+#else
+  static double ktau_ratings = cycles_per_sec()/1000000; //we need ratings per microsec to match tau's reporting
+  //static double ktau_ratings = TauGetMHz(); //we need ratings per microsec to match tau's reporting
+#endif
+  return ktau_ratings;
+}
+#endif /* TAUKTAU || TAUKTAU_MERGE */
+
 ///////////////////////////////////////////////////////////////////////////
 double TauWindowsUsecD(void)
 {
@@ -513,26 +577,42 @@ double TauWindowsUsecD(void)
 }
 ///////////////////////////////////////////////////////////////////////////
 
+#ifdef TAUKTAU_MERGE
+  //declare the sys_ktau_gettimeofday syscall
+  //#define __NR_ktau_gettimeofday ???
+  //_syscall2(int,ktau_gettimeofday,struct timeval *,tv,struct timezone *,tz);
+  extern "C" int ktau_gettimeofday(struct timeval *tv, struct timezone *tz);
+#endif // TAUKTAU_MERGE 
+
 #ifdef TAU_MULTIPLE_COUNTERS
 void RtsLayer::getUSecD (int tid, double *values){
+#if (defined(TAU_EPILOG) && !defined(PROFILING_ON)) 
+  return;
+#endif /* TAU_EPILOG, PROFILING_ON */
   MultipleCounterLayer::getCounters(tid, values);
 }
 #else //TAU_MULTIPLE_COUNTERS
 double RtsLayer::getUSecD (int tid) {
+
+#if (defined(TAU_EPILOG) && !defined(PROFILING_ON)) 
+  return 0;
+#endif /* TAU_EPILOG, PROFILING_ON */
+
+
 #ifdef TAU_PCL
   return PCL_Layer::getCounters(tid);
 #else  // TAU_PCL
 #ifdef TAU_PAPI
   static const char *papi_env = getenv("PAPI_EVENT");
   if (papi_env != NULL)
-    return PapiLayer::getCounters(tid);
+    return PapiLayer::getSingleCounter(tid);
 #ifdef TAU_PAPI_WALLCLOCKTIME
   return PapiLayer::getWallClockTime();
 #else /* TAU_PAPI_WALLCLOCKTIME */
 #ifdef TAU_PAPI_VIRTUAL
   return PapiLayer::getVirtualTime();
 #else  /* TAU_PAPI_VIRTUAL */
-  return PapiLayer::getCounters(tid);
+  return PapiLayer::getSingleCounter(tid);
 #endif /* TAU_PAPI_VIRTUAL */
 #endif /* TAU_PAPI_WALLCLOCKTIME */
 #else  // TAU_PAPI
@@ -542,6 +622,31 @@ double RtsLayer::getUSecD (int tid) {
 #ifdef JAVA_CPU_TIME
   return JavaThreadLayer::getCurrentThreadCpuTime();
 #else // JAVA_CPU_TIME
+#ifdef TAUKTAU_MERGE
+  struct timeval tp;
+  static double last_timestamp = 0.0;
+  double timestamp;
+  ktau_gettimeofday (&tp, 0);
+  timestamp = (double) tp.tv_sec * 1e6 + tp.tv_usec;
+  if (timestamp < last_timestamp)
+  {
+     DEBUGPROFMSG("RtsLayer::getUSecD(): ktau_gettimeofday() goes back in time. Fixing ...."<<endl;);
+     timestamp = last_timestamp;
+  }
+  last_timestamp = timestamp;
+  return timestamp;
+#else // TAUKTAU_MERGE
+#ifdef BGL_TIMERS
+  static double bgl_clockspeed = 0.0;
+
+  if (bgl_clockspeed == 0.0)
+  {
+    BGLPersonality mybgl;
+    rts_get_personality(&mybgl, sizeof(BGLPersonality));
+    bgl_clockspeed = 1.0e6/(double)BGLPersonality_clockHz(&mybgl);
+  }
+  return (rts_get_timebase() * bgl_clockspeed);
+#else // BGL_TIMERS
 #ifdef SGI_HW_COUNTERS
   return RtsLayer::GetEventCounter();
 #else  //SGI_HW_COUNTERS
@@ -553,8 +658,12 @@ double RtsLayer::getUSecD (int tid) {
 
 #else  // SGI_TIMERS
 #ifdef CRAY_TIMERS
+#ifdef TAU_CATAMOUNT /* for Cray XT3 */
+  return dclock()*1.0e6; 
+#else /* for Cray X1 */
   long long tick = _rtc();
   return (double) tick/HZ;
+#endif /* TAU_CATAMOUNT */
 #endif // CRAY_TIMERS
 #ifdef TAU_ALPHA_TIMERS
   struct timespec currenttime;
@@ -648,6 +757,8 @@ double RtsLayer::getUSecD (int tid) {
 #endif // SGI_TIMERS
 
 #endif // SGI_HW_COUNTERS
+#endif // BGL_TIMERS
+#endif // TAUKTAU_MERGE
 #endif // JAVA_CPU_TIME
 #endif // CPU_TIME
 #endif // TAU_PAPI
@@ -1295,7 +1406,7 @@ std::string RtsLayer::GetRTTI(const char *name)
 }
 
 /***************************************************************************
- * $RCSfile: RtsLayer.cpp,v $   $Author: amorris $
- * $Revision: 1.69 $   $Date: 2005/11/11 03:46:49 $
- * POOMA_VERSION_ID: $Id: RtsLayer.cpp,v 1.69 2005/11/11 03:46:49 amorris Exp $ 
+ * $RCSfile: RtsLayer.cpp,v $   $Author: sameer $
+ * $Revision: 1.81 $   $Date: 2007/02/13 02:20:45 $
+ * POOMA_VERSION_ID: $Id: RtsLayer.cpp,v 1.81 2007/02/13 02:20:45 sameer Exp $ 
  ***************************************************************************/
