@@ -36,6 +36,13 @@ using namespace std;
 #include <stdio.h>
 #include <stdlib.h>
 
+#if (!defined(TAU_WINDOWS)) 
+/* Needed for fork */
+#include <sys/types.h>
+#include <unistd.h>
+#endif /* TAU_WINDOWS */
+
+
 ///////////////////////////////////////////////////////////////////////////
 // Wrappers for corresponding C++ functions follow
 
@@ -358,6 +365,12 @@ int register_events(void)
 extern "C" void Tau_trace_sendmsg(int type, int destination, int length)
 {
   static int initialize = register_events();
+#ifdef TAU_PROFILEPARAM
+#ifndef TAU_DISABLE_PROFILEPARAM_IN_MPI
+  static string s("message size");
+  TAU_PROFILE_PARAM1L(length, s);
+#endif /* TAU_DISABLE_PROFILEPARAM_IN_MPI */
+#endif  /* TAU_PROFILEPARAM */
 #ifdef DEBUG_PROF
   printf("Node %d: Tau_trace_sendmsg: type %d dest %d len %d\n", 
         RtsLayer::myNode(), type, destination, length);
@@ -373,6 +386,12 @@ extern "C" void Tau_trace_sendmsg(int type, int destination, int length)
 extern "C" void Tau_trace_recvmsg(int type, int source, int length)
 {
   TAU_EVENT(TheRecvEvent(), length);
+#ifdef TAU_PROFILEPARAM
+#ifndef TAU_DISABLE_PROFILEPARAM_IN_MPI
+  static string s("message size");
+  TAU_PROFILE_PARAM1L(length, s);
+#endif /* TAU_DISABLE_PROFILEPARAM_IN_MPI */
+#endif  /* TAU_PROFILEPARAM */
   if (source >= 0) 
     TAU_TRACE_RECVMSG(type, source, length);
 }
@@ -689,10 +708,80 @@ extern "C" void Tau_mark_group_as_phase(void **ptr)
   fptr->SetPrimaryGroupName(newgroup); 
 
 }
+///////////////////////////////////////////////////////////////////////////
+extern "C" void Tau_profile_param1l(long data, const char * dataname)
+{
+  string dname(dataname);
+  TAU_PROFILE_PARAM1L(data, dname);
+}
+
+
+/*
+  The following is for supporting pure and elemental fortran subroutines
+*/
+
+static map<string, FunctionInfo *> pureMap;
+
+extern "C" void Tau_pure_start(char *name) {
+  FunctionInfo *fi = 0;
+  string n = string(name);
+  map<string, FunctionInfo *>::iterator it = pureMap.find(n);
+  if (it == pureMap.end()) {
+    tauCreateFI(&fi,n,"",TAU_USER,"TAU_USER");
+    pureMap[n] = fi;
+  } else {
+    fi = (*it).second;
+  }
+  Tau_start_timer(fi,0);
+}
+
+extern "C" void Tau_pure_stop(char *name) {
+  FunctionInfo *fi;
+  string n = string(name);
+  map<string, FunctionInfo *>::iterator it = pureMap.find(n);
+  fi = (*it).second;
+  Tau_stop_timer(fi);
+}
+
+#if (!defined(TAU_WINDOWS))
+extern "C" pid_t tau_fork() {
+  pid_t pid;
+
+  pid = fork();
+
+#ifdef TAU_WRAP_FORK
+  if (pid == 0) {
+    TAU_REGISTER_FORK(getpid(), TAU_EXCLUDE_PARENT_DATA);
+//     fprintf (stderr, "[%d] Registered Fork!\n", getpid());
+  } else {
+    /* nothing */
+  }
+#endif
+
+  return pid;
+}
+#endif /* TAU_WINDOWS */
+
+
+//////////////////////////////////////////////////////////////////////
+// Snapshot related routines
+//////////////////////////////////////////////////////////////////////
+
+extern "C" void Tau_profile_snapshot_1l(char *name, int number) {
+  char buffer[4096];
+  sprintf (buffer, "%s %d", name, number);
+  Profiler::Snapshot(buffer);
+}
+
+extern "C" void Tau_profile_snapshot(char *name) {
+  Profiler::Snapshot(name);
+}
+
+
 
 /***************************************************************************
  * $RCSfile: TauCAPI.cpp,v $   $Author: amorris $
- * $Revision: 1.54 $   $Date: 2005/11/11 03:46:49 $
- * VERSION: $Id: TauCAPI.cpp,v 1.54 2005/11/11 03:46:49 amorris Exp $
+ * $Revision: 1.61 $   $Date: 2007/01/04 02:36:29 $
+ * VERSION: $Id: TauCAPI.cpp,v 1.61 2007/01/04 02:36:29 amorris Exp $
  ***************************************************************************/
 
