@@ -33,6 +33,11 @@
 #include <string.h>
 #include "Profile/ProfileGroups.h"
 
+
+
+extern "C" void Tau_pure_start(char *name);
+extern "C" void Tau_pure_stop(char *name);
+
 /* 
 #define DEBUG_PROF
 */
@@ -81,7 +86,9 @@ void Tau_enable_tracking_memory_headroom(void);
 void Tau_disable_tracking_memory_headroom(void);
 void Tau_mark_group_as_phase(void **ptr);
 void Tau_profile_callstack(void );
-
+void Tau_profile_snapshot(char *name);
+void Tau_profile_snapshot_1l(char *name, int number);
+  
 
 
 #define EXTRACT_GROUP(n, l, gr, gr_name) TauGroup_t gr; char *gr_name = NULL; tau_extract_groupinfo(n, gr, gr_name); 
@@ -160,6 +167,61 @@ void tau_extract_groupinfo(char *& fname, TauGroup_t & gr, char *& gr_name)
 }
 
 
+void tau_pure_start(char *fname, int flen) {
+  // make a copy so that we can null terminate it
+  char *localname = (char *) malloc((size_t)flen+1);
+  strncpy(localname, fname, flen);
+  localname[flen] = '\0';
+  
+  // check for unprintable characters
+  for(int i=0; i<strlen(localname); i++) {
+    if (!VALID_NAME_CHAR(localname[i])) { 
+      localname[i] = '\0';
+      break;
+    }
+  }
+
+  Tau_pure_start(localname);
+  free(localname);
+}
+
+void TAU_PURE_START(char *fname, int flen) {
+  tau_pure_start(fname, flen);
+}
+void tau_pure_start_(char *fname, int flen) {
+  tau_pure_start(fname, flen);
+}
+void tau_pure_start__(char *fname, int flen) {
+  tau_pure_start(fname, flen);
+}
+
+void tau_pure_stop(char *fname, int flen) {
+  // make a copy so that we can null terminate it
+  char *localname = (char *) malloc((size_t)flen+1);
+  strncpy(localname, fname, flen);
+  localname[flen] = '\0';
+  
+  // check for unprintable characters
+  for(int i=0; i<strlen(localname); i++) {
+    if (!VALID_NAME_CHAR(localname[i])) { 
+      localname[i] = '\0';
+      break;
+    }
+  }
+
+  Tau_pure_stop(localname);
+  free(localname);
+}
+void TAU_PURE_STOP(char *fname, int flen) {
+  tau_pure_stop(fname, flen);
+}
+void tau_pure_stop_(char *fname, int flen) {
+  tau_pure_stop(fname, flen);
+}
+void tau_pure_stop__(char *fname, int flen) {
+  tau_pure_stop(fname, flen);
+}
+
 void tau_profile_timer_(void **ptr, char *fname, int flen)
 {
   
@@ -178,9 +240,13 @@ void tau_profile_timer_(void **ptr, char *fname, int flen)
 
     // make a copy so that we can null terminate it
     char *localname = (char *) malloc((size_t)flen+1);
+    char *modname = (char *) malloc((size_t)flen+1);
     // hold on to the original pointer to free it since EXTRACT_GROUP
     // might change fname
     char *tmp = localname;
+    char *tmp2 = modname;
+    int skipwhite = 1;
+    int idx = 0;
     strncpy(localname, fname, flen);
     localname[flen] = '\0';
 
@@ -192,10 +258,27 @@ void tau_profile_timer_(void **ptr, char *fname, int flen)
       }
     }
 
-    EXTRACT_GROUP(localname, flen, gr, gr_name)
+    // fix continuation lines
+    for(int j=0; j<strlen(localname); j++) {
+      if (localname[j] == '&') {
+	skipwhite = 1;
+      } else {
+	if (skipwhite && localname[j] == ' ') {
+	  // nothing, skip over it
+	} else {
+	  modname[idx++] = localname[j];
+	  skipwhite = 0;
+	}
+      }
+    }
+    modname[idx] = 0;
+    localname = modname;
+
+    EXTRACT_GROUP(localname, flen, gr, gr_name);
+
     *ptr = Tau_get_profiler(localname, (char *)" ", gr, gr_name);
     free(tmp); 
-
+    free(tmp2);
 #ifdef TAU_OPENMP
       }
     }
@@ -1327,11 +1410,59 @@ void TAU_PROFILE_CALLSTACK(void)
   tau_profile_callstack_();
 }
 
+//////////////////////////////////////////////////////////////////////
+// Snapshot related routines
+//////////////////////////////////////////////////////////////////////
+
+static char *getFortranName(char *name, int slen) {
+    char *fname = (char *) malloc((size_t) slen+1);
+    strncpy(fname, name, slen);
+    fname[slen] = '\0';  
+    return fname;
+}
+
+void tau_profile_snapshot_1l_(int *number, char *name, int slen) {
+  char *fname = getFortranName(name, slen);
+  Tau_profile_snapshot_1l(fname, *number);
+  free (fname);
+}
+
+void tau_profile_snapshot_1l(int *number, char *name, int slen) {
+  tau_profile_snapshot_1l_(number, name, slen);
+}
+
+void tau_profile_snapshot_1l__(int *number, char *name, int slen) {
+  tau_profile_snapshot_1l_(number, name, slen);
+}
+
+void TAU_PROFILE_SNAPSHOT_1L(int *number, char *name, int slen) {
+  tau_profile_snapshot_1l_(number, name, slen);
+}
+
+void tau_profile_snapshot_(char *name, int slen) {
+  char *fname = getFortranName(name, slen);
+  Tau_profile_snapshot(fname);
+  free (fname);
+}
+
+void tau_profile_snapshot(char *name, int slen) {
+  tau_profile_snapshot_(name, slen);
+}
+
+void tau_profile_snapshot__(char *name, int slen) {
+  tau_profile_snapshot_(name, slen);
+}
+
+void TAU_PROFILE_SNAPSHOT(char *name, int slen) {
+  tau_profile_snapshot_(name, slen);
+}
+
+
 } /* extern "C" */
 
 
 /***************************************************************************
  * $RCSfile: TauFAPI.cpp,v $   $Author: amorris $
- * $Revision: 1.50 $   $Date: 2005/11/11 03:46:49 $
- * POOMA_VERSION_ID: $Id: TauFAPI.cpp,v 1.50 2005/11/11 03:46:49 amorris Exp $ 
+ * $Revision: 1.55 $   $Date: 2007/01/04 02:36:29 $
+ * POOMA_VERSION_ID: $Id: TauFAPI.cpp,v 1.55 2007/01/04 02:36:29 amorris Exp $ 
  ***************************************************************************/
