@@ -14,35 +14,41 @@
  * only one set of drawing and sorting data at a time. Different windows must
  * create their own instances of this object to avoid conflicts.
  *  
+ *  
+ *  NOTE: This class will soon be obsolete and be removed as it is an unnecessary waste of memory
+ *  
  */
 
 package edu.uoregon.tau.paraprof;
 
+import java.awt.Color;
 import java.text.DecimalFormat;
-import java.util.*;
-import java.awt.*;
-import edu.uoregon.tau.paraprof.enums.*;
+import java.util.Iterator;
+
+import edu.uoregon.tau.paraprof.enums.SortType;
+import edu.uoregon.tau.paraprof.enums.ValueType;
 import edu.uoregon.tau.perfdmf.*;
+import edu.uoregon.tau.perfdmf.Thread;
 
 public class PPFunctionProfile implements Comparable {
 
     private DataSorter dataSorter;
-    private edu.uoregon.tau.perfdmf.Thread thread;
-
+    private Thread thread;
     private FunctionProfile functionProfile;
-    private FunctionProfile meanProfile;
 
-
-    public PPFunctionProfile(DataSorter dataSorter, edu.uoregon.tau.perfdmf.Thread thread, FunctionProfile fp) {
+    
+    public PPFunctionProfile(DataSorter dataSorter, Thread thread, FunctionProfile fp) {
         this.dataSorter = dataSorter;
         this.thread = thread;
         this.functionProfile = fp;
-        // prefetch this
-        this.meanProfile = functionProfile.getFunction().getMeanProfile();
     }
 
     public FunctionProfile getMeanProfile() {
-        return meanProfile;
+        return functionProfile.getFunction().getMeanProfile();
+    }
+
+    public int getID() {
+        return functionProfile.getFunction().getID();
     }
 
     public int getNodeID() {
@@ -65,7 +71,8 @@ public class PPFunctionProfile implements Comparable {
         return functionProfile.getFunction();
     }
 
-    public String getFunctionName() {
+    // handles reversed callpaths
+    public String getDisplayName() {
         if (ParaProf.preferences.getReversedCallPaths()) {
             return functionProfile.getFunction().getReversedName();
         } else {
@@ -73,10 +80,6 @@ public class PPFunctionProfile implements Comparable {
         }
     }
 
-    public String getFunctionNameReversedCallPath() {
-        return functionProfile.getFunction().getReversedName();
-    }
-
     public Color getColor() {
         return functionProfile.getFunction().getColor();
     }
@@ -94,7 +97,7 @@ public class PPFunctionProfile implements Comparable {
     }
 
     public double getExclusiveValue() {
-        if (functionProfile.getFunction().isPhase()) {
+        if (functionProfile.getFunction().isPhase() && functionProfile.getFunction().isCallPathFunction()) {
             return functionProfile.getInclusive(dataSorter.getSelectedMetricID());
         } else {
             return functionProfile.getExclusive(dataSorter.getSelectedMetricID());
@@ -121,8 +124,6 @@ public class PPFunctionProfile implements Comparable {
         return functionProfile.getInclusivePerCall(dataSorter.getSelectedMetricID());
     }
 
-    //Parent/child interface.
-
     public Iterator getChildProfiles() {
         return functionProfile.getChildProfiles();
     }
@@ -139,37 +140,44 @@ public class PPFunctionProfile implements Comparable {
         return dataSorter.getValueType().getValue(this.getFunctionProfile(), dataSorter.getSelectedMetricID());
     }
 
-    //   
-
     private int checkDescending(int value) {
-        if (dataSorter.getDescendingOrder())
+        if (dataSorter.getDescendingOrder()) {
             return -value;
+        }
         return value;
     }
 
+    
+    private int compareNCT(PPFunctionProfile other) {
+        if (other.getNodeID() != this.getNodeID()) {
+            return checkDescending(this.getNodeID() - other.getNodeID());
+        } else if (other.getContextID() != this.getContextID()) {
+            return checkDescending(this.getContextID() - other.getContextID());
+        } else {
+            return checkDescending(this.getThreadID() - other.getThreadID());
+        }
+    }
+    
     public int compareTo(Object inObject) {
-        ValueType valueType = dataSorter.getValueType();
+        ValueType valueType = dataSorter.getSortValueType();
 
         PPFunctionProfile other = (PPFunctionProfile) inObject;
 
+        //TODO: I think valueType might need to be sortValueType???
         if (dataSorter.getSortType() == SortType.NAME) {
-            return checkDescending(other.getFunctionName().compareTo(this.getFunctionName()));
-        } else if (dataSorter.getSortType() == SortType.NCT) {
-            if (other.getNodeID() != this.getNodeID())
-                return checkDescending(this.getNodeID() - other.getNodeID());
-            else if (other.getContextID() != this.getContextID())
-                return checkDescending(this.getContextID() - other.getContextID());
-            else
-                return checkDescending(this.getThreadID() - other.getThreadID());
-        } else if (dataSorter.getSortType() == SortType.MEAN_VALUE) {
+            return checkDescending(other.getDisplayName().compareTo(this.getDisplayName()));
 
-            return checkDescending(compareToHelper(valueType.getValue(this.meanProfile, dataSorter.getSelectedMetricID()),
-                    valueType.getValue(other.meanProfile, dataSorter.getSelectedMetricID()), this.meanProfile, other.meanProfile));
+        } else if (dataSorter.getSortType() == SortType.NCT) {
+            return compareNCT(other);
+        
+        } else if (dataSorter.getSortType() == SortType.MEAN_VALUE) {
+            return checkDescending(compareToHelper(valueType.getValue(this.getMeanProfile(), dataSorter.getSortMetric()),
+                    valueType.getValue(other.getMeanProfile(), dataSorter.getSortMetric()), this.getMeanProfile(),
+                    other.getMeanProfile()));
 
         } else if (dataSorter.getSortType() == SortType.VALUE) {
-            return checkDescending(compareToHelper(
-                    valueType.getValue(this.getFunctionProfile(), dataSorter.getSelectedMetricID()), valueType.getValue(
-                            other.getFunctionProfile(), dataSorter.getSelectedMetricID())));
+            return checkDescending(compareToHelper(valueType.getValue(this.getFunctionProfile(), dataSorter.getSortMetric()),
+                    valueType.getValue(other.getFunctionProfile(), dataSorter.getSortMetric())));
         } else {
             throw new ParaProfException("Unexpected sort type: " + dataSorter.getSortType());
         }
@@ -177,12 +185,13 @@ public class PPFunctionProfile implements Comparable {
 
     private int compareToHelper(double d1, double d2) {
         double result = d1 - d2;
-        if (result < 0.00)
+        if (result < 0.00) {
             return -1;
-        else if (result == 0.00)
+        } else if (result == 0.00) {
             return 0;
-        else
+        } else {
             return 1;
+        }
     }
 
     private int compareToHelper(double d1, double d2, FunctionProfile f1, FunctionProfile f2) {
@@ -199,7 +208,6 @@ public class PPFunctionProfile implements Comparable {
         }
     }
 
-
     public String getStatString(int type) {
 
         int metric = dataSorter.getSelectedMetricID();
@@ -208,7 +216,7 @@ public class PPFunctionProfile implements Comparable {
         DecimalFormat dF = new DecimalFormat("##0.0");
         tmpString = UtilFncs.lpad(dF.format(functionProfile.getInclusivePercent(metric)), 13);
 
-        if (functionProfile.getFunction().isPhase()) {
+        if (functionProfile.getFunction().isPhase() && functionProfile.getFunction().isCallPathFunction()) {
             tmpString = tmpString + "  " + UtilFncs.getOutputString(type, functionProfile.getInclusive(metric), 14);
         } else {
             tmpString = tmpString + "  " + UtilFncs.getOutputString(type, functionProfile.getExclusive(metric), 14);
@@ -222,22 +230,17 @@ public class PPFunctionProfile implements Comparable {
         return tmpString;
     }
 
-    // Static Functions
-
-    public static String getStatStringHeading(String metricType) {
-        //        return UtilFncs.lpad("%Total " + metricType, 13) + UtilFncs.lpad(metricType, 16)
-        //        + UtilFncs.lpad("Total " + metricType, 18) + UtilFncs.lpad("#Calls", 14)
-        //        + UtilFncs.lpad("#Child Calls", 14) + UtilFncs.lpad("Total " + metricType + "/Call", 21) + "   ";
-        return UtilFncs.lpad("%Total " + metricType, 13) + UtilFncs.lpad("Exclusive", 16) + UtilFncs.lpad("Inclusive", 18)
-                + UtilFncs.lpad("#Calls", 14) + UtilFncs.lpad("#Child Calls", 14) + UtilFncs.lpad("Inclusive/Call", 21) + "   ";
-    }
-
     public String toString() {
         return functionProfile.toString();
     }
 
-    public edu.uoregon.tau.perfdmf.Thread getThread() {
+    public Thread getThread() {
         return thread;
     }
 
+    public static String getStatStringHeading(String metricType) {
+        return UtilFncs.lpad("%Total " + metricType, 13) + UtilFncs.lpad("Exclusive", 16) + UtilFncs.lpad("Inclusive", 18)
+                + UtilFncs.lpad("#Calls", 14) + UtilFncs.lpad("#Child Calls", 14) + UtilFncs.lpad("Inclusive/Call", 21) + "   ";
+    }
+
 }
\ No newline at end of file
