@@ -4,15 +4,27 @@ import jargs.gnu.CmdLineParser;
 
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
-import java.io.*;
+import java.io.File;
+import java.io.FileInputStream;
+import java.io.FileNotFoundException;
+import java.io.FileOutputStream;
+import java.io.IOException;
+import java.io.ObjectInputStream;
+import java.io.ObjectOutputStream;
 import java.net.URL;
 import java.net.URLDecoder;
+import java.util.ArrayList;
 import java.util.Iterator;
 import java.util.List;
 
 import javax.swing.ToolTipManager;
 
+import edu.uoregon.tau.common.TauScripter;
+import edu.uoregon.tau.paraprof.interfaces.EclipseHandler;
+import edu.uoregon.tau.paraprof.script.ParaProfScript;
+import edu.uoregon.tau.paraprof.sourceview.SourceManager;
 import edu.uoregon.tau.perfdmf.DataSource;
+import edu.uoregon.tau.perfdmf.DataSourceExport;
 import edu.uoregon.tau.perfdmf.FileList;
 import edu.uoregon.tau.perfdmf.UtilFncs;
 
@@ -20,11 +32,11 @@ import edu.uoregon.tau.perfdmf.UtilFncs;
  * ParaProf This is the 'main' for paraprof
  * 
  * <P>
- * CVS $Id: ParaProf.java,v 1.43 2005/09/26 21:12:07 amorris Exp $
+ * CVS $Id: ParaProf.java,v 1.59 2007/01/04 01:55:31 amorris Exp $
  * </P>
  * 
  * @author Robert Bell, Alan Morris
- * @version $Revision: 1.43 $
+ * @version $Revision: 1.59 $
  */
 public class ParaProf implements ActionListener {
 
@@ -45,56 +57,63 @@ public class ParaProf implements ActionListener {
 
     private final static String VERSION = "XXXXX";
 
-    static ColorMap colorMap = new ColorMap();
-
-    //System wide stuff.
-    static File paraProfHomeDirectory = null;
     public static int defaultNumberPrecision = 6;
-    //static ParaProfLisp paraProfLisp = null;
-    public static Preferences preferences = null;
-    public static ColorChooser colorChooser;
 
-    public static ParaProfManagerWindow paraProfManagerWindow = null;
+    public static File paraProfHomeDirectory;
+    public static Preferences preferences = new Preferences();
+    public static ColorChooser colorChooser;
+    public static ColorMap colorMap = new ColorMap();
+
+    public static ParaProfManagerWindow paraProfManagerWindow;
     public static ApplicationManager applicationManager = new ApplicationManager();
-    public static HelpWindow helpWindow = null;
+    private static HelpWindow helpWindow;
     public static PreferencesWindow preferencesWindow;
-    public static Runtime runtime = null;
+    public static Runtime runtime;
     private static int numWindowsOpen = 0;
-    //End - System wide stuff.
 
     //Command line options related.
     private static int fileType = 0; //0:profile, 1:pprof, 2:dynaprof, 3:mpip, 4:hpmtoolkit, 5:gprof, 6:psrun, 7:ppk, 8:cube
     private static File sourceFiles[] = new File[0];
     private static boolean fixNames = false;
     private static boolean monitorProfiles;
+    private static String args[];
     //End - Command line options related.
 
     public static boolean demoMode;
-    
+    public static boolean usePathNameInTrial = false;
     public static FunctionBarChartWindow theComparisonWindow;
-    
     public static boolean JNLP = false;
+    public static List scripts = new ArrayList();
+    public static String scriptFile;
     
-    public ParaProf() {
-
-        try {
-            //            //UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName()); }
-            //            UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
-            //
-            //            
-            //            UIManager.setLookAndFeel(
-            //                                     "com.sun.java.swing.plaf.gtk.GTKLookAndFeel");
-            //     
-            //            UIManager.setLookAndFeel(
-            //            "javax.swing.plaf.metal.MetalLookAndFeel");
-            //
-            //            UIManager.setLookAndFeel(
-            //            "com.sun.java.swing.plaf.motif.MotifLookAndFeel");
-
-        } catch (Exception e) {
+    public static boolean insideEclipse;
+    public static EclipseHandler eclipseHandler;
+    public static SourceManager directoryManager;
+    
+    // static initializer block
+    static {
+        ParaProf.runtime = Runtime.getRuntime();
+    }
 
+    public static HelpWindow getHelpWindow() {
+        if (helpWindow == null) {
+            helpWindow = new HelpWindow();
         }
+        return helpWindow;
+    }
+    
+    public static SourceManager getDirectoryManager() {
+    	if (directoryManager == null) {
+    		directoryManager = new SourceManager(ParaProf.preferences.getSourceLocations());
+    	}
+    	return directoryManager;
+    }
+    
+    public static void registerScript(ParaProfScript pps) {
+        scripts.add(pps);
+    }
 
+    public ParaProf() {
     }
 
     private static void usage() {
@@ -114,8 +133,7 @@ public class ParaProf implements ActionListener {
                 + "                                    (does not launch ParaProf GUI)\n"
                 + "  --dump                          Dump profile data to TAU profile format\n"
                 + "                                    (does not launch ParaProf GUI)\n" + "\n" + "Notes:\n"
-                + "  -m, --monitor                   Perform runtime monitoring of profile data\n"
-                + "\n"
+                + "  -m, --monitor                   Perform runtime monitoring of profile data\n" + "\n"
                 + "  For the TAU profiles type, you can specify either a specific set of profile\n"
                 + "files on the commandline, or you can specify a directory (by default the current\n"
                 + "directory).  The specified directory will be searched for profile.*.*.* files,\n"
@@ -123,19 +141,19 @@ public class ParaProf implements ActionListener {
     }
 
     public static void incrementNumWindows() {
-        //        System.out.println ("incrementing");
         numWindowsOpen++;
+        //System.out.println ("incrementing: now " + numWindowsOpen);
     }
 
     public static void decrementNumWindows() {
-        //        System.out.println ("decrementing");
         numWindowsOpen--;
+        //System.out.println ("decrementing: now " + numWindowsOpen);
         if (numWindowsOpen <= 0) {
             exitParaProf(0);
         }
     }
 
-    public void loadDefaultTrial() {
+    public static void loadDefaultTrial() {
 
         // Create a default application.
         ParaProfApplication app = ParaProf.applicationManager.addApplication();
@@ -145,116 +163,120 @@ public class ParaProf implements ActionListener {
         ParaProfExperiment experiment = app.addExperiment();
         experiment.setName("Default Exp");
 
-        ParaProf.helpWindow = new HelpWindow();
-        ParaProf.paraProfManagerWindow = new ParaProfManagerWindow();
+        ParaProf.paraProfManagerWindow.setVisible(true);
 
         try {
-            paraProfManagerWindow.addTrial(app, experiment, sourceFiles, fileType, fixNames, monitorProfiles);
+
+            if (fileType == 7) {
+                for (int i = 0; i < sourceFiles.length; i++) {
+                    File files[] = new File[1];
+                    files[0] = sourceFiles[i];
+                    paraProfManagerWindow.addTrial(app, experiment, files, fileType, fixNames, monitorProfiles);
+                }
+            } else {
+                paraProfManagerWindow.addTrial(app, experiment, sourceFiles, fileType, fixNames, monitorProfiles);
+            }
         } catch (java.security.AccessControlException ace) {
             // running as Java Web Start without permission
         }
     }
 
-    public void startSystem() {
+    public static void initialize() {
+
         try {
-            // Initialization of static objects takes place on a need basis.
-            // This helps prevent the creation of a graphical system unless it is absolutely
-            // necessary. Static initializations are marked with "Static Initialization" 
-            // to make them easy to find.
-
-            ParaProf.preferences = new Preferences();
-
-            //Establish the presence of a .ParaProf directory. This is located
-            // by default in the user's home directory.
-
-            try {
-                if (System.getProperty("jnlp.running") != null) {
-                    ParaProf.JNLP = true;
-                }
-            } catch (java.security.AccessControlException ace) {
+            if (System.getProperty("jnlp.running") != null) {
                 ParaProf.JNLP = true;
             }
+        } catch (java.security.AccessControlException ace) {
+            // if we get this security exception, we are definitely in JNLP
+            ParaProf.JNLP = true;
+        }
 
-            if (ParaProf.JNLP == false) {
-                ParaProf.paraProfHomeDirectory = new File(System.getProperty("user.home") + "/.ParaProf");
-                if (paraProfHomeDirectory.exists()) {
+        if (ParaProf.JNLP == false) {
 
-                    //Try and load a preference file ... ParaProfPreferences.dat
-                    try {
-                        FileInputStream savedPreferenceFIS = new FileInputStream(ParaProf.paraProfHomeDirectory.getPath()
-                                + "/ParaProf.conf");
+            // Establish the presence of a .ParaProf directory. This is located
+            // by default in the user's home directory.
+            ParaProf.paraProfHomeDirectory = new File(System.getProperty("user.home") + "/.ParaProf");
+            if (paraProfHomeDirectory.exists()) {
 
-                        //If here, means that no exception was thrown, and there is a preference file present.
-                        //Create ObjectInputStream and try to read it in.
-                        ObjectInputStream inSavedPreferencesOIS = new ObjectInputStream(savedPreferenceFIS);
-                        ParaProf.preferences = (Preferences) inSavedPreferencesOIS.readObject();
-                        ParaProf.preferences.setLoaded(true);
-                        colorChooser = new ColorChooser(ParaProf.preferences);
-                    } catch (Exception e) {
-                        if (e instanceof FileNotFoundException) {
-                            //System.out.println("No preference file present, using defaults!");
-                        } else {
-                            //Print some kind of error message, and quit the system.
-                            System.out.println("Error while trying to read the ParaProf preferences file, using defaults");
-                            //                        System.out.println("Please delete this file, or replace it with a valid one!");
-                            //                        System.out.println("Note: Deleting the file will cause ParaProf to restore the default preferences");
-                        }
-                    }
+                // try to load preferences
+                try {
+                    FileInputStream savedPreferenceFIS = new FileInputStream(ParaProf.paraProfHomeDirectory.getPath()
+                            + "/ParaProf.conf");
 
-                    ParaProf.colorMap.setMap(preferences.getAssignedColors());
-                    ParaProf.preferences.setDatabasePassword(null);
-
-                    //Try and find perfdmf.cfg.
-                    File perfDMFcfg = new File(ParaProf.paraProfHomeDirectory.getPath() + "/perfdmf.cfg");
-                    if (perfDMFcfg.exists()) {
-                        //System.out.println("Found db configuration file: "
-                        //        + ParaProf.paraProfHomeDirectory.getPath() + "/perfdmf.cfg");
-                        ParaProf.preferences.setDatabaseConfigurationFile(ParaProf.paraProfHomeDirectory.getPath()
-                                + "/perfdmf.cfg");
+                    ObjectInputStream inSavedPreferencesOIS = new ObjectInputStream(savedPreferenceFIS);
+                    ParaProf.preferences = (Preferences) inSavedPreferencesOIS.readObject();
+                    ParaProf.preferences.setLoaded(true);
+                    colorChooser = new ColorChooser(ParaProf.preferences);
+                } catch (Exception e) {
+                    if (e instanceof FileNotFoundException) {
+                        //System.out.println("No preference file present, using defaults!");
                     } else {
-                        System.out.println("Did not find db configuration file ... load manually");
+                        System.out.println("Error while trying to read the ParaProf preferences file, using defaults");
+                        //System.out.println("Please delete this file, or replace it with a valid one!");
+                        //System.out.println("Note: Deleting the file will cause ParaProf to restore the default preferences");
                     }
-
-                } else {
-                    System.out.println("Did not find ParaProf home directory ... creating ...");
-                    paraProfHomeDirectory.mkdir();
-                    System.out.println("Done creating ParaProf home directory!");
                 }
+
+                ParaProf.colorMap.setMap(preferences.getAssignedColors());
+                ParaProf.preferences.setDatabasePassword(null);
+
+                // try to load perfdmf.cfg.
+                File perfDMFcfg = new File(ParaProf.paraProfHomeDirectory.getPath() + "/perfdmf.cfg");
+                if (perfDMFcfg.exists()) {
+                    //System.out.println("Found db configuration file: "
+                    //        + ParaProf.paraProfHomeDirectory.getPath() + "/perfdmf.cfg");
+                    ParaProf.preferences.setDatabaseConfigurationFile(ParaProf.paraProfHomeDirectory.getPath() + "/perfdmf.cfg");
+                } else {
+                    System.out.println("Did not find db configuration file ... load manually");
+                }
+
             } else {
-                // Java Web Start
-                //URL url = ParaProf.class.getResource("/perfdmf.cfg");
-                //throw new ParaProfException("URL = " + url);
-                
-                URL url = ParaProf.class.getResource("/perfdmf.cfg");
-                
-                String path = URLDecoder.decode(url.getPath());
-                
-                ParaProf.preferences.setDatabaseConfigurationFile(path);
-
+                System.out.println("Did not find ParaProf home directory ... creating ...");
+                paraProfHomeDirectory.mkdir();
+                System.out.println("Done creating ParaProf home directory!");
             }
+        } else { // ParaProf.JNLP == true
+            // Java Web Start
+            //URL url = ParaProf.class.getResource("/perfdmf.cfg");
+            //throw new ParaProfException("URL = " + url);
 
-            if (colorChooser == null) {
-                ParaProf.colorChooser = new ColorChooser(null);
-            }
+            URL url = ParaProf.class.getResource("/perfdmf.cfg");
+            String path = URLDecoder.decode(url.getPath());
+            ParaProf.preferences.setDatabaseConfigurationFile(path);
+        }
 
-            ParaProf.preferencesWindow = new PreferencesWindow(preferences);
+        if (colorChooser == null) {
+            // we create one if ParaProf.conf wasn't properly read
+            ParaProf.colorChooser = new ColorChooser(null);
+        }
 
-            DataSource.setMeanIncludeNulls(!preferences.getComputeMeanWithoutNulls());
+        ParaProf.preferencesWindow = new PreferencesWindow(preferences);
 
-            //            javax.swing.SwingUtilities.invokeLater(new Runnable() {
-            //              public void run() {
+        DataSource.setMeanIncludeNulls(!preferences.getComputeMeanWithoutNulls());
+
+        // Set the default exception handler for AWT
+        // This avoids the mess of having to put a try/catch around every AWT entry point
+        try {
+            System.setProperty("sun.awt.exception.handler", XThrowableHandler.class.getName());
+        } catch (java.security.AccessControlException ace) {
+            // running as Java Web Start without permission
+        }
+
+
+        // Initialize, but do not show the manager window
+        ParaProf.paraProfManagerWindow = new ParaProfManagerWindow();
+    }
+
+    public static void loadScripts() {
+        ParaProf.scripts.clear();
+        ParaProf.scriptFile = System.getProperty("user.home") + "/.ParaProf/ParaProf.py";
+        if (new File(scriptFile).exists()) {
             try {
-                System.setProperty("sun.awt.exception.handler", XThrowableHandler.class.getName());
-            } catch (java.security.AccessControlException ace) {
-                // running as Java Web Start without permission
+                TauScripter.execfile(System.getProperty("user.home") + "/.ParaProf/ParaProf.py");
+            } catch (Exception e) {
+                new ParaProfErrorDialog("Exception while executing script: ", e);
             }
-            loadDefaultTrial();
-
-            //            }
-            //      });
-
-        } catch (Exception e) {
-            ParaProfUtils.handleException(e);
         }
     }
 
@@ -270,7 +292,7 @@ public class ParaProf implements ActionListener {
     public static String getInfoString() {
         long memUsage = (Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory()) / 1024;
 
-        return new String("ParaProf 3\n" + getVersionString() + "\nJVM Heap Size: " + memUsage + "kb\n");
+        return new String("ParaProf\n" + getVersionString() + "\nJVM Heap Size: " + memUsage + "kb\n" + "http://www.cs.uoregon.edu/research/tau\n");
     }
 
     public static String getVersionString() {
@@ -303,19 +325,15 @@ public class ParaProf implements ActionListener {
     // This method is reponsible for any cleanup required in ParaProf 
     // before an exit takes place.
     public static void exitParaProf(int exitValue) {
-        //try {
-        //   throw new Exception();
-        //} catch (Exception e) {
-        //   e.printStackTrace();
-        //}
-
         try {
             savePreferences(new File(ParaProf.paraProfHomeDirectory.getPath() + "/ParaProf.conf"));
         } catch (Exception e) {
             // we'll get an exception here if running under Java Web Start
         }
-
-        System.exit(exitValue);
+        if (!insideEclipse) {
+            // never call System.exit when invoked by the eclipse plugin, it will close the whole JVM, including the user's eclipse!
+            System.exit(exitValue);
+        }
     }
 
     public static boolean savePreferences(File file) {
@@ -323,9 +341,7 @@ public class ParaProf implements ActionListener {
         ParaProf.colorChooser.setSavedColors();
         ParaProf.preferences.setAssignedColors(ParaProf.colorMap.getMap());
         ParaProf.preferences.setManagerWindowPosition(ParaProf.paraProfManagerWindow.getLocation());
-
-        //        System.out.println ("saving manager position = " + preferences.getManagerWindowPosition());
-
+        ParaProf.preferences.setSourceLocations(getDirectoryManager().getCurrentElements());
         try {
             ObjectOutputStream prefsOut = new ObjectOutputStream(new FileOutputStream(file));
             prefsOut.writeObject(ParaProf.preferences);
@@ -340,8 +356,7 @@ public class ParaProf implements ActionListener {
 
     // Main entry point
     static public void main(String[] args) {
-
-        final ParaProf paraProf = new ParaProf();
+        ParaProf.args = args;
 
         // Set the tooltip delay to 20 seconds
         ToolTipManager.sharedInstance().setDismissDelay(20000);
@@ -378,7 +393,7 @@ public class ParaProf implements ActionListener {
         if (monitor != null) {
             monitorProfiles = monitor.booleanValue();
         }
-        
+
         if (pack != null && unpack != null) {
             System.err.println("--pack and --dump are mutually exclusive");
             System.exit(-1);
@@ -419,25 +434,28 @@ public class ParaProf implements ActionListener {
                 ParaProf.fileType = 8;
             } else if (fileTypeString.equals("hpc")) {
                 ParaProf.fileType = 9;
+            } else if (fileTypeString.equals("snap")) {
+                ParaProf.fileType = 10;
             } else {
                 System.err.println("Please enter a valid file type.");
                 ParaProf.usage();
                 System.exit(-1);
             }
         } else {
-            if (sourceFilenames.length == 1) {
+            if (sourceFilenames.length >= 1) {
                 String filename = sourceFiles[0].getName();
-                if (filename.endsWith(".ppk")) {
+                if (filename.toLowerCase().endsWith(".ppk")) {
                     ParaProf.fileType = 7;
                 }
-                if (filename.endsWith(".cube")) {
+                if (filename.toLowerCase().endsWith(".cube")) {
                     ParaProf.fileType = 8;
                 }
+                if (filename.toLowerCase().endsWith(".mpip")) {
+                    ParaProf.fileType = 3;
+                }
             }
         }
 
-        ParaProf.runtime = Runtime.getRuntime();
-
         if (pack != null) {
             try {
 
@@ -445,7 +463,7 @@ public class ParaProf implements ActionListener {
                 System.out.println("Loading data...");
                 dataSource.load();
                 System.out.println("Packing data...");
-                ParaProfUtils.writePacked(dataSource, new File(pack));
+                DataSourceExport.writePacked(dataSource, new File(pack));
 
             } catch (Exception e) {
                 e.printStackTrace();
@@ -467,7 +485,7 @@ public class ParaProf implements ActionListener {
                 System.out.println("Loading data...");
                 dataSource.load();
                 System.out.println("Creating TAU Profile data...");
-                ParaProfUtils.writeProfiles(dataSource, new File("."));
+                DataSourceExport.writeProfiles(dataSource, new File("."));
 
             } catch (Exception e) {
                 e.printStackTrace();
@@ -477,8 +495,14 @@ public class ParaProf implements ActionListener {
 
         javax.swing.SwingUtilities.invokeLater(new Runnable() {
             public void run() {
-                paraProf.startSystem();
+                try {
+                    ParaProf.initialize();
+                    ParaProf.loadScripts();
+                    ParaProf.loadDefaultTrial();
+                } catch (Exception e) {
+                    ParaProfUtils.handleException(e);
+                }
             }
         });
     }
-}
\ No newline at end of file
+}
