@@ -30,12 +30,15 @@ public class Preferences implements Serializable {
     private Point managerWindowPosition;
     private Map assignedColors;
 
-    private boolean showValuesAsPercent = true;
-    private boolean showPathTitleInReverse = true;
-    private int units = 0;
+    private boolean showValuesAsPercent = false;
+    private boolean showPathTitleInReverse = false;
+    private int units = 2;
     private boolean reversedCallPaths = false;
     private boolean computeMeanWithoutNulls = false;
-
+    private boolean generateIntermediateCallPathData = false;
+    
+    private ArrayList sourceLocations;
+    
     static final long serialVersionUID = 183442743456314793L;
 
     public void setLoaded(boolean b) {
@@ -196,4 +199,20 @@ public class Preferences implements Serializable {
         this.computeMeanWithoutNulls = computeMeanWithoutNulls;
     }
 
+    public boolean getGenerateIntermediateCallPathData() {
+        return generateIntermediateCallPathData;
+    }
+
+    public void setGenerateIntermediateCallPathData(boolean generateIntermediateCallPathData) {
+        this.generateIntermediateCallPathData = generateIntermediateCallPathData;
+    }
+
+	public ArrayList getSourceLocations() {
+		return sourceLocations;
+	}
+
+	public void setSourceLocations(ArrayList sourceLocations) {
+		this.sourceLocations = sourceLocations;
+	}
+
 }
\ No newline at end of file
