@@ -2,10 +2,13 @@ package edu.uoregon.tau.paraprof;
 
 import java.awt.*;
 import java.awt.event.*;
-import java.util.Iterator;
 import java.util.List;
+import java.util.Vector;
 
 import javax.swing.*;
+import javax.swing.plaf.basic.BasicComboPopup;
+import javax.swing.plaf.basic.ComboPopup;
+import javax.swing.plaf.metal.MetalComboBoxUI;
 
 import edu.uoregon.tau.paraprof.enums.ValueType;
 import edu.uoregon.tau.paraprof.enums.VisType;
@@ -18,9 +21,9 @@ import edu.uoregon.tau.vis.VisRenderer;
  *    
  * TODO : ...
  *
- * <P>CVS $Id: ThreeDeeControlPanel.java,v 1.1 2005/09/26 21:12:11 amorris Exp $</P>
+ * <P>CVS $Id: ThreeDeeControlPanel.java,v 1.6 2007/01/04 01:55:32 amorris Exp $</P>
  * @author	Alan Morris
- * @version	$Revision: 1.1 $
+ * @version	$Revision: 1.6 $
  */
 public class ThreeDeeControlPanel extends JPanel implements ActionListener {
 
@@ -29,8 +32,8 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
     private ThreeDeeWindow window;
     private ParaProfTrial ppTrial;
 
-    private JComboBox heightValueBox, heightMetricBox;
-    private JComboBox colorValueBox, colorMetricBox;
+    private SteppedComboBox heightValueBox, heightMetricBox;
+    private SteppedComboBox colorValueBox, colorMetricBox;
 
     private JPanel subPanel;
     private VisRenderer visRenderer;
@@ -41,8 +44,94 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
     private int selectedTab;
     private JTabbedPane tabbedPane; // keep a handle to remember the selected tab
 
-    public ThreeDeeControlPanel(ThreeDeeWindow window, ThreeDeeSettings settings, ParaProfTrial ppTrial,
-            VisRenderer visRenderer) {
+    public class SliderComboBox extends JComboBox {
+        public SliderComboBox() {
+            super();
+            setUI(new SliderComboUI());
+        }
+
+        public SliderComboBox(Object obj[]) {
+            super(obj);
+            setUI(new SliderComboUI());
+        }
+
+        public class SliderComboUI extends MetalComboBoxUI {
+            protected ComboPopup createPopup() {
+                BasicComboPopup popup = new BasicComboPopup(comboBox) {
+                    protected JScrollPane createScroller() {
+                        return new JScrollPane(list, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
+                                ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
+                    }
+                };
+                return popup;
+            }
+        }
+    }
+
+    // From: http://forum.java.sun.com/thread.jspa?forumID=257&threadID=300107
+    class SteppedComboBoxUI extends MetalComboBoxUI {
+        protected ComboPopup createPopup() {
+            BasicComboPopup popup = new BasicComboPopup(comboBox) {
+
+                public void show() {
+                    Dimension popupSize = ((SteppedComboBox) comboBox).getPopupSize();
+                    popupSize.setSize(popupSize.width, getPopupHeightForRowCount(comboBox.getMaximumRowCount()));
+                    Rectangle popupBounds = computePopupBounds(0, comboBox.getBounds().height, popupSize.width, popupSize.height);
+                    scroller.setMaximumSize(popupBounds.getSize());
+                    scroller.setPreferredSize(popupBounds.getSize());
+                    scroller.setMinimumSize(popupBounds.getSize());
+                    list.invalidate();
+                    int selectedIndex = comboBox.getSelectedIndex();
+                    if (selectedIndex == -1) {
+                        list.clearSelection();
+                    } else {
+                        list.setSelectedIndex(selectedIndex);
+                    }
+                    list.ensureIndexIsVisible(list.getSelectedIndex());
+                    setLightWeightPopupEnabled(comboBox.isLightWeightPopupEnabled());
+
+                    show(comboBox, popupBounds.x, popupBounds.y);
+                }
+            };
+            popup.getAccessibleContext().setAccessibleParent(comboBox);
+            return popup;
+        }
+    }
+
+    public class SteppedComboBox extends JComboBox {
+        protected int popupWidth;
+
+        public SteppedComboBox(ComboBoxModel aModel) {
+            super(aModel);
+            setUI(new SteppedComboBoxUI());
+            popupWidth = 0;
+        }
+
+        public SteppedComboBox(final Object[] items) {
+            super(items);
+            setUI(new SteppedComboBoxUI());
+            popupWidth = 0;
+        }
+
+        public SteppedComboBox(Vector items) {
+            super(items);
+            setUI(new SteppedComboBoxUI());
+            popupWidth = 0;
+        }
+
+        public void setPopupWidth(int width) {
+            popupWidth = width;
+        }
+
+        public Dimension getPopupSize() {
+            Dimension size = getSize();
+            if (popupWidth < 1)
+                popupWidth = size.width;
+            return new Dimension(popupWidth, size.height);
+        }
+    }
+
+    public ThreeDeeControlPanel(ThreeDeeWindow window, ThreeDeeSettings settings, ParaProfTrial ppTrial, VisRenderer visRenderer) {
         this.settings = settings;
         this.window = window;
         this.ppTrial = ppTrial;
@@ -130,8 +219,17 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
             items[i] = ppTrial.getMetric(i).getName();
         }
 
-        final JComboBox valueBox = new JComboBox(ValueType.VALUES);
-        final JComboBox metricBox = new JComboBox(items);
+        Dimension d;
+        final SteppedComboBox valueBox = new SteppedComboBox(ValueType.VALUES);
+        d = valueBox.getPreferredSize();
+        valueBox.setMinimumSize(new Dimension(50, valueBox.getMinimumSize().height));
+        valueBox.setPopupWidth(d.width);
+
+        final SteppedComboBox metricBox = new SteppedComboBox(items);
+        d = metricBox.getPreferredSize();
+        metricBox.setMinimumSize(new Dimension(50, metricBox.getMinimumSize().height));
+        metricBox.setPopupWidth(d.width);
+
         valueBox.setSelectedItem(settings.getScatterValueTypes()[index]);
         metricBox.setSelectedIndex(settings.getScatterMetricIDs()[index]);
 
@@ -156,7 +254,7 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
                 try {
 
                     FunctionSelectorDialog fSelector = new FunctionSelectorDialog(window, true,
-                            ppTrial.getDisplayedFunctions().iterator(), settings.getScatterFunctions()[index], true);
+                            ppTrial.getDisplayedFunctions().iterator(), settings.getScatterFunctions()[index], true, false);
 
                     if (fSelector.choose()) {
                         Function selectedFunction = (Function) fSelector.getSelectedObject();
@@ -164,7 +262,7 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
 
                         String fname = "   <none>";
                         if (settings.getScatterFunctions()[index] != null) {
-                            fname = ParaProfUtils.getFunctionName(settings.getScatterFunctions()[index]);
+                            fname = ParaProfUtils.getDisplayName(settings.getScatterFunctions()[index]);
                         }
                         functionField.setText(fname);
                         window.redraw();
@@ -199,7 +297,6 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
         gbc.weightx = 0.5;
         gbc.weighty = 0.5;
         addCompItem(panel, valueBox, gbc, 1, 1, 1, 1);
-
         addCompItem(panel, metricBox, gbc, 2, 1, 1, 1);
 
         return panel;
@@ -224,12 +321,16 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
         gbc.weightx = 0.1;
         gbc.weighty = 0.1;
 
+        gbc.weightx = 0;
         addCompItem(panel, new JLabel("Width"), gbc, 0, 0, 1, 1);
         addCompItem(panel, createScatterSelectionPanel("Width", 0), gbc, 1, 0, 1, 1);
+        gbc.weightx = 0;
         addCompItem(panel, new JLabel("Depth"), gbc, 0, 1, 1, 1);
         addCompItem(panel, createScatterSelectionPanel("Depth", 1), gbc, 1, 1, 1, 1);
+        gbc.weightx = 0;
         addCompItem(panel, new JLabel("Height"), gbc, 0, 2, 1, 1);
         addCompItem(panel, createScatterSelectionPanel("Height", 2), gbc, 1, 2, 1, 1);
+        gbc.weightx = 0;
         addCompItem(panel, new JLabel("Color"), gbc, 0, 3, 1, 1);
         addCompItem(panel, createScatterSelectionPanel("Color", 3), gbc, 1, 3, 1, 1);
 
@@ -253,7 +354,6 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
 
     }
 
-    
     private JPanel createSelectorPanel(int min, int max, final List names, final int index) {
 
         JPanel panel = new JPanel();
@@ -262,9 +362,8 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
         GridBagConstraints gbc = new GridBagConstraints();
         gbc.insets = new Insets(5, 5, 5, 5);
 
-        final JScrollBar scrollBar = new JScrollBar(JScrollBar.HORIZONTAL, settings.getSelections()[index], 1,
-                min, max);
-        scrollBar.setBlockIncrement((max-min) / 10);
+        final JScrollBar scrollBar = new JScrollBar(JScrollBar.HORIZONTAL, settings.getSelections()[index], 1, min, max);
+        scrollBar.setBlockIncrement((max - min) / 10);
 
         final JTextField textField = new JTextField("<none>");
 
@@ -279,7 +378,6 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
         textField.setEditable(false);
         textField.setCaretPosition(0);
 
-        
         scrollBar.addAdjustmentListener(new AdjustmentListener() {
             public void adjustmentValueChanged(AdjustmentEvent e) {
                 int selection = scrollBar.getValue();
@@ -309,7 +407,6 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
         return panel;
     }
 
-    
     private JPanel createFullDataPanel() {
 
         JPanel regularPanel = new JPanel();
@@ -334,7 +431,6 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
                     settings.setRegularAim(visRenderer.getAim());
                     settings.setRegularEye(visRenderer.getEye());
 
-                    
                     heightValueField.setText(window.getSelectedHeightValue());
                     colorValueField.setText(window.getSelectedColorValue());
 
@@ -347,31 +443,43 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
 
         };
 
+        Dimension d;
+
+        heightValueBox = new SteppedComboBox(ValueType.VALUES);
+        d = heightValueBox.getPreferredSize();
+        heightValueBox.setMinimumSize(new Dimension(50, heightValueBox.getMinimumSize().height));
+        heightValueBox.setPopupWidth(d.width);
+        heightValueBox.setSelectedItem(settings.getHeightValue());
+        heightValueBox.addActionListener(metricChanger);
+
+        colorValueBox = new SteppedComboBox(ValueType.VALUES);
+        d = colorValueBox.getPreferredSize();
+        colorValueBox.setMinimumSize(new Dimension(50, colorValueBox.getMinimumSize().height));
+        colorValueBox.setPopupWidth(d.width);
+        colorValueBox.setSelectedItem(settings.getColorValue());
+        colorValueBox.addActionListener(metricChanger);
+
         String[] items = new String[ppTrial.getNumberOfMetrics()];
         for (int i = 0; i < ppTrial.getNumberOfMetrics(); i++) {
             items[i] = ppTrial.getMetric(i).getName();
         }
 
-        heightValueBox = new JComboBox(ValueType.VALUES);
-        heightValueBox.setSelectedItem(settings.getHeightValue());
-        heightValueBox.addActionListener(metricChanger);
-
-        heightMetricBox = new JComboBox(items);
+        heightMetricBox = new SteppedComboBox(items);
+        d = heightMetricBox.getPreferredSize();
+        heightMetricBox.setMinimumSize(new Dimension(50, heightMetricBox.getMinimumSize().height));
+        heightMetricBox.setPopupWidth(d.width);
         heightMetricBox.setSelectedIndex(settings.getHeightMetricID());
         heightMetricBox.addActionListener(metricChanger);
 
-        colorValueBox = new JComboBox(ValueType.VALUES);
-        colorValueBox.setSelectedItem(settings.getColorValue());
-        colorValueBox.addActionListener(metricChanger);
-
         items = new String[ppTrial.getNumberOfMetrics()];
         for (int i = 0; i < ppTrial.getNumberOfMetrics(); i++) {
             items[i] = ppTrial.getMetric(i).getName();
         }
-
-        colorMetricBox = new JComboBox(items);
+        colorMetricBox = new SteppedComboBox(items);
+        d = colorMetricBox.getPreferredSize();
+        colorMetricBox.setMinimumSize(new Dimension(50, colorMetricBox.getMinimumSize().height));
+        colorMetricBox.setPopupWidth(d.width);
         colorMetricBox.setSelectedIndex(settings.getColorMetricID());
-        colorMetricBox.addActionListener(metricChanger);
 
         tabbedPane = new JTabbedPane();
         Plot plot = window.getPlot();
@@ -383,10 +491,8 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
         tabbedPane.setMinimumSize(new Dimension(290, 160));
         tabbedPane.setSelectedIndex(selectedTab);
 
-        JPanel functionSelectorPanel = createSelectorPanel(-1, window.getFunctionNames().size(),
-                window.getFunctionNames(), 0);
-        JPanel nodeSelectorPanel = createSelectorPanel(0, ppTrial.getDataSource().getNumThreads(),
-                window.getThreadNames(), 1);
+        JPanel functionSelectorPanel = createSelectorPanel(-1, window.getFunctionNames().size(), window.getFunctionNames(), 0);
+        JPanel nodeSelectorPanel = createSelectorPanel(0, ppTrial.getDataSource().getNumThreads(), window.getThreadNames(), 1);
 
         gbc.fill = GridBagConstraints.HORIZONTAL;
         gbc.anchor = GridBagConstraints.WEST;
@@ -394,9 +500,10 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
         gbc.weighty = 0.0;
 
         addCompItem(regularPanel, new JLabel("Height Metric"), gbc, 0, 0, 2, 1);
+        addCompItem(regularPanel, new JLabel("Color Metric"), gbc, 0, 2, 2, 1);
+        gbc.weightx = 0.5;
         addCompItem(regularPanel, heightValueBox, gbc, 0, 1, 1, 1);
         addCompItem(regularPanel, heightMetricBox, gbc, 1, 1, 1, 1);
-        addCompItem(regularPanel, new JLabel("Color Metric"), gbc, 0, 2, 2, 1);
         addCompItem(regularPanel, colorValueBox, gbc, 0, 3, 1, 1);
         addCompItem(regularPanel, colorMetricBox, gbc, 1, 3, 1, 1);
 
@@ -406,7 +513,6 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
 
         heightValueField.setEditable(false);
         colorValueField.setEditable(false);
-        
 
         gbc.fill = GridBagConstraints.NONE;
         gbc.weightx = 0.0;
@@ -434,7 +540,6 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
         return regularPanel;
     }
 
-   
     public void actionPerformed(ActionEvent evt) {
         try {
             Object EventSrc = evt.getSource();
@@ -446,8 +551,7 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
                 String arg = evt.getActionCommand();
                 Plot plot = window.getPlot();
 
-                if (settings.getVisType() == VisType.BAR_PLOT
-                        || settings.getVisType() == VisType.TRIANGLE_MESH_PLOT) {
+                if (settings.getVisType() == VisType.BAR_PLOT || settings.getVisType() == VisType.TRIANGLE_MESH_PLOT) {
                     settings.setSize((int) plot.getWidth(), (int) plot.getDepth(), (int) plot.getHeight());
                     settings.setRegularAim(visRenderer.getAim());
                     settings.setRegularEye(visRenderer.getEye());
