@@ -15,15 +15,16 @@ import edu.uoregon.tau.paraprof.enums.SortType;
 import edu.uoregon.tau.paraprof.enums.ValueType;
 import edu.uoregon.tau.paraprof.enums.VisType;
 import edu.uoregon.tau.paraprof.interfaces.ParaProfWindow;
+import edu.uoregon.tau.paraprof.interfaces.SortListener;
 import edu.uoregon.tau.paraprof.interfaces.UnitListener;
 import edu.uoregon.tau.perfdmf.*;
 import edu.uoregon.tau.perfdmf.Thread;
 import edu.uoregon.tau.vis.*;
 
 public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListener, Observer, Printable, ParaProfWindow,
-        UnitListener {
+        UnitListener, SortListener {
 
-    private final int defaultToScatter = 4000;
+    private final int defaultToScatter = 15000;
 
     private VisCanvas visCanvas;
     private VisRenderer visRenderer = new VisRenderer();
@@ -75,8 +76,9 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
         dataSorter = new DataSorter(ppTrial);
         dataSorter.setSortType(SortType.NAME);
 
-        this.setTitle("ParaProf Visualizer: "
+        this.setTitle("TAU: ParaProf: 3D Visualizer: "
                 + ppTrial.getTrialIdentifier(ParaProf.preferences.getShowPathTitleInReverse()));
+        ParaProfUtils.setFrameIcon(this);
 
         //Add some window listener code
         addWindowListener(new java.awt.event.WindowAdapter() {
@@ -93,14 +95,12 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
         String os = System.getProperty("os.name").toLowerCase();
         String cpu = System.getProperty("os.arch").toLowerCase();
         if (os.startsWith("linux") && cpu.equals("ia64")) {
-            this.show();
+            this.setVisible(true);
         }
 
-
         DataSource dataSource = ppTrial.getDataSource();
         int numThreads = dataSource.getNumThreads();
 
-        
         // initialize the scatterplot functions to the 4 most varying functions
         // we just get the first four stddev functions
         DataSorter dataSorter = new DataSorter(ppTrial);
@@ -109,12 +109,11 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
         int count = 0;
         for (Iterator it = stdDevList.iterator(); it.hasNext() && count < 4;) {
             PPFunctionProfile fp = (PPFunctionProfile) it.next();
-            settings.setScatterFunction(fp.getFunction(),count);
+            settings.setScatterFunction(fp.getFunction(), count);
             count++;
         }
-        
-        // if the number of threads is above this threshold, we default to the 
-        // scatterplot
+
+        // if the number of threads is above this threshold, we default to the scatterplot
         if (numThreads > defaultToScatter) {
             settings.setVisType(VisType.SCATTER_PLOT);
         }
@@ -158,8 +157,8 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
         jSplitPane.addKeyListener(this);
         this.getContentPane().add(jSplitPane);
 
-        setSize(ParaProfUtils.checkSize(new Dimension(1000,700)));
-        this.setLocation(WindowPlacer.getNewLocation(this,invoker));
+        setSize(ParaProfUtils.checkSize(new Dimension(1000, 750)));
+        this.setLocation(WindowPlacer.getNewLocation(this, invoker));
 
         //        //Grab the screen size.
         //        Toolkit tk = Toolkit.getDefaultToolkit();
@@ -171,7 +170,7 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
         if (os.startsWith("linux") && cpu.equals("ia64")) {
             this.validate();
         }
-        this.show();
+        this.setVisible(true);
 
         if (System.getProperty("vis.fps") != null) {
             fpsTimer = new javax.swing.Timer(1000, this);
@@ -204,15 +203,14 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
         }
 
         for (Iterator it = ppTrial.getDataSource().getAllThreads().iterator(); it.hasNext();) {
-            edu.uoregon.tau.perfdmf.Thread thread = (edu.uoregon.tau.perfdmf.Thread) it.next();
+            Thread thread = (Thread) it.next();
 
             for (int f = 0; f < scatterFunctions.length; f++) {
                 if (scatterFunctions[f] != null) {
                     FunctionProfile functionProfile = thread.getFunctionProfile(scatterFunctions[f]);
 
                     if (functionProfile != null) {
-                        values[threadIndex][f] = (float) scatterValueTypes[f].getValue(functionProfile,
-                                scatterMetricIDs[f]);
+                        values[threadIndex][f] = (float) scatterValueTypes[f].getValue(functionProfile, scatterMetricIDs[f]);
                         maxScatterValues[f] = Math.max(maxScatterValues[f], values[threadIndex][f]);
                         minScatterValues[f] = Math.min(minScatterValues[f], values[threadIndex][f]);
                     }
@@ -283,7 +281,7 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
             threads = new ArrayList();
 
             for (Iterator it = ppTrial.getDataSource().getAllThreads().iterator(); it.hasNext();) {
-                edu.uoregon.tau.perfdmf.Thread thread = (edu.uoregon.tau.perfdmf.Thread) it.next();
+                Thread thread = (Thread) it.next();
                 threadNames.add(thread.getNodeID() + ":" + thread.getContextID() + ":" + thread.getThreadID());
                 threads.add(thread);
             }
@@ -305,12 +303,12 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
             //            }
 
             if (addFunctionNames) {
-                functionNames.add(ParaProfUtils.getFunctionName(function));
+                functionNames.add(ParaProfUtils.getDisplayName(function));
                 functions.add(function);
             }
             int threadIndex = 0;
             for (Iterator it = ppTrial.getDataSource().getAllThreads().iterator(); it.hasNext();) {
-                edu.uoregon.tau.perfdmf.Thread thread = (edu.uoregon.tau.perfdmf.Thread) it.next();
+                Thread thread = (Thread) it.next();
                 FunctionProfile functionProfile = thread.getFunctionProfile(function);
 
                 if (functionProfile != null) {
@@ -373,8 +371,8 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
             axes.setOnEdge(false);
             if (triangleMeshPlot == null) {
                 triangleMeshPlot = new TriangleMeshPlot();
-                triangleMeshPlot.initialize(axes, settings.getPlotWidth(), settings.getPlotDepth(),
-                        settings.getPlotHeight(), heightValues, colorValues, colorScale);
+                triangleMeshPlot.initialize(axes, settings.getPlotWidth(), settings.getPlotDepth(), settings.getPlotHeight(),
+                        heightValues, colorValues, colorScale);
                 plot = triangleMeshPlot;
             } else {
                 triangleMeshPlot.setValues(settings.getPlotWidth(), settings.getPlotDepth(), settings.getPlotHeight(),
@@ -391,8 +389,8 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
                         heightValues, colorValues, colorScale);
                 plot = barPlot;
             } else {
-                barPlot.setValues(settings.getPlotWidth(), settings.getPlotDepth(), settings.getPlotHeight(),
-                        heightValues, colorValues);
+                barPlot.setValues(settings.getPlotWidth(), settings.getPlotDepth(), settings.getPlotHeight(), heightValues,
+                        colorValues);
                 plot = barPlot;
             }
         }
@@ -417,8 +415,7 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
 
             if (newSettings.getVisType() == VisType.SCATTER_PLOT) {
                 visRenderer.setAim(settings.getScatterAim());
-            } else if (newSettings.getVisType() == VisType.TRIANGLE_MESH_PLOT
-                    || newSettings.getVisType() == VisType.BAR_PLOT) {
+            } else if (newSettings.getVisType() == VisType.TRIANGLE_MESH_PLOT || newSettings.getVisType() == VisType.BAR_PLOT) {
                 visRenderer.setAim(settings.getRegularAim());
             }
 
@@ -430,8 +427,7 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
                 generate3dModel(false, newSettings);
                 visRenderer.addShape(plot);
                 visRenderer.addShape(colorScale);
-            } else if (newSettings.getVisType() == VisType.TRIANGLE_MESH_PLOT
-                    || newSettings.getVisType() == VisType.BAR_PLOT) {
+            } else if (newSettings.getVisType() == VisType.TRIANGLE_MESH_PLOT || newSettings.getVisType() == VisType.BAR_PLOT) {
 
                 settings.setSize((int) plot.getWidth(), (int) plot.getDepth(), (int) plot.getHeight());
 
@@ -477,8 +473,6 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
     private void setupMenus() {
 
         JMenuBar mainMenu = new JMenuBar();
-        JMenu subMenu = null;
-        JMenuItem menuItem = null;
 
         optionsMenu = new JMenu("Options");
         optionsMenu.getPopupMenu().setLightWeightPopupEnabled(false);
@@ -487,6 +481,10 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
         unitsSubMenu.getPopupMenu().setLightWeightPopupEnabled(false);
         optionsMenu.add(unitsSubMenu);
 
+        JMenu sort = ParaProfUtils.createMetricSelectionMenu(ppTrial, "Sort by...", true, false, dataSorter, this, false);
+        sort.getPopupMenu().setLightWeightPopupEnabled(false);
+        optionsMenu.add(sort);
+
         // now add all the menus to the main menu
 
         JMenu fileMenu = ParaProfUtils.createFileMenu(this, this, this);
@@ -512,7 +510,6 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
         return units;
     }
 
-
     public void update(Observable o, Object arg) {
         String tmpString = (String) arg;
 
@@ -587,17 +584,19 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
 
     public void help(boolean display) {
         //Show the ParaProf help window.
-        ParaProf.helpWindow.clearText();
+        ParaProf.getHelpWindow().clearText();
         if (display)
-            ParaProf.helpWindow.show();
-        ParaProf.helpWindow.writeText("This is the 3D Window");
-        ParaProf.helpWindow.writeText("");
-        ParaProf.helpWindow.writeText("This window displays profile data in three dimensions through the Triangle Mesh Plot, the Bar Plot, and the ScatterPlot");
-        ParaProf.helpWindow.writeText("");
-        ParaProf.helpWindow.writeText("Change between the plots by selecting the desired type from the radio buttons in the upper right.");
-        ParaProf.helpWindow.writeText("");
-        ParaProf.helpWindow.writeText("Experiment with the controls at the right.");
-        ParaProf.helpWindow.writeText("");
+            ParaProf.getHelpWindow().show();
+        ParaProf.getHelpWindow().writeText("This is the 3D Window");
+        ParaProf.getHelpWindow().writeText("");
+        ParaProf.getHelpWindow().writeText(
+                "This window displays profile data in three dimensions through the Triangle Mesh Plot, the Bar Plot, and the ScatterPlot");
+        ParaProf.getHelpWindow().writeText("");
+        ParaProf.getHelpWindow().writeText(
+                "Change between the plots by selecting the desired type from the radio buttons in the upper right.");
+        ParaProf.getHelpWindow().writeText("");
+        ParaProf.getHelpWindow().writeText("Experiment with the controls at the right.");
+        ParaProf.getHelpWindow().writeText("");
     }
 
     private long lastCall = 0;
@@ -628,7 +627,7 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
 
             return Printable.PAGE_EXISTS;
         } catch (Exception e) {
-            new ParaProfErrorDialog(e);
+            ParaProfUtils.handleException(e);
             return NO_SUCH_PAGE;
         }
     }
@@ -727,7 +726,6 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
         if (!ppMetric.isTimeMetric() || !ValueType.isTimeUnits(settings.getHeightValue())) {
             units = 0;
         }
-        
 
         return UtilFncs.getOutputString(units, settings.getHeightValue().getValue(fp, settings.getHeightMetricID()), 6).trim()
                 + getUnitsString(units, settings.getHeightValue(), ppMetric);
@@ -782,12 +780,11 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
                 if (scatterFunctions[f] != null) {
                     // e.g. "MPI_Recv()\n(Exclusive, Time)"
                     if (scatterValueTypes[f] == ValueType.NUMCALLS || scatterValueTypes[f] == ValueType.NUMSUBR) {
-                        axisNames.add(ParaProfUtils.getFunctionName(scatterFunctions[f]) + "\n("
-                                + scatterValueTypes[f].toString() + ")");
-                    } else {
-                        axisNames.add(ParaProfUtils.getFunctionName(scatterFunctions[f]) + "\n("
-                                + scatterValueTypes[f].toString() + ", " + ppTrial.getMetricName(scatterMetricIDs[f])
+                        axisNames.add(ParaProfUtils.getDisplayName(scatterFunctions[f]) + "\n(" + scatterValueTypes[f].toString()
                                 + ")");
+                    } else {
+                        axisNames.add(ParaProfUtils.getDisplayName(scatterFunctions[f]) + "\n(" + scatterValueTypes[f].toString()
+                                + ", " + ppTrial.getMetricName(scatterMetricIDs[f]) + ")");
                     }
                 } else {
                     axisNames.add("none");
@@ -820,8 +817,8 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
             ParaProfMetric ppMetric = ppTrial.getMetric(scatterMetricIDs[3]);
             int units = scatterValueTypes[3].getUnits(this.units, ppMetric);
 
-            colorScale.setStrings(UtilFncs.getOutputString(units, minScatterValues[3], 6).trim(),
-                    UtilFncs.getOutputString(units, maxScatterValues[3], 6).trim(), (String) axisNames.get(3));
+            colorScale.setStrings(UtilFncs.getOutputString(units, minScatterValues[3], 6).trim(), UtilFncs.getOutputString(units,
+                    maxScatterValues[3], 6).trim(), (String) axisNames.get(3));
 
             scatterPlotAxes.setStrings((String) axisNames.get(0), (String) axisNames.get(1), (String) axisNames.get(2),
                     axisStrings[0], axisStrings[1], axisStrings[2]);
@@ -892,11 +889,14 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
     }
 
     public void setUnits(int units) {
-        // TODO Auto-generated method stub
         this.units = units;
         setAxisStrings();
         controlPanel.dataChanged();
         visRenderer.redraw();
     }
 
+    public void resort() {
+        sortLocalData();
+    }
+
 }
