@@ -15,9 +15,9 @@ import edu.uoregon.tau.perfdmf.UtilFncs;
 /**
  * Compares threads from (potentially) any trial
  * 
- * <P>CVS $Id: ComparisonBarChartModel.java,v 1.2 2005/11/10 01:29:07 amorris Exp $</P>
+ * <P>CVS $Id: ComparisonBarChartModel.java,v 1.6 2006/12/01 00:36:52 amorris Exp $</P>
  * @author  Alan Morris
- * @version $Revision: 1.2 $
+ * @version $Revision: 1.6 $
  */
 public class ComparisonBarChartModel extends AbstractBarChartModel {
 
@@ -52,7 +52,7 @@ public class ComparisonBarChartModel extends AbstractBarChartModel {
             }
             super.set(index, element);
         }
-        
+
         public Object get(int index) {
             if (index >= size()) {
                 return null;
@@ -60,7 +60,7 @@ public class ComparisonBarChartModel extends AbstractBarChartModel {
                 return super.get(index);
             }
         }
-        
+
     }
 
     public ComparisonBarChartModel(FunctionBarChartWindow window, ParaProfTrial ppTrial, Thread thread, DataSorter dataSorter) {
@@ -69,7 +69,6 @@ public class ComparisonBarChartModel extends AbstractBarChartModel {
         addThread(ppTrial, thread);
     }
 
-
     public LegendModel getLegendModel() {
         if (legendModel == null) {
             legendModel = new LegendModel() {
@@ -87,7 +86,7 @@ public class ComparisonBarChartModel extends AbstractBarChartModel {
                 }
 
                 public Color getColor(int index) {
-                    return ComparisonBarChartModel.this.getValueColor(0,index);
+                    return ComparisonBarChartModel.this.getValueColor(0, index);
                 }
             };
         }
@@ -99,7 +98,6 @@ public class ComparisonBarChartModel extends AbstractBarChartModel {
         threads.add(thread);
     }
 
-
     public void reloadData() {
         rows.clear();
         rowMap.clear();
@@ -109,13 +107,12 @@ public class ComparisonBarChartModel extends AbstractBarChartModel {
 
         dataSorter.setPpTrial(selectedTrial);
         List list = dataSorter.getFunctionProfiles(selectedThread);
-
         for (Iterator it = list.iterator(); it.hasNext();) {
             PPFunctionProfile ppFunctionProfile = (PPFunctionProfile) it.next();
-            RowBlob blob = new RowBlob(ppFunctionProfile.getFunctionName());
+            RowBlob blob = new RowBlob(ppFunctionProfile.getDisplayName());
             rows.add(blob);
             blob.add(ppFunctionProfile);
-            rowMap.put(ppFunctionProfile.getFunctionName(), blob);
+            rowMap.put(ppFunctionProfile.getDisplayName(), blob);
         }
 
         // add all the others
@@ -123,8 +120,10 @@ public class ComparisonBarChartModel extends AbstractBarChartModel {
             ParaProfTrial ppTrial = (ParaProfTrial) ppTrials.get(i);
             Thread thread = (Thread) threads.get(i);
 
-            dataSorter.setPpTrial(ppTrial);
-            list = dataSorter.getFunctionProfiles(thread);
+            // We now use the kludge wrapper so that the metric ids between trials get mapped
+            // dataSorter.setPpTrial(ppTrial);
+            DataSorter newDataSorter = new DataSorterWrapper(dataSorter, ppTrial);
+            list = newDataSorter.getFunctionProfiles(thread);
 
             for (Iterator it = list.iterator(); it.hasNext();) {
                 PPFunctionProfile fp = (PPFunctionProfile) it.next();
@@ -133,11 +132,10 @@ public class ComparisonBarChartModel extends AbstractBarChartModel {
                     if (blob != null) {
                         blob.add(i, fp);
                     } else {
-                        blob = new RowBlob(fp.getFunctionName());
+                        blob = new RowBlob(fp.getDisplayName());
                         rows.add(blob);
                         rowMap.put(fp.getFunction(), blob);
                         blob.add(i, fp);
-
                     }
                 }
             }
@@ -180,7 +178,14 @@ public class ComparisonBarChartModel extends AbstractBarChartModel {
             return UtilFncs.getOutputString(0, value, 6) + "%";
 
         } else {
-            return UtilFncs.getOutputString(window.units(), value, ParaProf.defaultNumberPrecision);
+            String percentString = "";
+            if (getValue(row, 0) != 0 && subIndex != 0) {
+            //if (getValue(row, 0) != 0) {
+                // compute the ratio of this value to the first one
+                double ratio = value / getValue(row, 0) * 100.0f;
+                percentString = " (" + UtilFncs.getOutputString(0, ratio, 2) + "%)";
+            }
+            return UtilFncs.getOutputString(window.units(), value, ParaProf.defaultNumberPrecision) + percentString;
         }
     }
 
@@ -192,7 +197,7 @@ public class ComparisonBarChartModel extends AbstractBarChartModel {
     public Color getValueColor(int row, int subIndex) {
         // we use the "default" colors for our legend and bars
         List colorList = ParaProf.colorChooser.getColors();
-        return (Color)colorList.get(subIndex % colorList.size());
+        return (Color) colorList.get(subIndex % colorList.size());
     }
 
     public Color getValueHighlightColor(int row, int subIndex) {
@@ -227,7 +232,7 @@ public class ComparisonBarChartModel extends AbstractBarChartModel {
 
     public static void main(String[] args) {
         final ParaProf paraProf = new ParaProf();
-        paraProf.startSystem();
+        paraProf.initialize();
 
         File[] files = new File[1];
         files[0] = new File("/home/amorris/data/packed/lu.C.512.ppk");
@@ -239,8 +244,7 @@ public class ComparisonBarChartModel extends AbstractBarChartModel {
         FunctionBarChartWindow window = FunctionBarChartWindow.CreateComparisonWindow(ppTrial,
                 ppTrial.getDataSource().getMeanData(), null);
         window.addThread(ppTrial, ppTrial.getDataSource().getThread(0, 0, 0));
-        window.addThread(ppTrial, ppTrial.getDataSource().getThread(1,0,0));
-
+        window.addThread(ppTrial, ppTrial.getDataSource().getThread(1, 0, 0));
 
         window.addThread(lu128, lu128.getDataSource().getMeanData());
 
