@@ -2,6 +2,8 @@ package edu.uoregon.tau.paraprof.treetable;
 
 import java.util.*;
 
+import edu.uoregon.tau.common.treetable.AbstractTreeTableModel;
+import edu.uoregon.tau.common.treetable.TreeTableModel;
 import edu.uoregon.tau.paraprof.DataSorter;
 import edu.uoregon.tau.paraprof.PPFunctionProfile;
 import edu.uoregon.tau.paraprof.ParaProfTrial;
@@ -13,9 +15,9 @@ import edu.uoregon.tau.perfdmf.*;
  *    
  * TODO : ...
  *
- * <P>CVS $Id: CallPathModel.java,v 1.1 2005/09/26 21:12:49 amorris Exp $</P>
+ * <P>CVS $Id: CallPathModel.java,v 1.5 2007/02/03 01:40:12 amorris Exp $</P>
  * @author  Alan Morris
- * @version $Revision: 1.1 $
+ * @version $Revision: 1.5 $
  */
 public class CallPathModel extends AbstractTreeTableModel {
 
@@ -26,11 +28,14 @@ public class CallPathModel extends AbstractTreeTableModel {
     private double[] maxValues;
 
     private int sortColumn;
+    private int colorMetric;
     private boolean sortAscending;
 
+    private boolean reversedCallPaths = true;
+    
     private TreeTableWindow window;
 
-    public CallPathModel(TreeTableWindow window, ParaProfTrial ppTrial, edu.uoregon.tau.perfdmf.Thread thread) {
+    public CallPathModel(TreeTableWindow window, ParaProfTrial ppTrial, edu.uoregon.tau.perfdmf.Thread thread, boolean reversedCallPaths) {
         super(null);
         this.window = window;
         root = new TreeTableNode(null, this, "root");
@@ -39,19 +44,25 @@ public class CallPathModel extends AbstractTreeTableModel {
         this.thread = thread;
         this.ppTrial = ppTrial;
 
+        this.reversedCallPaths = reversedCallPaths;
         setupData();
 
     }
+    
+    
 
     private void setupData() {
 
         roots = new ArrayList();
         DataSorter dataSorter = new DataSorter(ppTrial);
+        
 
         // don't ask the thread for its functions directly, since we want group masking to work
-        List functionProfileList = dataSorter.getFunctionProfiles(thread);
+        List functionProfileList = dataSorter.getCallPathFunctionProfiles(thread);
 
         Map rootNames = new HashMap();
+        
+        Group derived = ppTrial.getGroup("TAU_CALLPATH_DERIVED");
 
         if (window.getTreeMode()) {
             for (Iterator it = functionProfileList.iterator(); it.hasNext();) {
@@ -60,7 +71,16 @@ public class CallPathModel extends AbstractTreeTableModel {
                 FunctionProfile fp = ppFunctionProfile.getFunctionProfile();
 
                 if (fp != null && fp.isCallPathFunction()) {
-                    String rootName = UtilFncs.getLeftSide(fp.getName());
+                    String rootName;
+                    if (reversedCallPaths) {
+                        rootName = UtilFncs.getRevLeftSide(fp.getFunction().getReversedName());
+                    } else {
+                        if (fp.getFunction().isGroupMember(derived)) {
+                            continue;
+                        }
+                        rootName = UtilFncs.getLeftSide(fp.getName());
+                    }
+                    
                     if (rootNames.get(rootNames) == null) {
                         rootNames.put(rootName, "1");
                     }
@@ -214,4 +234,24 @@ public class CallPathModel extends AbstractTreeTableModel {
         return ppTrial;
     }
 
+    public boolean getReversedCallPaths() {
+        return reversedCallPaths;
+    }
+
+    public void setReversedCallPaths(boolean reversedCallPaths) {
+        this.reversedCallPaths = reversedCallPaths;
+    }
+
+
+
+    public int getColorMetric() {
+        return colorMetric;
+    }
+
+
+
+    public void setColorMetric(int colorMetric) {
+        this.colorMetric = colorMetric;
+    }
+
 }
