@@ -1,8 +1,7 @@
 package edu.uoregon.tau.paraprof.treetable;
 
 import java.awt.*;
-import java.awt.event.ActionEvent;
-import java.awt.event.ActionListener;
+import java.awt.event.*;
 import java.awt.print.PageFormat;
 import java.awt.print.Printable;
 import java.awt.print.PrinterException;
@@ -15,9 +14,13 @@ import javax.swing.event.TreeExpansionEvent;
 import javax.swing.event.TreeExpansionListener;
 import javax.swing.table.AbstractTableModel;
 import javax.swing.table.TableColumn;
+import javax.swing.tree.TreePath;
 
+import edu.uoregon.tau.common.ImageExport;
+import edu.uoregon.tau.common.treetable.AbstractTreeTableModel;
+import edu.uoregon.tau.common.treetable.JTreeTable;
+import edu.uoregon.tau.common.treetable.JTreeTable.TreeTableCellRenderer;
 import edu.uoregon.tau.paraprof.*;
-import edu.uoregon.tau.paraprof.interfaces.ImageExport;
 import edu.uoregon.tau.paraprof.interfaces.ParaProfWindow;
 import edu.uoregon.tau.paraprof.interfaces.UnitListener;
 import edu.uoregon.tau.paraprof.treetable.ColumnChooser.CheckBoxListItem;
@@ -30,9 +33,9 @@ import edu.uoregon.tau.paraprof.treetable.TreeTableColumn.*;
  *    
  * TODO : ...
  *
- * <P>CVS $Id: TreeTableWindow.java,v 1.1 2005/09/26 21:12:51 amorris Exp $</P>
+ * <P>CVS $Id: TreeTableWindow.java,v 1.10 2007/02/01 22:11:50 amorris Exp $</P>
  * @author  Alan Morris
- * @version $Revision: 1.1 $
+ * @version $Revision: 1.10 $
  */
 public class TreeTableWindow extends JFrame implements TreeExpansionListener, Observer, ParaProfWindow, Printable, UnitListener,
         ImageExport {
@@ -45,18 +48,27 @@ public class TreeTableWindow extends JFrame implements TreeExpansionListener, Ob
     private int colorMetricID;
     private int units = ParaProf.preferences.getUnits();
 
-    private final JMenuItem showAsTreeMenuItem = new JCheckBoxMenuItem("Show as Call Path Tree");
+    private final JMenuItem showAsTreeMenuItem = new JCheckBoxMenuItem("Show as Call Tree");
+    private final JMenuItem reverseTreeMenuItem = new JCheckBoxMenuItem("Reverse Call Tree", false);
     private final JMenuItem showInclExclMenuItem = new JCheckBoxMenuItem("Show Inclusive/Exclusive", true);
 
     private List columns;
     private ColumnChooser columnChooser;
 
+    public TreeTableWindow(ParaProfTrial ppTrial, edu.uoregon.tau.perfdmf.Thread thread) {
+        this(ppTrial, thread, null);
+    }
+
     public TreeTableWindow(ParaProfTrial ppTrial, edu.uoregon.tau.perfdmf.Thread thread, Component invoker) {
 
         this.ppTrial = ppTrial;
         this.thread = thread;
         ppTrial.addObserver(this);
 
+        if (!ppTrial.getDataSource().getReverseDataAvailable()) {
+            reverseTreeMenuItem.setEnabled(false);
+        }
+     
         // create the column chooser.  Note: the column chooser holds the data on which columns are shown
         columnChooser = new ColumnChooser(this, ppTrial);
 
@@ -65,17 +77,19 @@ public class TreeTableWindow extends JFrame implements TreeExpansionListener, Ob
         setLocation(WindowPlacer.getNewLocation(this, invoker));
 
         if (thread.getNodeID() == -1) {
-            this.setTitle("Mean Statistics - " + ppTrial.getTrialIdentifier(ParaProf.preferences.getShowPathTitleInReverse()));
+            this.setTitle("TAU: ParaProf: Mean Statistics - " + ppTrial.getTrialIdentifier(ParaProf.preferences.getShowPathTitleInReverse()));
         } else if (thread.getNodeID() == -2) {
-            this.setTitle("Total Statistics - " + ppTrial.getTrialIdentifier(ParaProf.preferences.getShowPathTitleInReverse()));
+            this.setTitle("TAU: ParaProf: Total Statistics - " + ppTrial.getTrialIdentifier(ParaProf.preferences.getShowPathTitleInReverse()));
         } else if (thread.getNodeID() == -3) {
-            this.setTitle("Std. Dev. Statistics - " + ppTrial.getTrialIdentifier(ParaProf.preferences.getShowPathTitleInReverse()));
+            this.setTitle("TAU: ParaProf: Std. Dev. Statistics - "
+                    + ppTrial.getTrialIdentifier(ParaProf.preferences.getShowPathTitleInReverse()));
         } else {
-            this.setTitle("Thread Statistics: " + "n,c,t, " + thread.getNodeID() + "," + thread.getContextID() + ","
+            this.setTitle("TAU: ParaProf: Thread Statistics: " + "n,c,t, " + thread.getNodeID() + "," + thread.getContextID() + ","
                     + thread.getThreadID() + " - " + ppTrial.getTrialIdentifier(ParaProf.preferences.getShowPathTitleInReverse()));
         }
+        ParaProfUtils.setFrameIcon(this);
 
-        if (ParaProf.helpWindow.isVisible()) {
+        if (ParaProf.getHelpWindow().isVisible()) {
             this.help(false);
         }
 
@@ -109,30 +123,54 @@ public class TreeTableWindow extends JFrame implements TreeExpansionListener, Ob
             CheckBoxListItem item = (CheckBoxListItem) metricModel.getElementAt(i);
             if (item.getSelected()) {
 
+                boolean absSet = false;
+                boolean percentSet = false;
+                boolean percallSet = false;
                 for (int j = 0; j < valueModel.getSize(); j++) {
                     CheckBoxListItem valueItem = (CheckBoxListItem) valueModel.getElementAt(j);
                     if (valueItem.getSelected()) {
                         String str = (String) valueItem.getUserObject();
-                        if (str.equals("Absolute Value")) {
+                        if (str.equals("Exclusive Value") || str.equals("Inclusive Value")) {
                             if (showInclExclMenuItem.isSelected()) {
-                                columns.add(new InclusiveColumn(this, i));
-                                columns.add(new ExclusiveColumn(this, i));
+                                if (str.equals("Inclusive Value")) {
+                                    columns.add(new InclusiveColumn(this, i));
+                                }
+                                if (str.equals("Exclusive Value")) {
+                                    columns.add(new ExclusiveColumn(this, i));
+                                }
                             } else {
-                                columns.add(new RegularMetricColumn(this, i));
+                                if (absSet == false) {
+                                    columns.add(new RegularMetricColumn(this, i));
+                                    absSet = true;
+                                }
                             }
-                        } else if (str.equals("Percent Value")) {
+                        } else if (str.equals("Inclusive Percent Value") || str.equals("Exclusive Percent Value")) {
                             if (showInclExclMenuItem.isSelected()) {
-                                columns.add(new InclusivePercentColumn(this, i));
-                                columns.add(new ExclusivePercentColumn(this, i));
+                                if (str.equals("Inclusive Percent Value")) {
+                                    columns.add(new InclusivePercentColumn(this, i));
+                                }
+                                if (str.equals("Exclusive Percent Value")) {
+                                    columns.add(new ExclusivePercentColumn(this, i));
+                                }
                             } else {
-                                columns.add(new RegularPercentMetricColumn(this, i));
+                                if (percentSet == false) {
+                                    columns.add(new RegularPercentMetricColumn(this, i));
+                                    percentSet = true;
+                                }
                             }
-                        } else if (str.equals("Absolute Value Per Call")) {
+                        } else if (str.equals("Inclusive Value Per Call") || str.equals("Exclusive Value Per Call")) {
                             if (showInclExclMenuItem.isSelected()) {
-                                columns.add(new InclusivePerCallColumn(this, i));
-                                columns.add(new ExclusivePerCallColumn(this, i));
+                                if (str.equals("Inclusive Value Per Call")) {
+                                    columns.add(new InclusivePerCallColumn(this, i));
+                                }
+                                if (str.equals("Exclusive Value Per Call")) {
+                                    columns.add(new ExclusivePerCallColumn(this, i));
+                                }
                             } else {
-                                columns.add(new RegularPerCallMetricColumn(this, i));
+                                if (percallSet == false) {
+                                    columns.add(new RegularPerCallMetricColumn(this, i));
+                                    percallSet = true;
+                                }
                             }
                         }
                     }
@@ -165,11 +203,28 @@ public class TreeTableWindow extends JFrame implements TreeExpansionListener, Ob
         } else {
             showInclExclMenuItem.setEnabled(true);
 
-            gbc.fill = GridBagConstraints.HORIZONTAL;
             gbc.anchor = GridBagConstraints.NORTH;
-            gbc.weightx = 1.0;
             gbc.weighty = 0.0;
-            addCompItem(new ColorBar(), gbc, 0, 0, 1, 1);
+           
+            
+            if (ppTrial.getNumberOfMetrics() > 1) {
+                final JComboBox metricBox = new JComboBox(ppTrial.getMetrics().toArray());
+                metricBox.setSelectedIndex(ppTrial.getDefaultMetricID());
+
+                metricBox.addActionListener(new ActionListener() {
+                    public void actionPerformed(ActionEvent e) {
+                        model.setColorMetric(metricBox.getSelectedIndex());
+                        repaint();
+                    } });
+                gbc.weightx = 0.0;
+                gbc.fill = GridBagConstraints.NONE;
+                addCompItem(metricBox, gbc, 0, 0, 1, 1);
+            }
+            
+            
+            gbc.weightx = 1.0;
+            gbc.fill = GridBagConstraints.HORIZONTAL;
+            addCompItem(new ColorBar(), gbc, 1, 0, 1, 1);
         }
 
         if (scrollPane != null) {
@@ -198,7 +253,7 @@ public class TreeTableWindow extends JFrame implements TreeExpansionListener, Ob
         //columns.add(new StdDevColumn(this, 0));
         //columns.add(new MiniHistogramColumn(this));
 
-        model = new CallPathModel(this, ppTrial, thread);
+        model = new CallPathModel(this, ppTrial, thread, reverseTreeMenuItem.isSelected());
         createTreeTable(model);
         addComponents();
 
@@ -227,11 +282,13 @@ public class TreeTableWindow extends JFrame implements TreeExpansionListener, Ob
 
                         String arg = evt.getActionCommand();
 
-                        if (arg.equals("Show as Call Path Tree")) {
+                        if (arg.equals("Show as Call Tree")) {
                             if (showAsTreeMenuItem.isSelected()) {
                                 showInclExclMenuItem.setSelected(false);
                             }
 
+                            setupData();
+                        } else if (arg.equals("Reverse Call Tree")) {
                             setupData();
                         } else if (arg.equals("Show Inclusive/Exclusive")) {
                             setupData();
@@ -250,6 +307,9 @@ public class TreeTableWindow extends JFrame implements TreeExpansionListener, Ob
         optionsMenu.add(showAsTreeMenuItem);
         showAsTreeMenuItem.setSelected(true);
 
+        reverseTreeMenuItem.addActionListener(actionListener);
+        optionsMenu.add(reverseTreeMenuItem);
+
         showInclExclMenuItem.addActionListener(actionListener);
         optionsMenu.add(showInclExclMenuItem);
 
@@ -285,7 +345,40 @@ public class TreeTableWindow extends JFrame implements TreeExpansionListener, Ob
 
     private void createTreeTable(AbstractTreeTableModel model) {
         //        treeTable = new JTreeTable(model, showAsTreeMenuItem.isSelected());
-        treeTable = new JTreeTable(model, true, this);
+        
+        
+
+        treeTable = new JTreeTable(model, true, true);
+
+        final JTree tree = treeTable.getTree();
+        
+        // Add a mouse listener for this tree.
+        MouseListener ml = new MouseAdapter() {
+            public void mousePressed(MouseEvent evt) {
+                try {
+                    int selRow = tree.getRowForLocation(evt.getX(), evt.getY());
+                    TreePath path = tree.getPathForLocation(evt.getX(), evt.getY());
+                    if (path != null) {
+                        TreeTableNode node = (TreeTableNode) path.getLastPathComponent();
+                        if (ParaProfUtils.rightClick(evt)) {
+                            JPopupMenu popup;
+                            if (node.getFunctionProfile() != null) {
+                                popup = ParaProfUtils.createFunctionClickPopUp(node.getModel().getPPTrial(),
+                                        node.getFunctionProfile().getFunction(), getThread(), treeTable);
+                                popup.show(treeTable, evt.getX(), evt.getY());
+                            } else {
+                                //popup = new JPopupMenu();
+                            }
+                        }
+                    }
+                } catch (Exception e) {
+                    ParaProfUtils.handleException(e);
+                }
+            }
+        };
+      
+        treeTable.addMouseListener(ml);  
+        
 
         treeTable.getTree().addTreeExpansionListener(this);
         treeTable.getTree().setCellRenderer(new TreePortionCellRenderer());
@@ -296,7 +389,6 @@ public class TreeTableWindow extends JFrame implements TreeExpansionListener, Ob
         TableColumn col = treeTable.getColumnModel().getColumn(0);
 
         int nameWidth = 500;
-        
 
         if (ppTrial.getNumberOfMetrics() > 1) {
             nameWidth = 350;
@@ -331,13 +423,13 @@ public class TreeTableWindow extends JFrame implements TreeExpansionListener, Ob
 
     public void help(boolean display) {
         //Show the ParaProf help window.
-        ParaProf.helpWindow.clearText();
+        ParaProf.getHelpWindow().clearText();
         if (display)
-            ParaProf.helpWindow.show();
-        ParaProf.helpWindow.writeText("This is the Statistics Table.\n");
-        ParaProf.helpWindow.writeText("This window shows you function data across a given thread (or mean/std.dev.)\n");
-        ParaProf.helpWindow.writeText("If callpath data is present, it will be shown as a tree on the left.");
-        ParaProf.helpWindow.writeText("In this mode, the metric values will show Inclusive when the node is collapsed, and exclusive when the node is expanded.\n");
+            ParaProf.getHelpWindow().show();
+        ParaProf.getHelpWindow().writeText("This is the Statistics Table.\n");
+        ParaProf.getHelpWindow().writeText("This window shows you function data across a given thread (or mean/std.dev.)\n");
+        ParaProf.getHelpWindow().writeText("If callpath data is present, it will be shown as a tree on the left.");
+        ParaProf.getHelpWindow().writeText("In this mode, the metric values will show Inclusive when the node is collapsed, and exclusive when the node is expanded.\n");
     }
 
     public void closeThisWindow() {
@@ -359,7 +451,7 @@ public class TreeTableWindow extends JFrame implements TreeExpansionListener, Ob
 
             return Printable.PAGE_EXISTS;
         } catch (Exception e) {
-            new ParaProfErrorDialog(e);
+            ParaProfUtils.handleException(e);
             return NO_SUCH_PAGE;
         }
     }
@@ -395,6 +487,11 @@ public class TreeTableWindow extends JFrame implements TreeExpansionListener, Ob
 
     public void export(Graphics2D g2D, boolean toScreen, boolean fullWindow, boolean drawHeader) {
         if (fullWindow) {
+            // first draw the column headers
+            scrollPane.getColumnHeader().paintAll(g2D);
+            // translate past the column headers
+            g2D.translate(0,scrollPane.getColumnHeader().getHeight());
+            // draw the entire treetable
             treeTable.paintAll(g2D);
         } else {
             scrollPane.paintAll(g2D);
@@ -403,9 +500,12 @@ public class TreeTableWindow extends JFrame implements TreeExpansionListener, Ob
 
     public Dimension getImageSize(boolean fullScreen, boolean header) {
         if (fullScreen) {
-            return scrollPane.getSize();
+            Dimension d = treeTable.getSize();
+            // we want to draw the column headers as well
+            d.setSize(d.getWidth(), d.getHeight() + scrollPane.getColumnHeader().getHeight());
+            return d;
         } else {
-            return treeTable.getSize();
+            return scrollPane.getSize();
         }
     }
 
