@@ -20,7 +20,7 @@ import java.sql.*;
  * an application from which the TAU performance data has been generated.
  * An application has zero or more experiments associated with it.
  *
- * <P>CVS $Id: Application.java,v 1.1 2005/09/26 20:24:24 amorris Exp $</P>
+ * <P>CVS $Id: Application.java,v 1.7 2006/10/30 18:10:26 amorris Exp $</P>
  * @author	Kevin Huck, Robert Bell
  * @version 0.1
  * @since   0.1
@@ -39,7 +39,7 @@ public class Application implements Serializable {
     // numFields, the number of optional fields found in the DB
     public Application() {
         if (Application.fieldNames == null) {
-            this.fields = new String[0];
+            fields = new String[0];
         } else {
             this.fields = new String[Application.fieldNames.length];
         }
@@ -52,7 +52,6 @@ public class Application implements Serializable {
         this.fields = (String[]) app.fields.clone();
     }
 
-    
     public void reallocMetaData() {
         if (Application.fieldNames == null) {
             this.fields = new String[0];
@@ -60,31 +59,30 @@ public class Application implements Serializable {
             this.fields = new String[Application.fieldNames.length];
         }
     }
-    
-    
+
     public String[] getFields() {
         return fields;
     }
-    
+
     public void setFields(String[] fields) {
         this.fields = fields;
     }
-    
-   	/**
-	 * Returns the column names for the Application table
-	 *
-	 * @param	db	the database connection
-	 * @return	String[] an array of String objects
-	 */
-	public static String[] getFieldNames(DB db) throws DatabaseException {
-		getMetaData(db);
-		return fieldNames;
-	}
- 
+
+    /**
+     * Returns the column names for the Application table
+     *
+     * @param	db	the database connection
+     * @return	String[] an array of String objects
+     */
+    public static String[] getFieldNames(DB db) throws DatabaseException {
+        getMetaData(db);
+        return fieldNames;
+    }
+
     public static void getMetaData(DB db) {
         // see if we've already have them
-        if (Application.fieldNames != null)
-            return;
+        //       if (Application.fieldNames != null)
+        //            return;
 
         try {
             ResultSet resultSet = null;
@@ -94,7 +92,8 @@ public class Application implements Serializable {
 
             DatabaseMetaData dbMeta = db.getMetaData();
 
-            if (db.getDBType().compareTo("oracle") == 0) {
+            if ((db.getDBType().compareTo("oracle") == 0) || (db.getDBType().compareTo("derby") == 0)
+                    || (db.getDBType().compareTo("db2") == 0)) {
                 resultSet = dbMeta.getColumns(null, null, "APPLICATION", "%");
             } else {
                 resultSet = dbMeta.getColumns(null, null, "application", "%");
@@ -102,6 +101,7 @@ public class Application implements Serializable {
 
             Vector nameList = new Vector();
             Vector typeList = new Vector();
+            boolean seenID = false;
 
             while (resultSet.next() != false) {
 
@@ -109,6 +109,14 @@ public class Application implements Serializable {
                 String cname = resultSet.getString("COLUMN_NAME");
                 String typename = resultSet.getString("TYPE_NAME");
 
+                // this code is because of a bug in derby...
+                if (cname.equals("ID")) {
+                    if (!seenID)
+                        seenID = true;
+                    else
+                        break;
+                }
+
                 // only integer and string types (for now)
                 // don't do name and id, we already know about them
                 if (DBConnector.isReadAbleType(ctype) && cname.toUpperCase().compareTo("ID") != 0
@@ -132,8 +140,6 @@ public class Application implements Serializable {
         }
     }
 
-   
-
     public int getNumFields() {
         return fields.length;
     }
@@ -223,14 +229,14 @@ public class Application implements Serializable {
     }
 
     public static Vector getApplicationList(DB db, String whereClause) {
+        StringBuffer buf = null;
         try {
-
             Application.getMetaData(db);
 
             ResultSet resultSet = null;
             Vector applications = new Vector();
 
-            StringBuffer buf = new StringBuffer("select id, name");
+            buf = new StringBuffer("select id, name");
 
             for (int i = 0; i < Application.fieldNames.length; i++) {
                 buf.append(", " + Application.fieldNames[i]);
@@ -242,6 +248,10 @@ public class Application implements Serializable {
 
             if (db.getDBType().compareTo("oracle") == 0) {
                 buf.append(" order by dbms_lob.substr(name) asc");
+            } else if (db.getDBType().compareTo("derby") == 0) {
+                buf.append(" order by cast (name as varchar(256)) asc");
+            } else if (db.getDBType().compareTo("db2") == 0) {
+                buf.append(" order by cast (name as varchar(256)) asc");
             } else {
                 buf.append(" order by name asc ");
             }
@@ -266,92 +276,88 @@ public class Application implements Serializable {
             resultSet.close();
 
             return applications;
-
-        } catch (SQLException e) { // this is unreal, I can't throw it because the base class Datasession doesn't throw it, why on earth does the base class have "getApplicationList"???
-            e.printStackTrace();
-            return null;
+        } catch (SQLException e) {
+            if (buf != null)
+                System.out.println(buf.toString());
+            throw new DatabaseException("", e);
         }
+
     }
 
-    public int saveApplication(DB db) {
+    public int saveApplication(DB db) throws SQLException {
 
-        try {
-            boolean itExists = false;
+        boolean itExists = false;
 
-            // First, determine whether it exists already (whether we are doing an insert or update)
-            PreparedStatement statement = db.prepareStatement("SELECT name FROM " + db.getSchemaPrefix()
-                    + "application WHERE id = ?");
-            statement.setInt(1, this.getID());
-            ResultSet resultSet = statement.executeQuery();
-            while (resultSet.next() != false) {
-                itExists = true;
-                break;
-            }
-            resultSet.close();
-            statement.close();
-
-            StringBuffer buf = new StringBuffer();
-            if (itExists) {
-                buf.append("UPDATE " + db.getSchemaPrefix() + "application SET name = ?");
-                for (int i = 0; i < this.getNumFields(); i++) {
-                    if (DBConnector.isWritableType(this.getFieldType(i)))
-                        buf.append(", " + this.getFieldName(i) + " = ?");
-                }
-                buf.append(" WHERE id = ?");
-            } else {
-                buf.append("INSERT INTO " + db.getSchemaPrefix() + "application (name");
-                for (int i = 0; i < this.getNumFields(); i++) {
-                    if (DBConnector.isWritableType(this.getFieldType(i)))
-                        buf.append(", " + this.getFieldName(i));
-                }
-                buf.append(") VALUES (?");
-                for (int i = 0; i < this.getNumFields(); i++) {
-                    if (DBConnector.isWritableType(this.getFieldType(i)))
-                        buf.append(", ?");
-                }
-                buf.append(")");
-            }
-
-            statement = db.prepareStatement(buf.toString());
-
-            int pos = 1;
-            statement.setString(pos++, this.getName());
+        // First, determine whether it exists already (whether we are doing an insert or update)
+        PreparedStatement statement = db.prepareStatement("SELECT name FROM " + db.getSchemaPrefix() + "application WHERE id = ?");
+        statement.setInt(1, this.getID());
+        ResultSet resultSet = statement.executeQuery();
+        while (resultSet.next() != false) {
+            itExists = true;
+            break;
+        }
+        resultSet.close();
+        statement.close();
 
+        StringBuffer buf = new StringBuffer();
+        if (itExists) {
+            buf.append("UPDATE " + db.getSchemaPrefix() + "application SET name = ?");
             for (int i = 0; i < this.getNumFields(); i++) {
                 if (DBConnector.isWritableType(this.getFieldType(i)))
-                    statement.setString(pos++, this.getField(i));
+                    buf.append(", " + this.getFieldName(i) + " = ?");
             }
-
-            if (itExists) {
-                statement.setInt(pos++, this.getID());
+            buf.append(" WHERE id = ?");
+        } else {
+            buf.append("INSERT INTO " + db.getSchemaPrefix() + "application (name");
+            for (int i = 0; i < this.getNumFields(); i++) {
+                if (DBConnector.isWritableType(this.getFieldType(i)))
+                    buf.append(", " + this.getFieldName(i));
             }
-            statement.executeUpdate();
-            statement.close();
-
-            int newApplicationID = 0;
-
-            if (itExists) {
-                newApplicationID = this.getID();
-            } else {
-                String tmpStr = new String();
-                if (db.getDBType().compareTo("mysql") == 0) {
-                    tmpStr = "select LAST_INSERT_ID();";
-                } else if (db.getDBType().compareTo("db2") == 0) {
-                    tmpStr = "select IDENTITY_VAL_LOCAL() FROM application";
-                } else if (db.getDBType().compareTo("oracle") == 0) {
-                    tmpStr = "SELECT " + db.getSchemaPrefix() + "application_id_seq.currval FROM DUAL";
-                } else { // postgresql 
-                    tmpStr = "select currval('application_id_seq');";
-                }
-                newApplicationID = Integer.parseInt(db.getDataItem(tmpStr));
+            buf.append(") VALUES (?");
+            for (int i = 0; i < this.getNumFields(); i++) {
+                if (DBConnector.isWritableType(this.getFieldType(i)))
+                    buf.append(", ?");
             }
-            return newApplicationID;
-
-        } catch (SQLException e) { // this is unreal, I can't throw it because the base class Datasession doesn't throw it, why on earth does the base class have "getApplicationList"???
-            e.printStackTrace();
-            return -1;
+            buf.append(")");
         }
 
+        statement = db.prepareStatement(buf.toString());
+
+        int pos = 1;
+        statement.setString(pos++, this.getName());
+
+        for (int i = 0; i < this.getNumFields(); i++) {
+            if (DBConnector.isWritableType(this.getFieldType(i)))
+                statement.setString(pos++, this.getField(i));
+        }
+
+        if (itExists) {
+            statement.setInt(pos++, this.getID());
+        }
+        statement.executeUpdate();
+        statement.close();
+
+        int newApplicationID = 0;
+
+        if (itExists) {
+            newApplicationID = this.getID();
+        } else {
+            String tmpStr = new String();
+            if (db.getDBType().compareTo("mysql") == 0) {
+                tmpStr = "select LAST_INSERT_ID();";
+            } else if (db.getDBType().compareTo("db2") == 0) {
+                tmpStr = "select IDENTITY_VAL_LOCAL() FROM application";
+            } else if (db.getDBType().compareTo("derby") == 0) {
+                tmpStr = "select IDENTITY_VAL_LOCAL() FROM application";
+            } else if (db.getDBType().compareTo("oracle") == 0) {
+                tmpStr = "SELECT " + db.getSchemaPrefix() + "application_id_seq.currval FROM DUAL";
+            } else { // postgresql 
+                tmpStr = "select currval('application_id_seq');";
+            }
+            newApplicationID = Integer.parseInt(db.getDataItem(tmpStr));
+        }
+        return newApplicationID;
+
     }
 
     private boolean exists(DB db) {
@@ -401,4 +407,12 @@ public class Application implements Serializable {
         aOutputStream.writeObject(fieldTypes);
     }
 
+    /**
+     *  hack - needed to delete meta so that it is reloaded each time a new database is created.
+     */
+    public void removeMetaData() {
+        fieldNames = null;
+        fieldTypes = null;
+    }
+
 }
