@@ -1,23 +1,37 @@
 package edu.uoregon.tau.perfdmf;
 
+import java.text.SimpleDateFormat;
 import java.util.*;
 import java.io.*;
 import java.math.BigDecimal;
 import java.sql.*;
 
-
 /**
  * This class represents a data source.  After loading, data is availiable through the
  * public methods.
  *  
- * <P>CVS $Id: DataSource.java,v 1.2 2005/10/18 22:48:55 amorris Exp $</P>
- * @author	Robert Bell, Alan Morris
- * @version	$Revision: 1.2 $
- * @see		TrialData
- * @see		NCT
+ * <P>CVS $Id: DataSource.java,v 1.13.2.1 2007/02/20 01:18:26 amorris Exp $</P>
+ * @author  Robert Bell, Alan Morris
+ * @version $Revision: 1.13.2.1 $
  */
 public abstract class DataSource {
 
+    public static final SimpleDateFormat dateTime = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");
+
+    public static int TAUPROFILE = 0;
+    public static int PPROF = 1;
+    public static int DYNAPROF = 2;
+    public static int MPIP = 3;
+    public static int HPM = 4;
+    public static int GPROF = 5;
+    public static int PSRUN = 6;
+    public static int PPK = 7;
+    public static int CUBE = 8;
+    public static int HPCTOOLKIT = 9;
+
+    public static String formatTypeStrings[] = { "Tau profiles", "Tau pprof.dat", "Dynaprof", "MpiP", "HPMToolkit", "Gprof",
+            "PSRun", "ParaProf Packed Profile", "Cube", "HPCToolkit" };
+
     private static boolean meanIncludeNulls = true;
 
     private boolean userEventsPresent = false;
@@ -25,8 +39,7 @@ public abstract class DataSource {
     private boolean groupNamesPresent = false;
     private boolean phasesPresent = false;
     private Function topLevelPhase;
-    
-    
+
     // data structures
     private List metrics = null;
     protected Thread meanData = null;
@@ -38,6 +51,9 @@ public abstract class DataSource {
     private Map userEvents = new TreeMap();
     private List allThreads;
 
+    private boolean generateIntermediateCallPathData;
+    private boolean reverseDataAvailable;
+
     // just a holder for the output of getMaxNCTNumbers(), makes subsequent calls instantaneous
     private int[] maxNCT = null;
 
@@ -47,9 +63,14 @@ public abstract class DataSource {
 
     abstract public void cancelLoad();
 
-    
     protected volatile boolean reloading;
-    
+
+    protected Map metaData;
+
+    public DataSource() {
+    // nothing
+    }
+
     // by default no files
     public List getFiles() {
         return new ArrayList();
@@ -58,7 +79,7 @@ public abstract class DataSource {
     public boolean isReloading() {
         return reloading;
     }
-    
+
     public boolean reloadData() throws Exception {
         if (this.reloading) {
             return false;
@@ -71,7 +92,7 @@ public abstract class DataSource {
         this.reloading = false;
         return true;
     }
-    
+
     protected void cleanData() {
         for (Iterator it = this.getAllThreads().iterator(); it.hasNext();) {
             Thread thread = (Thread) it.next();
@@ -79,8 +100,8 @@ public abstract class DataSource {
                 FunctionProfile fp = (FunctionProfile) it2.next();
                 if (fp != null) {
                     for (int m = 0; m < this.getNumberOfMetrics(); m++) {
-                        fp.setExclusive(m,0);
-                        fp.setInclusive(m,0);
+                        fp.setExclusive(m, 0);
+                        fp.setInclusive(m, 0);
                     }
                     fp.setNumSubr(0);
                     fp.setNumCalls(0);
@@ -88,7 +109,7 @@ public abstract class DataSource {
             }
         }
     }
-    
+
     public Thread getMeanData() {
         return meanData;
     }
@@ -101,7 +122,7 @@ public abstract class DataSource {
         return totalData;
     }
 
-    protected void setCallPathDataPresent(boolean callPathDataPresent) {
+    private void setCallPathDataPresent(boolean callPathDataPresent) {
         this.callPathDataPresent = callPathDataPresent;
     }
 
@@ -131,14 +152,15 @@ public abstract class DataSource {
 
     public Function addFunction(String name, int numMetrics) {
         name = name.trim();
-        Object obj = functions.get(name);
+        Function function = (Function) functions.get(name);
 
         // return the function if found
-        if (obj != null)
-            return (Function) obj;
+        if (function != null) {
+            return function;
+        }
 
         // otherwise, add it and return it
-        Function function = new Function(name, functions.size(), numMetrics);
+        function = new Function(name, functions.size(), numMetrics);
         functions.put(name, function);
         return function;
     }
@@ -158,11 +180,13 @@ public abstract class DataSource {
     public UserEvent addUserEvent(String name) {
         Object obj = userEvents.get(name);
 
-        if (obj != null)
+        if (obj != null) {
             return (UserEvent) obj;
+        }
 
         UserEvent userEvent = new UserEvent(name, userEvents.size() + 1);
         userEvents.put(name, userEvent);
+        setUserEventsPresent(true);
         return userEvent;
     }
 
@@ -181,8 +205,9 @@ public abstract class DataSource {
     public Group getGroup(String name) {
         return (Group) groups.get(name);
     }
-    
+
     public Group addGroup(String name) {
+        name = name.trim();
         Object obj = groups.get(name);
 
         if (obj != null)
@@ -193,6 +218,21 @@ public abstract class DataSource {
         return group;
     }
 
+    protected void addGroups(String groupNames, Function function) {
+        if (groupNames == null) {
+            return;
+        }
+        StringTokenizer st = new StringTokenizer(groupNames, "|");
+        while (st.hasMoreTokens()) {
+            String groupName = st.nextToken();
+            if (groupName != null) {
+                Group group = this.addGroup(groupName.trim());
+                function.addGroup(group);
+            }
+        }
+        this.setGroupNamesPresent(true);
+    }
+
     public int getNumGroups() {
         return groups.size();
     }
@@ -213,20 +253,16 @@ public abstract class DataSource {
 
             for (Iterator it = this.getNodes(); it.hasNext();) {
                 Node node = (Node) it.next();
-                if (node.getNodeID() > maxNCT[0])
-                    maxNCT[0] = node.getNodeID();
+                maxNCT[0] = Math.max(maxNCT[0], node.getNodeID());
                 for (Iterator it2 = node.getContexts(); it2.hasNext();) {
                     Context context = (Context) it2.next();
-                    if (context.getContextID() > maxNCT[1])
-                        maxNCT[1] = context.getContextID();
+                    maxNCT[1] = Math.max(maxNCT[1], context.getContextID());
                     for (Iterator it3 = context.getThreads(); it3.hasNext();) {
-                        edu.uoregon.tau.perfdmf.Thread thread = (edu.uoregon.tau.perfdmf.Thread) it3.next();
-                        if (thread.getThreadID() > maxNCT[2])
-                            maxNCT[2] = thread.getThreadID();
+                        Thread thread = (Thread) it3.next();
+                        maxNCT[2] = Math.max(maxNCT[2], thread.getThreadID());
                     }
                 }
             }
-
         }
         return maxNCT;
     }
@@ -265,6 +301,17 @@ public abstract class DataSource {
     public void addMetric(Metric metric) {
         if (this.metrics == null) {
             this.metrics = new ArrayList();
+        } else {
+            for (Iterator it = getAllThreads().iterator(); it.hasNext();) {
+                Thread thread = (Thread) it.next();
+                thread.addMetric();
+            }
+
+            if (meanData != null) {
+                meanData.addMetric();
+                totalData.addMetric();
+                stddevData.addMetric();
+            }
         }
 
         metric.setID(this.getNumberOfMetrics());
@@ -278,11 +325,13 @@ public abstract class DataSource {
      *            Name of metric to be added
      */
     public Metric addMetric(String metricName) {
+        //System.err.println("addMetric("+metricName+")");
         if (metrics != null) {
             for (Iterator it = metrics.iterator(); it.hasNext();) {
                 Metric metric = (Metric) it.next();
-                if (metric.getName().equals(metricName))
+                if (metric.getName().equals(metricName)) {
                     return metric;
+                }
             }
         }
 
@@ -298,7 +347,17 @@ public abstract class DataSource {
      * @return List of Metrics
      */
     public List getMetrics() {
-        return this.metrics;
+        return metrics;
+    }
+
+    public Metric getMetric(String name) {
+        for (Iterator it = metrics.iterator(); it.hasNext();) {
+            Metric metric = (Metric) it.next();
+            if (metric.getName().equals(name)) {
+                return metric;
+            }
+        }
+        return null;
     }
 
     /**
@@ -310,10 +369,11 @@ public abstract class DataSource {
      * @return Metric with given id.
      */
     public Metric getMetric(int metricID) {
-        if ((this.metrics != null) && (metricID < this.metrics.size()))
+        if ((this.metrics != null) && (metricID < this.metrics.size())) {
             return (Metric) this.metrics.get(metricID);
-        else
+        } else {
             return null;
+        }
     }
 
     /**
@@ -327,10 +387,11 @@ public abstract class DataSource {
      * @return The metric name as a String.
      */
     public String getMetricName(int metricID) {
-        if ((this.metrics != null) && (metricID < this.metrics.size()))
+        if ((this.metrics != null) && (metricID < this.metrics.size())) {
             return ((Metric) this.metrics.get(metricID)).getName();
-        else
+        } else {
             return null;
+        }
     }
 
     /**
@@ -342,10 +403,92 @@ public abstract class DataSource {
      */
     public int getNumberOfMetrics() {
         //Try getting the metric name.
-        if (this.metrics != null)
+        if (this.metrics != null) {
             return metrics.size();
-        else
-            return -1;
+        } else {
+            return 0;
+        }
+    }
+
+    private void generateBonusCallPathData() {
+        /*
+         * The "bonus" data is necessary for the reverse call tree
+         * With normal callpath data we have:
+         *     A => B1 => C => D
+         *     A => B2 => C => D
+         *     D
+         *     
+         * Here, D is duplicate data, it is the sum of the other two
+         * The Bonus data is additional duplicate data:
+         *     B1 => C => D
+         *     B2 => C => D
+         *     C => D
+         *     
+         * It provides an aggregate view from each root for all parents
+         */
+        if (!getCallPathDataPresent()) {
+            return;
+        }
+
+        List functions = new ArrayList();
+
+        for (Iterator l = this.getFunctions(); l.hasNext();) {
+            Function function = (Function) l.next();
+            functions.add(function);
+        }
+
+        // make sure that the allThreads list is initialized;
+        this.initAllThreadsList();
+
+        int numThreads = allThreads.size();
+
+        Group derivedGroup = this.addGroup("TAU_CALLPATH_DERIVED");
+        reverseDataAvailable = true;
+
+        for (Iterator l = functions.iterator(); l.hasNext();) {
+            Function function = (Function) l.next();
+
+            if (function.isCallPathFunction() && CallPathUtilFuncs.containsDoublePath(function.getName())) {
+
+                String bonusName = function.getName();
+                bonusName = bonusName.substring(bonusName.indexOf("=>") + 2);
+                while (bonusName.indexOf("=>") != -1) {
+
+                    Function bonusFunction = this.getFunction(bonusName);
+                    if (bonusFunction == null) {
+                        bonusFunction = addFunction(bonusName);
+                        for (Iterator g = function.getGroups().iterator(); g.hasNext();) {
+                            bonusFunction.addGroup((Group) g.next());
+                        }
+                    }
+                    bonusFunction.addGroup(derivedGroup);
+                    bonusName = bonusName.substring(bonusName.indexOf("=>") + 2);
+
+                    for (int i = 0; i < numThreads; i++) {
+                        Thread thread = (Thread) allThreads.get(i);
+
+                        FunctionProfile functionProfile = thread.getFunctionProfile(function);
+                        if (functionProfile != null) {
+                            FunctionProfile bfp = thread.getFunctionProfile(bonusFunction);
+                            if (bfp == null) {
+                                bfp = new FunctionProfile(bonusFunction, getNumberOfMetrics());
+                                thread.addFunctionProfile(bfp);
+                            }
+
+                            for (int metric = 0; metric < this.getNumberOfMetrics(); metric++) {
+
+                                bfp.setExclusive(metric, bfp.getExclusive(metric) + functionProfile.getExclusive(metric));
+                                bfp.setInclusive(metric, bfp.getInclusive(metric) + functionProfile.getInclusive(metric));
+                            }
+                            bfp.setNumCalls(bfp.getNumCalls() + functionProfile.getNumCalls());
+                            bfp.setNumSubr(bfp.getNumSubr() + functionProfile.getNumSubr());
+
+                        }
+                    }
+                }
+            }
+        }
+
     }
 
     /*
@@ -355,6 +498,17 @@ public abstract class DataSource {
     public void generateDerivedData() {
         //long time = System.currentTimeMillis();
 
+	// reset the list of all threads so that it will be recreated
+        allThreads = null;
+
+        if (CallPathUtilFuncs.checkCallPathsPresent(getFunctions())) {
+            setCallPathDataPresent(true);
+        }
+
+        if (generateIntermediateCallPathData) {
+            generateBonusCallPathData();
+        }
+
         checkForPhases();
 
         for (Iterator it = this.getAllThreads().iterator(); it.hasNext();) {
@@ -366,7 +520,7 @@ public abstract class DataSource {
         this.stddevData.setThreadDataAllMetrics();
 
         this.generateUserEventStatistics();
-        
+
         finishPhaseAnalysis();
 
         //time = (System.currentTimeMillis()) - time;
@@ -374,17 +528,15 @@ public abstract class DataSource {
 
     }
 
-    
     private double computeStdDev(double sumSqr, double mean, int count) {
         double stdDev = 0;
         if (count > 1) {
-            stdDev = java.lang.Math.sqrt(java.lang.Math.abs((sumSqr / count)
-                    - (mean * mean)));
+            stdDev = java.lang.Math.sqrt(java.lang.Math.abs((sumSqr / count) - (mean * mean)));
         }
-        
+
         return stdDev;
     }
-    
+
     private void generateUserEventStatistics() {
 
         for (Iterator it = this.getUserEvents(); it.hasNext();) {
@@ -414,7 +566,6 @@ public abstract class DataSource {
             }
             //ue.setStddevProfile(stddevProfile);
 
-
             int numProfiles = 0;
             double numSampSum = 0;
             double numSampSumSqr = 0;
@@ -427,21 +578,22 @@ public abstract class DataSource {
             double stdDevSum = 0;
             double stdDevSumSqr = 0;
 
-            
+            // make sure that the allThreads list is initialized;
+            this.initAllThreadsList();
             int numThreads = allThreads.size();
 
             for (int i = 0; i < numThreads; i++) { // for each thread
-                edu.uoregon.tau.perfdmf.Thread thread = (edu.uoregon.tau.perfdmf.Thread) allThreads.get(i);
+                Thread thread = (Thread) allThreads.get(i);
                 UserEventProfile uep = thread.getUserEventProfile(ue);
                 if (uep != null) {
                     numProfiles++;
-                    
+
                     numSampSum += uep.getNumSamples();
                     numSampSumSqr += uep.getNumSamples() * uep.getNumSamples();
-                    
+
                     maxSum += uep.getMaxValue();
                     maxSumSqr += uep.getMaxValue() * uep.getMaxValue();
-                    
+
                     minSum += uep.getMinValue();
                     minSumSqr += uep.getMinValue() * uep.getMinValue();
 
@@ -457,19 +609,18 @@ public abstract class DataSource {
                 numThreads = numProfiles;
             }
 
-            
-            totalProfile.setNumSamples((int)numSampSum);
+            totalProfile.setNumSamples((int) numSampSum);
             totalProfile.setMaxValue(maxSum);
             totalProfile.setMinValue(minSum);
             totalProfile.setMeanValue(maxSum);
             totalProfile.setStdDev(stdDevSum);
-            
-            meanProfile.setNumSamples((int)(numSampSum / numThreads));
+
+            meanProfile.setNumSamples((int) (numSampSum / numThreads));
             meanProfile.setMaxValue(maxSum / numThreads);
             meanProfile.setMinValue(minSum / numThreads);
             meanProfile.setMeanValue(meanSum / numThreads);
             meanProfile.setStdDev(stdDevSum / numThreads);
-            
+
             stddevProfile.setNumSamples(computeStdDev(numSampSumSqr, meanProfile.getNumSamples(), numThreads));
             stddevProfile.setMaxValue(computeStdDev(maxSumSqr, meanProfile.getMaxValue(), numThreads));
             stddevProfile.setMinValue(computeStdDev(minSumSqr, meanProfile.getMinValue(), numThreads));
@@ -478,7 +629,7 @@ public abstract class DataSource {
         }
 
     }
-    
+
     public void generateStatistics(int startMetric, int endMetric) {
 
         /*
@@ -541,6 +692,9 @@ public abstract class DataSource {
                 Thread thread = (Thread) it.next();
                 topLevelInclSum[i] += thread.getMaxInclusive(i);
             }
+            totalData.setPercentDivider(i, topLevelInclSum[i] / 100.0);
+            meanData.setPercentDivider(i, topLevelInclSum[i] / 100.0);
+            stddevData.setPercentDivider(i, topLevelInclSum[i] / 100.0);
         }
 
         for (Iterator l = this.getFunctions(); l.hasNext();) { // for each function
@@ -585,7 +739,7 @@ public abstract class DataSource {
             int numThreads = allThreads.size();
 
             for (int i = 0; i < numThreads; i++) { // for each thread
-                edu.uoregon.tau.perfdmf.Thread thread = (edu.uoregon.tau.perfdmf.Thread) allThreads.get(i);
+                Thread thread = (Thread) allThreads.get(i);
                 FunctionProfile functionProfile = thread.getFunctionProfile(function);
 
                 if (functionProfile != null) { // only if this function was called for this nct
@@ -670,6 +824,7 @@ public abstract class DataSource {
                 if (topLevelInclSum[i] != 0) {
                     totalProfile.setInclusivePercent(i, totalProfile.getInclusive(i) / topLevelInclSum[i] * 100);
                     totalProfile.setExclusivePercent(i, totalProfile.getExclusive(i) / topLevelInclSum[i] * 100);
+
                     meanProfile.setInclusivePercent(i, totalProfile.getInclusivePercent(i));
                     meanProfile.setExclusivePercent(i, totalProfile.getExclusivePercent(i));
                     if (meanProfile.getInclusive(i) != 0) {
@@ -706,6 +861,15 @@ public abstract class DataSource {
         return node;
     }
 
+    public Thread addThread(int nodeID, int contextID, int threadID) {
+        Node node = addNode(nodeID);
+        Context context = node.addContext(contextID);
+        Thread thread = context.addThread(threadID, this.getNumberOfMetrics());
+        // reset the list of all threads so that it will be recreated
+        allThreads = null;
+        return thread;
+    }
+
     /**
      * Gets the node with the specified node id.  If the node is not found, the function returns null.
      *
@@ -792,7 +956,7 @@ public abstract class DataSource {
             for (Iterator it2 = node.getContexts(); it2.hasNext();) {
                 Context context = (Context) it2.next();
                 for (Iterator it3 = context.getThreads(); it3.hasNext();) {
-                    edu.uoregon.tau.perfdmf.Thread thread = (edu.uoregon.tau.perfdmf.Thread) it3.next();
+                    Thread thread = (Thread) it3.next();
                     list.add(thread);
                 }
             }
@@ -821,7 +985,7 @@ public abstract class DataSource {
             for (Iterator it2 = node.getContexts(); it2.hasNext();) {
                 Context context = (Context) it2.next();
                 for (Iterator it3 = context.getThreads(); it3.hasNext();) {
-                    edu.uoregon.tau.perfdmf.Thread thread = (edu.uoregon.tau.perfdmf.Thread) it3.next();
+                    Thread thread = (Thread) it3.next();
                     allThreads.add(thread);
                 }
             }
@@ -848,65 +1012,64 @@ public abstract class DataSource {
     }
 
     protected void checkForPhases() {
-        
+
         Group tau_phase = this.getGroup("TAU_PHASE");
 
         if (tau_phase != null) {
             phasesPresent = true;
-            
+
             for (Iterator it = this.getFunctions(); it.hasNext();) {
                 Function function = (Function) it.next();
-                
+
                 if (function.isGroupMember(tau_phase)) {
                     function.setPhase(true);
                     function.setActualPhase(function);
                 }
-            }   
+            }
 
             for (Iterator it = this.getFunctions(); it.hasNext();) {
                 Function function = (Function) it.next();
-                
-              int location = function.getName().indexOf("=>");
-              
-              if (location > 0) {
-                  // split "A => B"
-                  String phaseRoot = UtilFncs.getLeftSide(function.getName());
-                  String phaseChild = UtilFncs.getRightSide(function.getName());
 
-                  Function f = this.getFunction(phaseChild);
-                  if (f.isPhase()) {
-                      function.setPhase(true);
-                      function.setActualPhase(f);
-                  }
-                  
-                  function.setParentPhase(this.getFunction(phaseRoot));
-              }
-            }   
+                int location = function.getName().indexOf("=>");
+
+                if (location > 0) {
+                    // split "A => B"
+                    String phaseRoot = UtilFncs.getLeftSide(function.getName());
+                    String phaseChild = UtilFncs.getRightSide(function.getName());
+
+                    Function f = this.getFunction(phaseChild);
+                    if (f.isPhase()) {
+                        function.setPhase(true);
+                        function.setActualPhase(f);
+                    }
+
+                    function.setParentPhase(this.getFunction(phaseRoot));
+                }
+            }
 
-    
         }
     }
-    
+
     protected void finishPhaseAnalysis() {
-  
+
         if (phasesPresent) {
             Group tau_phase = this.getGroup("TAU_PHASE");
             ArrayList phases = new ArrayList();
-            
+
             for (Iterator it = this.getFunctions(); it.hasNext();) {
                 Function function = (Function) it.next();
                 if (function.isGroupMember(tau_phase)) {
                     phases.add(function);
                 }
-            }   
-            
+            }
+
             // there must be at least one
             if (phases.size() == 0) {
                 throw new RuntimeException("Error: TAU_PHASE found, but no phases!");
             }
-            
+
             // try to find the "top level phase", usually 'main'
-            topLevelPhase = (Function)phases.get(0);
+            topLevelPhase = (Function) phases.get(0);
             for (Iterator it = phases.iterator(); it.hasNext();) {
                 Function function = (Function) it.next();
                 if (function.getMeanInclusive(0) > topLevelPhase.getMeanInclusive(0)) {
@@ -929,4 +1092,73 @@ public abstract class DataSource {
         return topLevelPhase;
     }
 
-}
\ No newline at end of file
+    public boolean getGenerateIntermediateCallPathData() {
+        return generateIntermediateCallPathData;
+    }
+
+    public void setGenerateIntermediateCallPathData(boolean generateIntermediateCallPathData) {
+        this.generateIntermediateCallPathData = generateIntermediateCallPathData;
+    }
+
+    public boolean getReverseDataAvailable() {
+        return reverseDataAvailable;
+    }
+
+    public Map getMetaData() {
+        return metaData;
+    }
+
+    public void setMetaData(Map metaData) {
+        this.metaData = metaData;
+    }
+
+    public void aggregateMetaData() {
+
+        metaData = new TreeMap();
+        
+        Thread node0 = (Thread) getAllThreads().get(0);
+
+        // must have at least one thread
+        if (node0 != null) {
+
+            for (Iterator it = node0.getMetaData().keySet().iterator(); it.hasNext();) {
+                String name = (String) it.next();
+                String value = (String) node0.getMetaData().get(name);
+                metaData.put(name, value);
+            }
+
+            for (Iterator it = getAllThreads().iterator(); it.hasNext();) {
+                Thread thread = (Thread) it.next();
+                for (Iterator it2 = thread.getMetaData().keySet().iterator(); it2.hasNext();) {
+                    String name = (String) it2.next();
+                    String value = (String) thread.getMetaData().get(name);
+
+                    String trialValue = (String) metaData.get(name);
+                    if (trialValue == null || !value.equals(trialValue)) {
+                        metaData.remove(name);
+                    }
+                }
+            }
+
+            for (Iterator it = getAllThreads().iterator(); it.hasNext();) {
+                Thread thread = (Thread) it.next();
+
+                for (Iterator it2 = metaData.keySet().iterator(); it2.hasNext();) {
+                    String name = (String) it2.next();
+                    thread.getMetaData().remove(name);
+                }
+            }
+
+            // now add node0's different values
+
+//            for (Iterator it = node0.getMetaData().keySet().iterator(); it.hasNext();) {
+//                String name = (String) it.next();
+//                String value = (String) node0.getMetaData().get(name);
+//                if (metaData.get(name) == null) {
+//                    metaData.put(name + " (node 0)", value);
+//                }
+//            }
+        }
+    }
+
+}
