@@ -9,11 +9,11 @@ import java.sql.*;
  * This is the top level class for the Database API.
  * 
  * <P>
- * CVS $Id: DatabaseAPI.java,v 1.2 2005/10/18 22:48:55 amorris Exp $
+ * CVS $Id: DatabaseAPI.java,v 1.11 2006/12/28 03:05:59 amorris Exp $
  * </P>
  * 
  * @author Kevin Huck, Robert Bell
- * @version $Revision: 1.2 $
+ * @version $Revision: 1.11 $
  */
 public class DatabaseAPI {
 
@@ -90,6 +90,15 @@ public class DatabaseAPI {
         Trial.getMetaData(db);
     }
 
+    public void initialize(ParseConfig parser, String password) throws SQLException {
+        connector = new ConnectionManager(parser, password);
+        connector.connect();
+        db = connector.getDB();
+        Application.getMetaData(db);
+        Experiment.getMetaData(db);
+        Trial.getMetaData(db);
+    }
+
     public void initialize(Object obj, String password) throws SQLException {
         String configFileName = (String) (obj);
         //Initialize the connection to the database,
@@ -111,7 +120,7 @@ public class DatabaseAPI {
     }
 
     // returns Vector of ALL Application objects
-    public List getApplicationList() {
+    public List getApplicationList() throws DatabaseException {
         String whereClause = "";
         return Application.getApplicationList(db, whereClause);
     }
@@ -518,7 +527,11 @@ public class DatabaseAPI {
     }
 
     public int saveApplication(Application app) {
-        return app.saveApplication(db);
+        try {
+            return app.saveApplication(db);
+        } catch (SQLException e) {
+            throw new DatabaseException("Error saving application", e);
+        }
     }
 
     public int saveExperiment(Experiment exp) throws DatabaseException {
@@ -555,6 +568,8 @@ public class DatabaseAPI {
             tmpStr = "select LAST_INSERT_ID();";
         else if (db.getDBType().compareTo("db2") == 0)
             tmpStr = "select IDENTITY_VAL_LOCAL() FROM metric";
+        else if (db.getDBType().compareTo("derby") == 0)
+            tmpStr = "select IDENTITY_VAL_LOCAL() FROM metric";
         else if (db.getDBType().compareTo("oracle") == 0)
             tmpStr = "select " + db.getSchemaPrefix() + "metric_id_seq.currval FROM dual";
         else
@@ -720,10 +735,12 @@ public class DatabaseAPI {
         int fcount = 0;
         int ucount = 0;
 
+        Group derived = dataSource.getGroup("TAU_CALLPATH_DERIVED");
+
         // create the intervalEvents
         for (Iterator it = dataSource.getFunctions(); it.hasNext();) {
             Function f = (Function) it.next();
-            if (f != null) {
+            if (!f.isGroupMember(derived)) {
                 // create a intervalEvent
                 IntervalEvent intervalEvent = new IntervalEvent(this);
                 intervalEvent.setName(f.getName());
@@ -748,7 +765,7 @@ public class DatabaseAPI {
                 intervalEvents.add(intervalEvent);
 
                 int numThreads = trial.getDataSource().getAllThreads().size();
-                
+
                 IntervalLocationProfile ilpTotal = new IntervalLocationProfile(metricCount);
                 IntervalLocationProfile ilpMean = new IntervalLocationProfile(metricCount);
                 for (int i = 0; i < metricCount; i++) {
@@ -760,11 +777,11 @@ public class DatabaseAPI {
                     ilpTotal.setExclusive(i, f.getTotalExclusive(i));
                     ilpTotal.setInclusivePerCall(i, f.getTotalInclusivePerCall(i));
                     ilpMean.setNumCalls(f.getTotalNumCalls() / numThreads);
-                    ilpMean.setNumSubroutines(f.getTotalNumSubr()/ numThreads);
+                    ilpMean.setNumSubroutines(f.getTotalNumSubr() / numThreads);
                     ilpMean.setInclusivePercentage(i, f.getTotalInclusivePercent(i));
-                    ilpMean.setInclusive(i, f.getTotalInclusive(i)/ numThreads);
+                    ilpMean.setInclusive(i, f.getTotalInclusive(i) / numThreads);
                     ilpMean.setExclusivePercentage(i, f.getTotalExclusivePercent(i));
-                    ilpMean.setExclusive(i, f.getTotalExclusive(i)/ numThreads);
+                    ilpMean.setExclusive(i, f.getTotalExclusive(i) / numThreads);
                     ilpMean.setInclusivePerCall(i, f.getTotalInclusivePerCall(i));
                 }
                 intervalEvent.setTotalSummary(ilpTotal);
@@ -791,7 +808,7 @@ public class DatabaseAPI {
             // create interval location profiles
             for (Iterator e4 = intervalEvents.iterator(); e4.hasNext();) {
                 FunctionProfile fp = (FunctionProfile) e4.next();
-                if (fp != null) {
+                if (fp != null && !fp.getFunction().isGroupMember(derived)) {
                     IntervalLocationProfile ilp = new IntervalLocationProfile(metricCount);
                     ilp.setNode(thread.getNodeID());
                     ilp.setContext(thread.getContextID());
@@ -821,7 +838,7 @@ public class DatabaseAPI {
                         udo.setNode(thread.getNodeID());
                         udo.setContext(thread.getContextID());
                         udo.setThread(thread.getThreadID());
-                        udo.setSampleCount((int)uep.getNumSamples());
+                        udo.setSampleCount((int) uep.getNumSamples());
                         udo.setMaximumValue(uep.getMaxValue());
                         udo.setMinimumValue(uep.getMinValue());
                         udo.setMeanValue(uep.getMeanValue());
@@ -855,8 +872,7 @@ public class DatabaseAPI {
                 if (intervalEvents != null && intervalEvents.size() > 0) {
                     Hashtable functionHash = saveIntervalEvents(newTrialID, metricHash, saveMetricIndex);
 
-                    saveIntervalLocationProfiles(db, functionHash, intervalEventData.elements(), metricHash,
-                            saveMetricIndex);
+                    saveIntervalLocationProfiles(db, functionHash, intervalEventData.elements(), metricHash, saveMetricIndex);
                 }
                 if (atomicEvents != null && atomicEvents.size() > 0) {
                     Hashtable atomicEventHash = saveAtomicEvents(newTrialID);
@@ -873,8 +889,7 @@ public class DatabaseAPI {
 
                 if (intervalEvents != null && intervalEvents.size() > 0) {
                     Hashtable newFunHash = saveIntervalEvents(newTrialID, newMetHash, saveMetricIndex);
-                    saveIntervalLocationProfiles(db, newFunHash, intervalEventData.elements(), newMetHash,
-                            saveMetricIndex);
+                    saveIntervalLocationProfiles(db, newFunHash, intervalEventData.elements(), newMetHash, saveMetricIndex);
 
                 }
 
@@ -884,9 +899,10 @@ public class DatabaseAPI {
         } catch (SQLException e) {
             try {
                 db.rollback();
-                throw new DatabaseException("Saving Trial Failed, rollback successful", e);
+                e.printStackTrace();
+                throw new DatabaseException("Saving Trial Failed, rollbacks successful", e);
             } catch (SQLException e2) {
-                throw new DatabaseException("Saving Trial Failed, rollback failed!", e2);
+                throw new DatabaseException("Saving Trial Failed, rollbacks failed!", e2);
             }
 
         }
@@ -923,6 +939,8 @@ public class DatabaseAPI {
                 tmpStr = "select LAST_INSERT_ID();";
             else if (db.getDBType().compareTo("db2") == 0)
                 tmpStr = "select IDENTITY_VAL_LOCAL() FROM metric";
+            else if (db.getDBType().compareTo("derby") == 0)
+                tmpStr = "select IDENTITY_VAL_LOCAL() FROM metric";
             else if (db.getDBType().compareTo("oracle") == 0)
                 tmpStr = "select " + db.getSchemaPrefix() + "metric_id_seq.currval FROM dual";
             else
@@ -937,8 +955,12 @@ public class DatabaseAPI {
     private Map uploadFunctions(int trialID, DataSource dataSource) throws SQLException {
         Map map = new HashMap();
 
+        Group derived = dataSource.getGroup("TAU_CALLPATH_DERIVED");
         for (Iterator it = dataSource.getFunctions(); it.hasNext();) {
             Function f = (Function) it.next();
+            if (f.isGroupMember(derived)) {
+                continue;
+            }
 
             String group = null;
             List groups = f.getGroups();
@@ -966,6 +988,8 @@ public class DatabaseAPI {
                 tmpStr = "select LAST_INSERT_ID();";
             else if (db.getDBType().compareTo("db2") == 0)
                 tmpStr = "select IDENTITY_VAL_LOCAL() FROM interval_event";
+            else if (db.getDBType().compareTo("derby") == 0)
+                tmpStr = "select IDENTITY_VAL_LOCAL() FROM interval_event";
             else if (db.getDBType().compareTo("oracle") == 0)
                 tmpStr = "select " + db.getSchemaPrefix() + "interval_event_id_seq.currval FROM dual";
             else
@@ -1002,6 +1026,8 @@ public class DatabaseAPI {
                 tmpStr = "select LAST_INSERT_ID();";
             else if (db.getDBType().compareTo("db2") == 0)
                 tmpStr = "select IDENTITY_VAL_LOCAL() FROM atomic_event";
+            else if (db.getDBType().compareTo("derby") == 0)
+                tmpStr = "select IDENTITY_VAL_LOCAL() FROM atomic_event";
             else if (db.getDBType().compareTo("oracle") == 0)
                 tmpStr = "select " + db.getSchemaPrefix() + "atomic_event_id_seq.currval FROM dual";
             else
@@ -1014,8 +1040,8 @@ public class DatabaseAPI {
         return map;
     }
 
-    private void addBatchFunctionProfile(PreparedStatement stmt, Thread thread, int metricID, int dbMetricID,
-            FunctionProfile fp, int intervalEventID, boolean createMean, int numThreads) throws SQLException {
+    private void addBatchFunctionProfile(PreparedStatement stmt, Thread thread, int metricID, int dbMetricID, FunctionProfile fp,
+            int intervalEventID, boolean createMean, int numThreads) throws SQLException {
 
         stmt.setInt(1, intervalEventID);
         stmt.setInt(2, dbMetricID);
@@ -1048,16 +1074,20 @@ public class DatabaseAPI {
         try {
             stmt.executeUpdate();
         } catch (Exception e) {
+            e.printStackTrace();
+            System.out.println(e);
+            System.out.println(stmt.toString());
             System.out.println("exclusive: " + fp.getExclusive(metricID));
+            System.out.println("inclusive: " + fp.getInclusive(metricID));
+            System.out.println("numThreads: " + numThreads);
             System.out.println("numcalls: " + fp.getNumCalls());
             System.out.println("numsubr: " + fp.getNumSubr());
             System.out.println("inclusivepercall: " + fp.getInclusivePerCall(metricID));
-            System.out.println("asdf");
+            System.exit(9);
         }
     }
 
-    private void uploadFunctionProfiles(int trialID, DataSource dataSource, Map functionMap, Map metricMap)
-            throws SQLException {
+    private void uploadFunctionProfiles(int trialID, DataSource dataSource, Map functionMap, Map metricMap) throws SQLException {
 
         PreparedStatement totalInsertStatement = null;
         PreparedStatement meanInsertStatement = null;
@@ -1073,6 +1103,26 @@ public class DatabaseAPI {
             threadInsertStatement = db.prepareStatement("INSERT INTO "
                     + db.getSchemaPrefix()
                     + "interval_location_profile (interval_event, metric, inclusive_percentage, inclusive, exclusive_percentage, excl, call, subroutines, inclusive_per_call, node, context, thread) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
+        } else if (db.getDBType().compareTo("derby") == 0) {
+            totalInsertStatement = db.prepareStatement("INSERT INTO "
+                    + db.getSchemaPrefix()
+                    + "interval_total_summary (interval_event, metric, inclusive_percentage, inclusive, exclusive_percentage, exclusive, num_calls, subroutines, inclusive_per_call) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)");
+            meanInsertStatement = db.prepareStatement("INSERT INTO "
+                    + db.getSchemaPrefix()
+                    + "interval_mean_summary (interval_event, metric, inclusive_percentage, inclusive, exclusive_percentage, exclusive, num_calls, subroutines, inclusive_per_call) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)");
+            threadInsertStatement = db.prepareStatement("INSERT INTO "
+                    + db.getSchemaPrefix()
+                    + "interval_location_profile (interval_event, metric, inclusive_percentage, inclusive, exclusive_percentage, exclusive, num_calls, subroutines, inclusive_per_call, node, context, thread) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
+        } else if (db.getDBType().compareTo("mysql") == 0) {
+            totalInsertStatement = db.prepareStatement("INSERT INTO "
+                    + db.getSchemaPrefix()
+                    + "interval_total_summary (interval_event, metric, inclusive_percentage, inclusive, exclusive_percentage, exclusive, `call`, subroutines, inclusive_per_call) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)");
+            meanInsertStatement = db.prepareStatement("INSERT INTO "
+                    + db.getSchemaPrefix()
+                    + "interval_mean_summary (interval_event, metric, inclusive_percentage, inclusive, exclusive_percentage, exclusive, `call`, subroutines, inclusive_per_call) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)");
+            threadInsertStatement = db.prepareStatement("INSERT INTO "
+                    + db.getSchemaPrefix()
+                    + "interval_location_profile (interval_event, metric, inclusive_percentage, inclusive, exclusive_percentage, exclusive, `call`, subroutines, inclusive_per_call, node, context, thread) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
         } else {
             totalInsertStatement = db.prepareStatement("INSERT INTO "
                     + db.getSchemaPrefix()
@@ -1085,18 +1135,22 @@ public class DatabaseAPI {
                     + "interval_location_profile (interval_event, metric, inclusive_percentage, inclusive, exclusive_percentage, exclusive, call, subroutines, inclusive_per_call, node, context, thread) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
         }
 
+        Group derived = dataSource.getGroup("TAU_CALLPATH_DERIVED");
+
         for (Iterator it5 = dataSource.getMetrics().iterator(); it5.hasNext();) {
             Metric metric = (Metric) it5.next();
             Integer dbMetricID = (Integer) metricMap.get(metric);
 
             for (Iterator it4 = dataSource.getFunctions(); it4.hasNext();) {
                 Function function = (Function) it4.next();
+                if (function.isGroupMember(derived)) {
+                    continue;
+                }
                 Integer intervalEventID = (Integer) functionMap.get(function);
 
                 edu.uoregon.tau.perfdmf.Thread totalData = dataSource.getTotalData();
                 addBatchFunctionProfile(totalInsertStatement, totalData, metric.getID(), dbMetricID.intValue(),
-                        function.getTotalProfile(), intervalEventID.intValue(), false,
-                        dataSource.getAllThreads().size());
+                        function.getTotalProfile(), intervalEventID.intValue(), false, dataSource.getAllThreads().size());
 
                 //edu.uoregon.tau.dms.dss.Thread meanData = dataSource.getMeanData();
                 //addBatchFunctionProfile(meanInsertStatement, meanData, metric.getID(), dbMetricID.intValue(),
@@ -1114,8 +1168,8 @@ public class DatabaseAPI {
                         if (this.cancelUpload)
                             return;
 
-                        addBatchFunctionProfile(threadInsertStatement, thread, metric.getID(), dbMetricID.intValue(),
-                                fp, intervalEventID.intValue(), false, dataSource.getAllThreads().size());
+                        addBatchFunctionProfile(threadInsertStatement, thread, metric.getID(), dbMetricID.intValue(), fp,
+                                intervalEventID.intValue(), false, dataSource.getAllThreads().size());
                     }
                 }
             }
@@ -1157,7 +1211,7 @@ public class DatabaseAPI {
                             statement.setInt(2, thread.getNodeID());
                             statement.setInt(3, thread.getContextID());
                             statement.setInt(4, thread.getThreadID());
-                            statement.setInt(5, (int)uep.getNumSamples());
+                            statement.setInt(5, (int) uep.getNumSamples());
                             statement.setDouble(6, uep.getMaxValue());
                             statement.setDouble(7, uep.getMinValue());
                             statement.setDouble(8, uep.getMeanValue());
@@ -1237,9 +1291,10 @@ public class DatabaseAPI {
         } catch (SQLException e) {
             try {
                 db.rollback();
-                throw new DatabaseException("Saving Trial Failed, rollback successful", e);
+                e.printStackTrace();
+                throw new DatabaseException("Saving Trial Failed, rollbacks successful", e);
             } catch (SQLException e2) {
-                throw new DatabaseException("Saving Trial Failed, rollback failed!", e2);
+                throw new DatabaseException("Saving Trial Failed, rollbacks failed!", e2);
             }
 
         }
@@ -1260,16 +1315,24 @@ public class DatabaseAPI {
 
     public int saveApplication() {
         int appid = 0;
-        if (application != null) {
-            appid = application.saveApplication(db);
+        try {
+            if (application != null) {
+                appid = application.saveApplication(db);
+            }
+        } catch (SQLException e) {
+            throw new DatabaseException("Error saving application", e);
         }
         return appid;
     }
 
-    public int saveExperiment() throws SQLException {
+    public int saveExperiment() {
         int expid = 0;
-        if (experiment != null) {
-            expid = experiment.saveExperiment(db);
+        try {
+            if (experiment != null) {
+                expid = experiment.saveExperiment(db);
+            }
+        } catch (SQLException e) {
+            throw new DatabaseException("Error saving experiment", e);
         }
         return expid;
     }
@@ -1355,6 +1418,14 @@ public class DatabaseAPI {
             statement = db.prepareStatement("INSERT INTO "
                     + db.getSchemaPrefix()
                     + "interval_location_profile (interval_event, node, context, thread, metric, inclusive_percentage, inclusive, exclusive_percentage, excl, call, subroutines, inclusive_per_call) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
+        } else if (db.getDBType().compareTo("derby") == 0) {
+            statement = db.prepareStatement("INSERT INTO "
+                    + db.getSchemaPrefix()
+                    + "interval_location_profile (interval_event, node, context, thread, metric, inclusive_percentage, inclusive, exclusive_percentage, exclusive, num_calls, subroutines, inclusive_per_call) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
+        } else if (db.getDBType().compareTo("derby") == 0) {
+            statement = db.prepareStatement("INSERT INTO "
+                    + db.getSchemaPrefix()
+                    + "interval_location_profile (interval_event, node, context, thread, metric, inclusive_percentage, inclusive, exclusive_percentage, exclusive, `call`, subroutines, inclusive_per_call) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
         } else {
             statement = db.prepareStatement("INSERT INTO "
                     + db.getSchemaPrefix()
@@ -1393,4 +1464,62 @@ public class DatabaseAPI {
         statement.close();
     }
 
+    public Application getApplication(String name, boolean create) {
+        List apps = getApplicationList();
+        for (Iterator it = apps.iterator(); it.hasNext();) {
+            Application app = (Application) it.next();
+            if (app.getName().equals(name)) {
+                return app;
+            }
+        }
+        // didn't find one with that name
+        if (create) {
+            Application newApp = new Application();
+            newApp.setName(name);
+            setApplication(newApp);
+            int appId = saveApplication();
+            newApp.setID(appId);
+            return newApp;
+        }
+        return null;
+    }
+
+    public Experiment getExperiment(Application app, String name, boolean create) {
+        setApplication(app);
+        List exps = getExperimentList();
+        for (Iterator it = exps.iterator(); it.hasNext();) {
+            Experiment exp = (Experiment) it.next();
+            if (exp.getName().equals(name)) {
+                return exp;
+            }
+        }
+
+        if (create) {
+            Experiment newExp = new Experiment();
+            newExp.setName(name);
+            newExp.setApplicationID(app.getID());
+            setExperiment(newExp);
+            int expId = saveExperiment();
+            newExp.setID(expId);
+            return newExp;
+        }
+
+        return null;
+    }
+
+    public Experiment getExperiment(String appName, String expName, boolean create) {
+        Application app = getApplication(appName, create);
+        if (app == null) {
+            return null;
+        }
+        return getExperiment(app, expName, create);
+    }
+
+    public DB getDb() {
+        return db;
+    }
+
+    public Trial getTrial() {
+        return trial;
+    }
 };
