@@ -9,9 +9,9 @@ import java.util.List;
  * This class represents a "function".  A function is defined over all threads
  * in the profile, so per-thread data is not stored here.
  *  
- * <P>CVS $Id: Function.java,v 1.1 2005/09/26 20:24:28 amorris Exp $</P>
+ * <P>CVS $Id: Function.java,v 1.14 2006/11/08 23:17:16 amorris Exp $</P>
  * @author	Robert Bell, Alan Morris
- * @version	$Revision: 1.1 $
+ * @version	$Revision: 1.14 $
  * @see		FunctionProfile
  */
 /**
@@ -24,7 +24,7 @@ public class Function implements Serializable, Comparable {
     private String name = null;
     private String reversedName = null;
     private int id = -1;
-    private List groups = null;
+    private List groups = new ArrayList();
     private boolean phase = false;
     private Function actualPhase;
     private Function parentPhase;
@@ -43,6 +43,9 @@ public class Function implements Serializable, Comparable {
     private Color color = null;
     private Color specificColor = null;
 
+    // source code link
+    private SourceRegion sourceLink;
+
     public Function(String name, int id, int numMetrics) {
         this.name = name;
         this.id = id;
@@ -56,6 +59,10 @@ public class Function implements Serializable, Comparable {
         return name;
     }
 
+    public void setName(String name) {
+        this.name = name;
+    }
+
     /**
      * Retrieve the reversed (callpath) name
      * If the function's name is "A => B => C", this will return "C <= B <= A"
@@ -88,27 +95,112 @@ public class Function implements Serializable, Comparable {
             }
         }
 
-        return reversedName;
+        return reversedName.trim();
     }
 
     public String toString() {
         return name;
     }
 
+    public SourceRegion getSourceLink() {
+        if (sourceLink == null) {
+            sourceLink = new SourceRegion();
+
+            if (name.indexOf("file:") != -1 && name.indexOf("line:") != -1) {
+                // MpiP source information
+                String name = this.name;
+
+                // start with the last section of location information
+                // we may have: "main [file:ring.c line:37] =>  func [file:ring.c line:19] => MPI_Recv"
+                // and we want the last section (for now)
+                name = name.substring(name.lastIndexOf("["));
+                int fileIndex = name.indexOf("file:");
+                int lineIndex = name.indexOf("line:");
+                
+                String filename = name.substring(fileIndex+5, lineIndex).trim();
+                sourceLink.setFilename(filename);
+                int lineNumber;
+                if (name.indexOf("]",lineIndex) != -1) {
+                    // new mpiP
+                    lineNumber = Integer.parseInt(name.substring(lineIndex+5,name.indexOf("]")));
+                } else {
+                    // old mpiP
+                    lineNumber = Integer.parseInt(name.substring(lineIndex+5).trim());
+                }
+                sourceLink.setStartLine(lineNumber);
+                sourceLink.setEndLine(lineNumber);
+                return sourceLink;
+            }
+
+            // for TAU, look at the leaf location information
+            String name = this.name;
+            if (isCallPathFunction()) {
+                name = name.substring(name.lastIndexOf("=>")+2);
+            }
+
+            int filenameStart = name.indexOf("[{");
+            if (filenameStart == -1) {
+                return sourceLink;
+            }
+            int filenameEnd = name.indexOf("}", filenameStart);
+            if (filenameEnd == -1) {
+                // quit, it's not valid
+                return sourceLink;
+            }
+
+            int openbracket1 = name.indexOf("{", filenameEnd + 1);
+            int comma1 = name.indexOf(",", filenameEnd + 1);
+            int closebracket1 = name.indexOf("}", filenameEnd + 1);
+            int dash = name.indexOf("-", closebracket1 + 1);
+            int openbracket2 = name.indexOf("{", openbracket1 + 1);
+            int comma2 = name.indexOf(",", comma1 + 1);
+            int closebracket2 = name.indexOf("}", closebracket1 + 1);
+
+            sourceLink.setFilename(name.substring(filenameStart + 2, filenameEnd));
+
+            if (openbracket1 == -1) {
+                return sourceLink;
+            }
+
+            if (dash == -1) {
+                // fortran (e.g. "foo [{foo.cpp} {1,1}]")
+                if (closebracket1 == -1 || comma1 == -1) {
+                    return sourceLink;
+                }
+                int linenumber = Integer.parseInt(name.substring(openbracket1 + 1, comma1));
+                sourceLink.setStartLine(linenumber);
+                sourceLink.setEndLine(linenumber);
+                return sourceLink;
+            } else {
+                // loop or c/c++ (e.g. "foo [{foo.cpp} {1,1}-{5,5}]")
+                if (openbracket1 == -1 || openbracket2 == -1 || comma1 == -1 || comma2 == -1 || closebracket1 == -1
+                        || closebracket2 == -1) {
+                    return sourceLink;
+                }
+                int startLine = Integer.parseInt(name.substring(openbracket1 + 1, comma1));
+                int startColumn = Integer.parseInt(name.substring(comma1 + 1, closebracket1));
+                int endLine = Integer.parseInt(name.substring(openbracket2 + 1, comma2));
+                int endColumn = Integer.parseInt(name.substring(comma2 + 1, closebracket2));
+
+                sourceLink.setStartLine(startLine);
+                sourceLink.setStartColumn(startColumn);
+                sourceLink.setEndLine(endLine);
+                sourceLink.setEndColumn(endColumn);
+            }
+
+        }
+        return sourceLink;
+    }
+
     // Group section
     public void addGroup(Group group) {
         //Don't add group if already a member.
         if (this.isGroupMember(group))
             return;
-        if (groups == null)
-            groups = new ArrayList();
-
         groups.add(group);
     }
 
     public boolean isGroupMember(Group group) {
-        if (groups == null)
-            return false;
         return groups.contains(group);
     }
 
@@ -116,7 +208,25 @@ public class Function implements Serializable, Comparable {
         return groups;
     }
 
+    public String getGroupString() {
+        String groupString = "";
+        for (int i = 0; i < groups.size(); i++) {
+            Group group = (Group) groups.get(i);
+            if (i == 0) {
+                groupString = group.getName();
+            } else {
+                groupString = groupString + " | " + group.getName();
+            }
+        }
+        return groupString;
+    }
+
     public boolean isPhaseMember(Function phase) {
+
+        if (phase == this) {
+            return true;
+        }
+
         if (phase == null) {
             return true;
         }
@@ -135,7 +245,6 @@ public class Function implements Serializable, Comparable {
         return false;
     }
 
-
     public boolean isCallPathFunction() {
         if (!callpathFunctionSet) {
             if (name.indexOf("=>") > 0) {
@@ -146,61 +255,59 @@ public class Function implements Serializable, Comparable {
         return callpathFunction;
     }
 
-//    public boolean isPhase() {
-//        if (!phaseSet) {
-//            for (int i = 0; i < groups.size(); i++) {
-//                if (((Group) groups.get(i)).getName().compareTo("TAU_PHASE") == 0) {
-//                    phase = true;
-//                }
-//            }
-//            phaseSet = true;
-//        }
-//        return phase;
-//    }
+    //    public boolean isPhase() {
+    //        if (!phaseSet) {
+    //            for (int i = 0; i < groups.size(); i++) {
+    //                if (((Group) groups.get(i)).getName().compareTo("TAU_PHASE") == 0) {
+    //                    phase = true;
+    //                }
+    //            }
+    //            phaseSet = true;
+    //        }
+    //        return phase;
+    //    }
+
+    //    private String getRightSide() {
+    //        if (!getCallPathFunction()) {
+    //            return null;
+    //        } 
+    //
+    //        
+    //        int location = name.indexOf("=>");
+    //        String phaseRoot = name.substring(0, location).trim();
+    //        String phaseChild = name.substring(location).trim();
+    //        
+    //        return phaseChild;
+    //    }
+    //    
+    //    public boolean isPhase() {
+    //        if (!phaseSet) {
+    //
+    //            if (name.indexOf("=>") > 0) {
+    //                callpathFunction = true;
+    //            }
+    //
+    //            for (int i = 0; i < groups.size(); i++) {
+    //                if (((Group) groups.get(i)).getName().compareTo("TAU_PHASE") == 0) {
+    //                    phase = true;
+    //                }
+    //            }
+    //            phaseSet = true;
+    //        }
+    //        return phase;
+    //    }
 
-    
-//    private String getRightSide() {
-//        if (!getCallPathFunction()) {
-//            return null;
-//        } 
-//
-//        
-//        int location = name.indexOf("=>");
-//        String phaseRoot = name.substring(0, location).trim();
-//        String phaseChild = name.substring(location).trim();
-//        
-//        return phaseChild;
-//    }
-//    
-//    public boolean isPhase() {
-//        if (!phaseSet) {
-//
-//            if (name.indexOf("=>") > 0) {
-//                callpathFunction = true;
-//            }
-//
-//            for (int i = 0; i < groups.size(); i++) {
-//                if (((Group) groups.get(i)).getName().compareTo("TAU_PHASE") == 0) {
-//                    phase = true;
-//                }
-//            }
-//            phaseSet = true;
-//        }
-//        return phase;
-//    }
-    
-    
-    
     // color section
     public void setColor(Color color) {
         this.color = color;
     }
 
     public Color getColor() {
-        if (colorFlag)
+        if (colorFlag) {
             return specificColor;
-        else
+        } else {
             return color;
+        }
     }
 
     public void setColorFlag(boolean colorFlag) {
