@@ -1,17 +1,19 @@
 package edu.uoregon.tau.perfdmf;
 
 import java.io.*;
+import java.net.URL;
 import java.sql.SQLException;
 import java.util.zip.GZIPInputStream;
+import java.security.Security;
 
 /**
  * Reads the ParaProf Packed Format (.ppk)
  *    
  * TODO : nothing, this class is complete
  *
- * <P>CVS $Id: PackedProfileDataSource.java,v 1.2 2005/10/18 22:48:55 amorris Exp $</P>
+ * <P>CVS $Id: PackedProfileDataSource.java,v 1.9 2007/02/13 18:44:37 amorris Exp $</P>
  * @author  Alan Morris
- * @version $Revision: 1.2 $
+ * @version $Revision: 1.9 $
  */
 public class PackedProfileDataSource extends DataSource {
 
@@ -45,8 +47,9 @@ public class PackedProfileDataSource extends DataSource {
      * @return          progress
      */
     public int getProgress() {
-        if (totalBytes != 0)
+        if (totalBytes != 0) {
             return (int) ((float) tracker.byteCount() / (float) totalBytes * 100);
+        }
         return 0;
     }
 
@@ -58,8 +61,15 @@ public class PackedProfileDataSource extends DataSource {
     public void load() throws FileNotFoundException, IOException, DataSourceException, SQLException {
         long time = System.currentTimeMillis();
 
-        // set up the streams
-        FileInputStream istream = new FileInputStream(file);
+        InputStream istream;
+        if (file.toString().toLowerCase().startsWith("http:/")) {
+            // When it gets converted from a String to a File http:// turns into http:/
+            URL url = new URL("http://" + file.toString().substring(6));
+            istream = url.openStream();
+        }  else {
+            istream = new FileInputStream(file);
+        }
+        
         tracker = new TrackerInputStream(istream);
         GZIPInputStream gzip = new GZIPInputStream(tracker);
         BufferedInputStream bis = new BufferedInputStream(gzip);
@@ -138,22 +148,20 @@ public class PackedProfileDataSource extends DataSource {
             int contextID = p.readInt();
             int threadID = p.readInt();
 
-            Node node = this.addNode(nodeID);
-            Context context = node.addContext(contextID);
-            Thread thread = context.addThread(threadID, numMetrics);
+            Thread thread = addThread(nodeID, contextID, threadID);
 
             // get function profiles
             int numFunctionProfiles = p.readInt();
             for (int j = 0; j < numFunctionProfiles; j++) {
                 int functionID = p.readInt();
                 FunctionProfile fp = new FunctionProfile(functions[functionID], numMetrics);
+                thread.addFunctionProfile(fp);
                 fp.setNumCalls(p.readDouble());
                 fp.setNumSubr(p.readDouble());
                 for (int k = 0; k < numMetrics; k++) {
                     fp.setExclusive(k, p.readDouble());
                     fp.setInclusive(k, p.readDouble());
                 }
-                thread.addFunctionProfile(fp);
             }
 
             // get user event profiles
@@ -182,13 +190,9 @@ public class PackedProfileDataSource extends DataSource {
             setUserEventsPresent(true);
         }
 
-
+        
         this.generateDerivedData(); // mean, percentages, etc.
 
-        if (CallPathUtilFuncs.checkCallPathsPresent(this.getFunctions())) {
-            setCallPathDataPresent(true);
-        }
-
         //time = (System.currentTimeMillis()) - time;
         //System.out.println("Time to process (in milliseconds): " + time);
 
