@@ -1,7 +1,9 @@
 /*
  * TauOutputSession.java
  * 
- * Title: ParaProf Author: Robert Bell Description:
+ * Title: ParaProf 
+ * Author: Robert Bell 
+ * Description:
  */
 
 /*
@@ -25,8 +27,6 @@ public class TauDataSource extends DataSource {
     private volatile boolean abort = false;
     private volatile int totalFiles = 0;
     private volatile int filesRead = 0;
-    private LineData functionDataLine = new LineData();
-    private LineData usereventDataLine = new LineData();
     private boolean profileStatsPresent = false;
     private boolean groupCheck = false;
     private List dirs; // list of directories (e.g. MULTI__PAPI_FP_INS, MULTI__PAPI_L1_DCM)
@@ -92,11 +92,6 @@ public class TauDataSource extends DataSource {
 
         boolean metricNameProcessed = false;
 
-        Function func = null;
-        FunctionProfile functionProfile = null;
-
-        UserEventProfile userEventProfile = null;
-
         int nodeID = -1;
         int contextID = -1;
         int threadID = -1;
@@ -113,26 +108,6 @@ public class TauDataSource extends DataSource {
             //Reset metricNameProcessed flag.
             metricNameProcessed = false;
 
-            //Only need to call addDefaultToVectors() if not the first run.
-            if (metric != 0) { // If this isn't the first metric, call incrementStorage
-                for (Iterator it = this.getNodes(); it.hasNext();) {
-                    Node node = (Node) it.next();
-                    for (Iterator it2 = node.getContexts(); it2.hasNext();) {
-                        Context context = (Context) it2.next();
-                        for (Iterator it3 = context.getThreads(); it3.hasNext();) {
-                            Thread thread = (Thread) it3.next();
-                            thread.incrementStorage();
-                            for (Iterator e6 = thread.getFunctionProfiles().iterator(); e6.hasNext();) {
-                                FunctionProfile fp = (FunctionProfile) e6.next();
-                                if (fp != null) // fp == null would mean this thread didn't call this function
-                                    fp.incrementStorage();
-                            }
-                        }
-                    }
-                }
-
-            }
-
             for (int i = 0; i < files.length; i++) {
 
                 boolean finished = false;
@@ -148,7 +123,7 @@ public class TauDataSource extends DataSource {
                             return;
                         }
 
-                        int[] nct = this.getNCT(files[i].getName());
+                        int[] nct = getNCT(files[i].getName());
 
                         if (nct == null && dirs.size() == 1 && files.length == 1) {
                             throw new DataSourceException(
@@ -165,12 +140,7 @@ public class TauDataSource extends DataSource {
                         contextID = nct[1];
                         threadID = nct[2];
 
-                        Node node = this.addNode(nodeID);
-                        Context context = node.addContext(contextID);
-                        thread = context.getThread(threadID);
-                        if (thread == null) {
-                            thread = context.addThread(threadID);
-                        }
+                        thread = this.addThread(nodeID, contextID, threadID);
 
                         foundValidFile = true;
 
@@ -180,8 +150,12 @@ public class TauDataSource extends DataSource {
 
                         if (modernJava) {
                             channel = fileIn.getChannel();
-                            lock = channel.lock(0, Long.MAX_VALUE, true);
-
+                            try {
+                                lock = channel.lock(0, Long.MAX_VALUE, true);
+                            } catch (IOException ioe) {
+                                modernJava = false;
+                                lock = null;
+                            }
                         }
                         InputStreamReader inReader = new InputStreamReader(fileIn);
                         BufferedReader br = new BufferedReader(inReader);
@@ -220,6 +194,14 @@ public class TauDataSource extends DataSource {
                             throw new DataSourceException("Unexpected end of file: " + files[i].getName()
                                     + "\nLooking for '# Name Calls ...' line");
                         }
+                        
+                        if (inputString.indexOf("<metadata>") != -1) {
+                            int start = inputString.indexOf("<metadata>");
+                            int end = inputString.indexOf("</metadata>")+11;
+                            String metadata = inputString.substring(start,end);
+                            MetaDataParser.parse(thread.getMetaData(), metadata);
+                        }
+                        
                         if (i == 0) {
                             //Determine if profile stats or profile calls data is present.
                             if (inputString.indexOf("SumExclSqr") != -1)
@@ -234,44 +216,7 @@ public class TauDataSource extends DataSource {
                                         + (j - 2) + " of " + numFunctions + " Function Lines");
                             }
 
-                            this.getFunctionDataLine(inputString);
-                            String groupNames = this.getGroupNames(inputString);
-
-                            if (functionDataLine.i0 != 0) {
-                                func = this.addFunction(functionDataLine.s0, 1);
-
-                                functionProfile = thread.getFunctionProfile(func);
-
-                                if (functionProfile == null) {
-                                    functionProfile = new FunctionProfile(func);
-                                    thread.addFunctionProfile(functionProfile);
-                                }
-
-                                //When we encounter duplicate names in the profile.x.x.x file, treat as additional
-                                //data for the name (that is, don't just overwrite what was there before).
-                                functionProfile.setExclusive(metric, functionProfile.getExclusive(metric) + functionDataLine.d0);
-                                functionProfile.setInclusive(metric, functionProfile.getInclusive(metric) + functionDataLine.d1);
-                                if (metric == 0) {
-                                    functionProfile.setNumCalls(functionProfile.getNumCalls() + functionDataLine.i0);
-                                    functionProfile.setNumSubr(functionProfile.getNumSubr() + functionDataLine.i1);
-                                }
-
-                                if (metric == 0 && groupNames != null) {
-                                    StringTokenizer st = new StringTokenizer(groupNames, "|");
-                                    while (st.hasMoreTokens()) {
-                                        String groupName = st.nextToken();
-                                        if (groupName != null) {
-                                            // The potential new group is added here. If the group is already present,
-                                            // then the addGroup function will just return the
-                                            // already existing group id. See the TrialData
-                                            // class for more details.
-                                            Group group = this.addGroup(groupName.trim());
-                                            func.addGroup(group);
-                                        }
-                                    }
-                                }
-
-                            }
+                            this.processFunctionLine(inputString, thread, metric);
 
                             // unused profile calls
 
@@ -326,28 +271,8 @@ public class TauDataSource extends DataSource {
                                                 + "\nOnly found " + (j - 2) + " of " + numUserEvents + " User Event Lines");
                                     }
 
-                                    this.getUserEventData(inputString);
+                                    this.processUserEventLine(inputString, thread);
 
-                                    // User events
-                                    if (usereventDataLine.i0 != 0) {
-
-                                        UserEvent userEvent = this.addUserEvent(usereventDataLine.s0);
-                                        userEventProfile = thread.getUserEventProfile(userEvent);
-
-                                        if (userEventProfile == null) {
-                                            userEventProfile = new UserEventProfile(userEvent);
-                                            thread.addUserEventProfile(userEventProfile);
-                                        }
-
-                                        userEventProfile.setNumSamples(usereventDataLine.i0);
-                                        userEventProfile.setMaxValue(usereventDataLine.d0);
-                                        userEventProfile.setMinValue(usereventDataLine.d1);
-                                        userEventProfile.setMeanValue(usereventDataLine.d2);
-                                        userEventProfile.setSumSquared(usereventDataLine.d3);
-
-                                        userEventProfile.updateMax();
-
-                                    }
                                 }
                             }
                         }
@@ -416,10 +341,7 @@ public class TauDataSource extends DataSource {
 
         //Generate derived data.
         this.generateDerivedData();
-
-        if (CallPathUtilFuncs.checkCallPathsPresent(this.getFunctions())) {
-            setCallPathDataPresent(true);
-        }
+        this.aggregateMetaData();
 
         //time = (System.currentTimeMillis()) - time;
         //System.out.println("Time to process (in milliseconds): " + time);
@@ -430,7 +352,7 @@ public class TauDataSource extends DataSource {
     }
 
     //profile.*.*.* string processing methods.
-    private int[] getNCT(String string) {
+    public static int[] getNCT(String string) {
         try {
             int[] nct = new int[3];
             StringTokenizer st = new StringTokenizer(string, ".\t\n\r");
@@ -473,7 +395,17 @@ public class TauDataSource extends DataSource {
 
     }
 
-    private void getFunctionDataLine(String string) throws DataSourceException {
+    private void processFunctionLine(String string, Thread thread, int metric) throws DataSourceException {
+
+        String name;
+        double numcalls;
+        double numsubr;
+        double exclusive;
+        double inclusive;
+        double profileCalls;
+        double sumExclSqr;
+
+        String groupNames = this.getGroupNames(string);
 
         // first, count the number of double-quotes to determine if the
         // function contains a double-quote
@@ -491,7 +423,7 @@ public class TauDataSource extends DataSource {
 
         if (quoteCount == 2 || quoteCount == 4) { // assume all is well
             StringTokenizer st1 = new StringTokenizer(string, "\"");
-            functionDataLine.s0 = st1.nextToken(); //Name
+            name = st1.nextToken(); //Name
 
             st2 = new StringTokenizer(st1.nextToken(), " \t\n\r");
         } else {
@@ -507,25 +439,52 @@ public class TauDataSource extends DataSource {
                 i++;
             }
 
-            functionDataLine.s0 = string.substring(1, i - 1);
+            name = string.substring(1, i - 1);
             st2 = new StringTokenizer(string.substring(i + 1), " \t\n\r");
         }
 
-        functionDataLine.i0 = Integer.parseInt(st2.nextToken()); //Calls
-        functionDataLine.i1 = Integer.parseInt(st2.nextToken()); //Subroutines
-        functionDataLine.d0 = Double.parseDouble(st2.nextToken()); //Exclusive
-        functionDataLine.d1 = Double.parseDouble(st2.nextToken()); //Inclusive
-        if (this.getProfileStatsPresent())
-            functionDataLine.d2 = Double.parseDouble(st2.nextToken()); //SumExclSqr
-        functionDataLine.i2 = Integer.parseInt(st2.nextToken()); //ProfileCalls
-
-        if (functionDataLine.d0 < 0) {
-            System.err.println("Warning, negative values found in profile, ignoring!");
-            System.err.println("string = " + string);
-
-            functionDataLine.d0 = 0;
+        numcalls = Double.parseDouble(st2.nextToken()); //Calls
+        numsubr = Double.parseDouble(st2.nextToken()); //Subroutines
+        exclusive = Double.parseDouble(st2.nextToken()); //Exclusive
+        inclusive = Double.parseDouble(st2.nextToken()); //Inclusive
+        if (this.getProfileStatsPresent()) {
+            sumExclSqr = Double.parseDouble(st2.nextToken()); //SumExclSqr
         }
 
+        profileCalls = Integer.parseInt(st2.nextToken()); //ProfileCalls
+
+        if (inclusive < 0) {
+            System.err.println("Warning, negative values found in profile, ignoring!");
+            inclusive = 0;
+        }
+        if (exclusive < 0) {
+            System.err.println("Warning, negative values found in profile, ignoring!");
+            exclusive = 0;
+        }
+
+        if (numcalls != 0) {
+            Function func = this.addFunction(name, 1);
+
+            FunctionProfile functionProfile = thread.getFunctionProfile(func);
+
+            if (functionProfile == null) {
+                functionProfile = new FunctionProfile(func);
+                thread.addFunctionProfile(functionProfile);
+            }
+
+            //When we encounter duplicate names in the profile.x.x.x file, treat as additional
+            //data for the name (that is, don't just overwrite what was there before).
+            functionProfile.setExclusive(metric, functionProfile.getExclusive(metric) + exclusive);
+            functionProfile.setInclusive(metric, functionProfile.getInclusive(metric) + inclusive);
+            if (metric == 0) {
+                functionProfile.setNumCalls(functionProfile.getNumCalls() + numcalls);
+                functionProfile.setNumSubr(functionProfile.getNumSubr() + numsubr);
+            }
+
+            if (metric == 0) {
+                addGroups(groupNames, func);
+            }
+        }
     }
 
     private String getGroupNames(String string) {
@@ -575,7 +534,14 @@ public class TauDataSource extends DataSource {
         return null;
     }
 
-    private void getUserEventData(String string) {
+    private void processUserEventLine(String string, Thread thread) {
+
+        String name;
+        double numSamples;
+        double sampleMax;
+        double sampleMin;
+        double sampleMean;
+        double sampleSumSquared;
 
         // first, count the number of double-quotes to determine if the
         // user event contains a double-quote
@@ -589,7 +555,7 @@ public class TauDataSource extends DataSource {
 
         if (quoteCount == 2) { // proceed as usual
             StringTokenizer st1 = new StringTokenizer(string, "\"");
-            usereventDataLine.s0 = st1.nextToken();
+            name = st1.nextToken();
             st2 = new StringTokenizer(st1.nextToken(), " \t\n\r");
         } else {
 
@@ -602,15 +568,32 @@ public class TauDataSource extends DataSource {
                 i++;
             }
 
-            usereventDataLine.s0 = string.substring(1, i - 1);
+            name = string.substring(1, i - 1);
             st2 = new StringTokenizer(string.substring(i + 1), " \t\n\r");
         }
 
-        usereventDataLine.i0 = (int) Double.parseDouble(st2.nextToken()); //Number of calls
-        usereventDataLine.d0 = Double.parseDouble(st2.nextToken()); //Max
-        usereventDataLine.d1 = Double.parseDouble(st2.nextToken()); //Min
-        usereventDataLine.d2 = Double.parseDouble(st2.nextToken()); //Mean
-        usereventDataLine.d3 = Double.parseDouble(st2.nextToken()); //Standard Deviation
+        numSamples = Double.parseDouble(st2.nextToken()); //Number of calls
+        sampleMax = Double.parseDouble(st2.nextToken()); //Max
+        sampleMin = Double.parseDouble(st2.nextToken()); //Min
+        sampleMean = Double.parseDouble(st2.nextToken()); //Mean
+        sampleSumSquared = Double.parseDouble(st2.nextToken()); //Standard Deviation
+
+        if (numSamples != 0) {
+            UserEvent userEvent = this.addUserEvent(name);
+            UserEventProfile userEventProfile = thread.getUserEventProfile(userEvent);
+
+            if (userEventProfile == null) {
+                userEventProfile = new UserEventProfile(userEvent);
+                thread.addUserEventProfile(userEventProfile);
+            }
+
+            userEventProfile.setNumSamples(numSamples);
+            userEventProfile.setMaxValue(sampleMax);
+            userEventProfile.setMinValue(sampleMin);
+            userEventProfile.setMeanValue(sampleMean);
+            userEventProfile.setSumSquared(sampleSumSquared);
+            userEventProfile.updateMax();
+        }
     }
 
     protected void setProfileStatsPresent(boolean profileStatsPresent) {
@@ -629,4 +612,4 @@ public class TauDataSource extends DataSource {
         return groupCheck;
     }
 
-}
\ No newline at end of file
+}
