@@ -1,15 +1,12 @@
 package edu.uoregon.tau.perfdmf;
 
-import edu.uoregon.tau.perfdmf.database.*;
-
+import java.io.*;
 import java.sql.*;
-import java.util.Vector;
-import java.util.Enumeration;
-import java.lang.String;
-import java.io.Serializable;
-import java.io.ObjectInputStream;
-import java.io.ObjectOutputStream;
-import java.io.IOException;
+import java.util.*;
+import java.util.Date;
+
+import edu.uoregon.tau.perfdmf.database.DB;
+import edu.uoregon.tau.perfdmf.database.DBConnector;
 
 /**
  * Holds all the data for a trial in the database. This object is returned by
@@ -22,7 +19,7 @@ import java.io.IOException;
  * number of threads per context and the metrics collected during the run.
  * 
  * <P>
- * CVS $Id: Trial.java,v 1.1 2005/09/26 20:24:34 amorris Exp $
+ * CVS $Id: Trial.java,v 1.11 2007/02/13 21:56:02 amorris Exp $
  * </P>
  * 
  * @author Kevin Huck, Robert Bell
@@ -38,6 +35,7 @@ import java.io.IOException;
 public class Trial implements Serializable {
     private static String fieldNames[];
     private static int fieldTypes[];
+    private static final String XML_METADATA = new String("XML_METADATA");
 
     private int trialID;
     private int experimentID;
@@ -47,7 +45,10 @@ public class Trial implements Serializable {
     private String fields[];
 
     protected DataSource dataSource = null;
+    private File metadataFile = null;
+    private FileInputStream inStream = null;
 
+    private Map metaData = new TreeMap();
 
     public Trial() {
         if (Trial.fieldNames == null) {
@@ -63,9 +64,10 @@ public class Trial implements Serializable {
         this.applicationID = trial.getApplicationID();
         this.experimentID = trial.getExperimentID();
         this.trialID = trial.getID();
-        this.fields = (String[])trial.fields.clone();
+        this.fields = (String[]) trial.fields.clone();
+        this.metaData = trial.metaData;
     }
-    
+
     public void reallocMetaData() {
         if (Trial.fieldNames == null) {
             this.fields = new String[0];
@@ -343,21 +345,22 @@ public class Trial implements Serializable {
         return;
     }
 
- 	/**
-	 * Returns the column names for the Trial table
-	 *
-	 * @param	db	the database connection
-	 * @return	String[] an array of String objects
-	 */
-	public static String[] getFieldNames(DB db) {
-		getMetaData(db);
-		return fieldNames;
-	}
+    /**
+     * Returns the column names for the Trial table
+     *
+     * @param	db	the database connection
+     * @return	String[] an array of String objects
+     */
+    public static String[] getFieldNames(DB db) {
+        getMetaData(db);
+        return fieldNames;
+    }
 
-   public static void getMetaData(DB db) {
+    public static void getMetaData(DB db) {
         // see if we've already have them
-        if (Trial.fieldNames != null)
-            return;
+        // need to load each time in case we are working with a new database. 
+        //        if (Trial.fieldNames != null)
+        //            return;
 
         try {
             ResultSet resultSet = null;
@@ -367,7 +370,8 @@ public class Trial implements Serializable {
 
             DatabaseMetaData dbMeta = db.getMetaData();
 
-            if (db.getDBType().compareTo("oracle") == 0) {
+            if ((db.getDBType().compareTo("oracle") == 0) || (db.getDBType().compareTo("derby") == 0)
+                    || (db.getDBType().compareTo("db2") == 0)) {
                 resultSet = dbMeta.getColumns(null, null, "TRIAL", "%");
             } else {
                 resultSet = dbMeta.getColumns(null, null, "trial", "%");
@@ -375,18 +379,27 @@ public class Trial implements Serializable {
 
             Vector nameList = new Vector();
             Vector typeList = new Vector();
+            boolean seenID = false;
+
             while (resultSet.next() != false) {
 
                 int ctype = resultSet.getInt("DATA_TYPE");
                 String cname = resultSet.getString("COLUMN_NAME");
                 String typename = resultSet.getString("TYPE_NAME");
 
+                // this code is because of a bug in derby...
+                if (cname.equals("ID")) {
+                    if (!seenID)
+                        seenID = true;
+                    else
+                        break;
+                }
+
                 // only integer and string types (for now)
                 // don't do name and id, we already know about them
 
                 if (DBConnector.isReadAbleType(ctype) && cname.toUpperCase().compareTo("ID") != 0
-                        && cname.toUpperCase().compareTo("NAME") != 0
-                        && cname.toUpperCase().compareTo("APPLICATION") != 0
+                        && cname.toUpperCase().compareTo("NAME") != 0 && cname.toUpperCase().compareTo("APPLICATION") != 0
                         && cname.toUpperCase().compareTo("EXPERIMENT") != 0) {
 
                     nameList.add(resultSet.getString("COLUMN_NAME"));
@@ -421,8 +434,7 @@ public class Trial implements Serializable {
                 buf.append(", t." + Trial.fieldNames[i]);
             }
 
-            buf.append(" from " + db.getSchemaPrefix() + "trial t inner join " + db.getSchemaPrefix()
-                    + "experiment e ");
+            buf.append(" from " + db.getSchemaPrefix() + "trial t inner join " + db.getSchemaPrefix() + "experiment e ");
             buf.append("on t.experiment = e.id ");
             buf.append(whereClause);
             buf.append(" order by t.id ");
@@ -470,10 +482,28 @@ public class Trial implements Serializable {
         int newTrialID = 0;
 
         try {
+            // determine if we have a data meta-data item
+            boolean haveDate = false;
+            java.sql.Timestamp timestamp = null;
+            String dateString = (String) getMetaData().get("UTC Time");
+            if (dateString == null) {
+                dateString = (String) ((Thread)getDataSource().getAllThreads().get(0)).getMetaData().get("UTC Time");
+            }
+            if (dateString != null) {
+                try {
+                    Date date = DataSource.dateTime.parse(dateString);
+                    timestamp = new java.sql.Timestamp(date.getTime());
+                    haveDate = true;
+                } catch (java.text.ParseException e) {
+                    e.printStackTrace();
+                }
+            }
+
+            // FIRST!  Check if the trial table has a metadata column
+            checkForMetadataColumn(db);
 
             // get the fields since this is an insert
             if (!itExists) {
-
                 Trial.getMetaData(db);
                 this.fields = new String[Trial.fieldNames.length];
             }
@@ -482,20 +512,33 @@ public class Trial implements Serializable {
                 // If the user is simply manipulating apps/exps/trials in the treeview
                 // there may not be a dataSource for this trial (it isn't loaded)
                 this.setField("node_count", Integer.toString(1 + this.getDataSource().getMaxNCTNumbers()[0]));
-                this.setField("contexts_per_node",
-                        Integer.toString(1 + this.getDataSource().getMaxNCTNumbers()[1]));
-                this.setField("threads_per_context",
-                        Integer.toString(1 + this.getDataSource().getMaxNCTNumbers()[2]));
+                this.setField("contexts_per_node", Integer.toString(1 + this.getDataSource().getMaxNCTNumbers()[1]));
+                this.setField("threads_per_context", Integer.toString(1 + this.getDataSource().getMaxNCTNumbers()[2]));
+            }
+
+            // Check if the date column exists and is a timestamp
+            boolean dateColumnFound = false;
+            for (int i = 0; i < this.getNumFields(); i++) {
+                if (getFieldName(i).equals("date")) {
+                    if (getFieldType(i) == java.sql.Types.TIMESTAMP) {
+                        dateColumnFound = true;
+                    }
+                }
             }
 
             StringBuffer buf = new StringBuffer();
-
             if (itExists) {
                 buf.append("UPDATE " + db.getSchemaPrefix() + "trial SET name = ?, experiment = ?");
                 for (int i = 0; i < this.getNumFields(); i++) {
-                    if (DBConnector.isWritableType(this.getFieldType(i)))
+                    if (DBConnector.isWritableType(this.getFieldType(i))) {
                         buf.append(", " + this.getFieldName(i) + " = ?");
+                    }
                 }
+
+                if (haveDate && dateColumnFound) {
+                    buf.append(", date = ?");
+                }
+
                 buf.append(" WHERE id = ?");
             } else {
                 buf.append("INSERT INTO " + db.getSchemaPrefix() + "trial (name, experiment");
@@ -503,14 +546,22 @@ public class Trial implements Serializable {
                     if (DBConnector.isWritableType(this.getFieldType(i)))
                         buf.append(", " + this.getFieldName(i));
                 }
+
+                if (haveDate && dateColumnFound) {
+                    buf.append(", date");
+                }
                 buf.append(") VALUES (?, ?");
                 for (int i = 0; i < this.getNumFields(); i++) {
                     if (DBConnector.isWritableType(this.getFieldType(i)))
                         buf.append(", ?");
                 }
+                if (haveDate && dateColumnFound) {
+                    buf.append(", ?");
+                }
                 buf.append(")");
             }
 
+            //System.out.println(buf.toString());
             PreparedStatement statement = db.prepareStatement(buf.toString());
 
             int pos = 1;
@@ -518,15 +569,35 @@ public class Trial implements Serializable {
             statement.setString(pos++, name);
             statement.setInt(pos++, experimentID);
             for (int i = 0; i < this.getNumFields(); i++) {
-                if (DBConnector.isWritableType(this.getFieldType(i)))
-                    statement.setString(pos++, this.getField(i));
+                if (DBConnector.isWritableType(this.getFieldType(i))) {
+                    if ((this.getFieldName(i).equalsIgnoreCase(Trial.XML_METADATA)) && (this.metadataFile != null)) {
+                        statement.setAsciiStream(pos++, inStream, (int) this.metadataFile.length());
+                    } else {
+                        statement.setString(pos++, this.getField(i));
+                    }
+                }
+            }
+
+            if (haveDate && dateColumnFound) {
+                statement.setTimestamp(pos++, timestamp);
             }
 
             if (itExists) {
                 statement.setInt(pos, trialID);
             }
+
             statement.executeUpdate();
             statement.close();
+            if (this.metadataFile != null) {
+                try {
+                    inStream.close();
+                } catch (IOException e) {
+                    System.err.println("Unable to close file:");
+                    System.err.println(e.getMessage());
+                    e.printStackTrace();
+                }
+            }
+
             if (itExists) {
                 newTrialID = trialID;
             } else {
@@ -535,6 +606,8 @@ public class Trial implements Serializable {
                     tmpStr = "select LAST_INSERT_ID();";
                 else if (db.getDBType().compareTo("db2") == 0)
                     tmpStr = "select IDENTITY_VAL_LOCAL() FROM trial";
+                else if (db.getDBType().compareTo("derby") == 0)
+                    tmpStr = "select IDENTITY_VAL_LOCAL() FROM trial";
                 else if (db.getDBType().compareTo("oracle") == 0)
                     tmpStr = "select " + db.getSchemaPrefix() + "trial_id_seq.currval FROM dual";
                 else
@@ -598,116 +671,113 @@ public class Trial implements Serializable {
         statement.execute();
         statement.close();
 
-        statement = db.prepareStatement("DELETE FROM interval_mean_summary WHERE interval_event IN"
-                + deleteString.toString() + ")");
+        statement = db.prepareStatement("DELETE FROM interval_mean_summary WHERE interval_event IN" + deleteString.toString()
+                + ")");
         statement.execute();
         statement.close();
 
-        statement = db.prepareStatement("DELETE FROM interval_total_summary WHERE interval_event IN"
-                + deleteString.toString() + ")");
+        statement = db.prepareStatement("DELETE FROM interval_total_summary WHERE interval_event IN" + deleteString.toString()
+                + ")");
         statement.execute();
         statement.close();
 
     }
 
     public static void deleteTrial(DB db, int trialID) throws SQLException {
-            // save this trial
-            PreparedStatement statement = null;
+        // save this trial
+        PreparedStatement statement = null;
 
-            // delete from the atomic_location_profile table
-            if (db.getDBType().compareTo("mysql") == 0) {
+        // delete from the atomic_location_profile table
+        if (db.getDBType().compareTo("mysql") == 0) {
 
-                Trial.deleteAtomicLocationProfilesMySQL(db, trialID);
+            Trial.deleteAtomicLocationProfilesMySQL(db, trialID);
 
-                //                statement = db.prepareStatement(" DELETE atomic_location_profile.* FROM "
-                //                        + db.getSchemaPrefix()
-                //                        + "atomic_location_profile LEFT JOIN "
-                //                        + db.getSchemaPrefix()
-                //                        + "atomic_event ON atomic_location_profile.atomic_event = atomic_event.id WHERE atomic_event.trial = ?");
-            } else {
-                // Postgresql, oracle, and DB2?
-                statement = db.prepareStatement(" DELETE FROM " + db.getSchemaPrefix()
-                        + "atomic_location_profile WHERE atomic_event in (SELECT id FROM "
-                        + db.getSchemaPrefix() + "atomic_event WHERE trial = ?)");
-                statement.setInt(1, trialID);
-                statement.execute();
-                statement.close();
-            }
-
-            // delete the from the atomic_events table
+            //                statement = db.prepareStatement(" DELETE atomic_location_profile.* FROM "
+            //                        + db.getSchemaPrefix()
+            //                        + "atomic_location_profile LEFT JOIN "
+            //                        + db.getSchemaPrefix()
+            //                        + "atomic_event ON atomic_location_profile.atomic_event = atomic_event.id WHERE atomic_event.trial = ?");
+        } else {
+            // Postgresql, oracle, and DB2?
             statement = db.prepareStatement(" DELETE FROM " + db.getSchemaPrefix()
-                    + "atomic_event WHERE trial = ?");
+                    + "atomic_location_profile WHERE atomic_event in (SELECT id FROM " + db.getSchemaPrefix()
+                    + "atomic_event WHERE trial = ?)");
             statement.setInt(1, trialID);
             statement.execute();
             statement.close();
+        }
 
-            // delete from the interval_location_profile table
-            if (db.getDBType().compareTo("mysql") == 0) {
+        // delete the from the atomic_events table
+        statement = db.prepareStatement(" DELETE FROM " + db.getSchemaPrefix() + "atomic_event WHERE trial = ?");
+        statement.setInt(1, trialID);
+        statement.execute();
+        statement.close();
 
-                Trial.deleteIntervalLocationProfilesMySQL(db, trialID);
+        // delete from the interval_location_profile table
+        if (db.getDBType().compareTo("mysql") == 0) {
 
-                //                statement = db.prepareStatement(" DELETE interval_location_profile.* FROM "
-                //                        + db.getSchemaPrefix()
-                //                        + "interval_location_profile LEFT JOIN "
-                //                        + db.getSchemaPrefix()
-                //                        + "interval_event ON interval_location_profile.interval_event = interval_event.id WHERE interval_event.trial = ?");
-            } else {
-                // Postgresql and DB2?
-                statement = db.prepareStatement(" DELETE FROM " + db.getSchemaPrefix()
-                        + "interval_location_profile WHERE interval_event IN (SELECT id FROM "
-                        + db.getSchemaPrefix() + "interval_event WHERE trial = ?)");
-                statement.setInt(1, trialID);
-                statement.execute();
-                statement.close();
-            }
-
-            // delete from the interval_mean_summary table
-            if (db.getDBType().compareTo("mysql") == 0) {
-                //statement = db.prepareStatement(" DELETE interval_mean_summary.* FROM interval_mean_summary LEFT JOIN interval_event ON interval_mean_summary.interval_event = interval_event.id WHERE interval_event.trial = ?");
-            } else {
-                // Postgresql and DB2?
-                statement = db.prepareStatement(" DELETE FROM " + db.getSchemaPrefix()
-                        + "interval_mean_summary WHERE interval_event IN (SELECT id FROM "
-                        + db.getSchemaPrefix() + "interval_event WHERE trial = ?)");
-                statement.setInt(1, trialID);
-                statement.execute();
-                statement.close();
-            }
-
-            if (db.getDBType().compareTo("mysql") == 0) {
-                //statement = db.prepareStatement(" DELETE interval_total_summary.* FROM interval_total_summary LEFT JOIN interval_event ON interval_total_summary.interval_event = interval_event.id WHERE interval_event.trial = ?");
-            } else {
-                // Postgresql and DB2?
-                statement = db.prepareStatement(" DELETE FROM " + db.getSchemaPrefix()
-                        + "interval_total_summary WHERE interval_event IN (SELECT id FROM "
-                        + db.getSchemaPrefix() + "interval_event WHERE trial = ?)");
-                statement.setInt(1, trialID);
-                statement.execute();
-                statement.close();
-            }
+            Trial.deleteIntervalLocationProfilesMySQL(db, trialID);
 
+            //                statement = db.prepareStatement(" DELETE interval_location_profile.* FROM "
+            //                        + db.getSchemaPrefix()
+            //                        + "interval_location_profile LEFT JOIN "
+            //                        + db.getSchemaPrefix()
+            //                        + "interval_event ON interval_location_profile.interval_event = interval_event.id WHERE interval_event.trial = ?");
+        } else {
+            // Postgresql and DB2?
             statement = db.prepareStatement(" DELETE FROM " + db.getSchemaPrefix()
-                    + "interval_event WHERE trial = ?");
+                    + "interval_location_profile WHERE interval_event IN (SELECT id FROM " + db.getSchemaPrefix()
+                    + "interval_event WHERE trial = ?)");
             statement.setInt(1, trialID);
             statement.execute();
             statement.close();
+        }
 
-            statement = db.prepareStatement(" DELETE FROM " + db.getSchemaPrefix() + "metric WHERE trial = ?");
+        // delete from the interval_mean_summary table
+        if (db.getDBType().compareTo("mysql") == 0) {
+            //statement = db.prepareStatement(" DELETE interval_mean_summary.* FROM interval_mean_summary LEFT JOIN interval_event ON interval_mean_summary.interval_event = interval_event.id WHERE interval_event.trial = ?");
+        } else {
+            // Postgresql and DB2?
+            statement = db.prepareStatement(" DELETE FROM " + db.getSchemaPrefix()
+                    + "interval_mean_summary WHERE interval_event IN (SELECT id FROM " + db.getSchemaPrefix()
+                    + "interval_event WHERE trial = ?)");
             statement.setInt(1, trialID);
             statement.execute();
             statement.close();
+        }
 
-            statement = db.prepareStatement(" DELETE FROM " + db.getSchemaPrefix() + "trial WHERE id = ?");
+        if (db.getDBType().compareTo("mysql") == 0) {
+            //statement = db.prepareStatement(" DELETE interval_total_summary.* FROM interval_total_summary LEFT JOIN interval_event ON interval_total_summary.interval_event = interval_event.id WHERE interval_event.trial = ?");
+        } else {
+            // Postgresql and DB2?
+            statement = db.prepareStatement(" DELETE FROM " + db.getSchemaPrefix()
+                    + "interval_total_summary WHERE interval_event IN (SELECT id FROM " + db.getSchemaPrefix()
+                    + "interval_event WHERE trial = ?)");
             statement.setInt(1, trialID);
             statement.execute();
             statement.close();
+        }
+
+        statement = db.prepareStatement(" DELETE FROM " + db.getSchemaPrefix() + "interval_event WHERE trial = ?");
+        statement.setInt(1, trialID);
+        statement.execute();
+        statement.close();
+
+        statement = db.prepareStatement(" DELETE FROM " + db.getSchemaPrefix() + "metric WHERE trial = ?");
+        statement.setInt(1, trialID);
+        statement.execute();
+        statement.close();
+
+        statement = db.prepareStatement(" DELETE FROM " + db.getSchemaPrefix() + "trial WHERE id = ?");
+        statement.setInt(1, trialID);
+        statement.execute();
+        statement.close();
     }
 
     private boolean exists(DB db) {
         boolean retval = false;
         try {
-            PreparedStatement statement = db.prepareStatement("SELECT name FROM " + db.getSchemaPrefix()
-                    + "trial WHERE id = ?");
+            PreparedStatement statement = db.prepareStatement("SELECT name FROM " + db.getSchemaPrefix() + "trial WHERE id = ?");
             statement.setInt(1, trialID);
             ResultSet results = statement.executeQuery();
             while (results.next() != false) {
@@ -738,4 +808,75 @@ public class Trial implements Serializable {
         aOutputStream.writeObject(fieldTypes);
     }
 
+    /**
+     *  hack - needed to delete meta so that it is reloaded each time a new database is created.
+     */
+    public void removeMetaData() {
+        fieldNames = null;
+        fieldTypes = null;
+    }
+
+    /**
+     * If the user passes in a metadata file, parse it into the trial.
+     * 
+     * @param metadataFileName
+     * @throws IOException
+     */
+    public void setMetadataFile(String metadataFileName) throws IOException {
+        this.metadataFile = new File(metadataFileName);
+        if (!this.metadataFile.exists())
+            throw new FileNotFoundException("The file " + metadataFileName + " does not exist.");
+        if (!this.metadataFile.canRead())
+            throw new IOException("The file " + metadataFileName + " does not have read permission.");
+        if (!this.metadataFile.isFile())
+            throw new FileNotFoundException(metadataFileName + " is not a valid file.");
+        inStream = new FileInputStream(this.metadataFile);
+        return;
+    }
+
+    public void checkForMetadataColumn(DB db) {
+        if (this.metadataFile != null) {
+            String[] columns = Trial.getFieldNames(db);
+            boolean found = false;
+            // loop through the column names, and see if we have this column already
+            for (int i = 0; i < columns.length; i++) {
+                if (columns[i].equalsIgnoreCase(XML_METADATA)) {
+                    found = true;
+                    break;
+                }
+            }
+            if (!found) {
+                StringBuffer sql = new StringBuffer();
+                // create the column in the database
+                sql.append("ALTER TABLE " + db.getSchemaPrefix() + "trial ADD COLUMN ");
+                sql.append(XML_METADATA);
+                if ((db.getDBType().equalsIgnoreCase("oracle")) || (db.getDBType().equalsIgnoreCase("derby"))) {
+                    sql.append(" CLOB");
+                } else if (db.getDBType().equalsIgnoreCase("db2")) {
+                    sql.append(" CLOB");
+                } else if (db.getDBType().equalsIgnoreCase("mysql")) {
+                    sql.append(" TEXT");
+                } else if (db.getDBType().equalsIgnoreCase("postgresql")) {
+                    sql.append(" TEXT");
+                }
+
+                try {
+                    db.execute(sql.toString());
+                } catch (SQLException e) {
+                    System.err.println("Unable to add " + XML_METADATA + " column to trial table.");
+                    e.printStackTrace();
+                }
+            }
+        }
+    }
+
+    
+    public Map getMetaData() {
+        return metaData;
+    }
+
+    public void setMetaData(Map metaDataMap) {
+        this.metaData = metaDataMap;
+    }
+
 }
