@@ -2,9 +2,9 @@ package edu.uoregon.tau.perfdmf.loader;
 
 import jargs.gnu.CmdLineParser;
 
-import java.io.*;
-import java.util.List;
-import java.util.Vector;
+import java.io.BufferedReader;
+import java.io.File;
+import java.io.FileReader;
 
 import edu.uoregon.tau.perfdmf.*;
 
@@ -13,6 +13,7 @@ public class LoadTrial {
     private File writeXml;
     private String trialTime;
     private String sourceFiles[];
+    private String metadataFile;
     private Application app;
     private Experiment exp;
     private boolean fixNames = false;
@@ -38,7 +39,8 @@ public class LoadTrial {
                 + "                                   profiles (default), pprof, dynaprof, mpip,\n"
                 + "                                   gprof, psrun, hpm, packed, cube, hpc\n"
                 + "  -t, --trialid <number>         Specify trial ID\n"
-                + "  -i, --fixnames                 Use the fixnames option for gprof\n\n" + "Notes:\n"
+                + "  -i, --fixnames                 Use the fixnames option for gprof\n\n"
+                + "  -m, --metadata <filename>      XML metadata for the trial\n" + "Notes:\n"
                 + "  For the TAU profiles type, you can specify either a specific set of profile\n"
                 + "files on the commandline, or you can specify a directory (by default the current\n"
                 + "directory).  The specified directory will be searched for profile.*.*.* files,\n"
@@ -46,7 +48,7 @@ public class LoadTrial {
                 + "Examples:\n\n" + "  perfdmf_loadtrial -e 12 -n \"Batch 001\"\n"
                 + "    This will load profile.* (or multiple counters directories MULTI_*) into\n"
                 + "    experiment 12 and give the trial the name \"Batch 001\"\n\n"
-                + "  perfdmf_loadtrial -e 12 -n \"HPM data 01\" perfhpm*\n"
+                + "  perfdmf_loadtrial -e 12 -n \"HPM data 01\" -f hpm perfhpm*\n"
                 + "    This will load perfhpm* files of type HPMToolkit into experiment 12 and give\n"
                 + "    the trial the name \"HPM data 01\"\n");
     }
@@ -128,6 +130,19 @@ public class LoadTrial {
             e.printStackTrace();
         }
 
+        // set the meta data from the datasource
+        trial.setMetaData(dataSource.getMetaData());
+        
+        try {
+            if (metadataFile != null) {
+                trial.setMetadataFile(metadataFile);
+            }
+        } catch (Exception e) {
+            System.err.println("Error Loading metadata:");
+            e.printStackTrace();
+            System.exit(1);
+        }
+
         if (trialID == 0) {
             saveTrial();
         } else {
@@ -210,8 +225,7 @@ public class LoadTrial {
         // close the problem file
         try {
             reader.close();
-        } catch (Exception e) {
-        }
+        } catch (Exception e) {}
 
         // return the string
         return problemString.toString();
@@ -232,6 +246,7 @@ public class LoadTrial {
         CmdLineParser.Option trialOpt = parser.addStringOption('t', "trialid");
         CmdLineParser.Option typeOpt = parser.addStringOption('f', "filetype");
         CmdLineParser.Option fixOpt = parser.addBooleanOption('i', "fixnames");
+        CmdLineParser.Option metadataOpt = parser.addStringOption('m', "metadata");
 
         try {
             parser.parse(args);
@@ -251,6 +266,7 @@ public class LoadTrial {
         String trialID = (String) parser.getOptionValue(trialOpt);
         String fileTypeString = (String) parser.getOptionValue(typeOpt);
         Boolean fixNames = (Boolean) parser.getOptionValue(fixOpt);
+        String metadataFile = (String) parser.getOptionValue(metadataOpt);
 
         if (help != null && help.booleanValue()) {
             LoadTrial.outputHelp();
@@ -265,11 +281,10 @@ public class LoadTrial {
             System.err.println("Error: Missing trial name\n");
             LoadTrial.usage();
             System.exit(-1);
-
-//        } else if (sourceFiles == null) {
-//            System.err.println("Please enter a valid source file.");
-//            LoadTrial.usage();
-//            System.exit(-1);
+            //        } else if (sourceFiles == null) {
+            //            System.err.println("Please enter a valid source file.");
+            //            LoadTrial.usage();
+            //            System.exit(-1);
         } else if (experimentID == null) {
             System.err.println("Error: Missing experiment id\n");
             LoadTrial.usage();
@@ -311,12 +326,15 @@ public class LoadTrial {
         } else {
             if (sourceFiles.length == 1) {
                 String filename = sourceFiles[0];
-                if (filename.endsWith(".ppk")) {
+                if (filename.toLowerCase().endsWith(".ppk")) {
                     fileType = 7;
                 }
-                if (filename.endsWith(".cube")) {
+                if (filename.toLowerCase().endsWith(".cube")) {
                     fileType = 8;
                 }
+                if (filename.toLowerCase().endsWith(".mpip")) {
+                    fileType = 3;
+                }
             }
         }
 
@@ -337,6 +355,7 @@ public class LoadTrial {
         trans.trialName = trialName;
         //trans.problemFile = problemFile;
         trans.fixNames = fixNames.booleanValue();
+        trans.metadataFile = metadataFile;
         trans.loadTrial(fileType);
         // the trial will be saved when the load is finished (update is called)
     }
