@@ -1,20 +1,28 @@
 package client;
 
 import javax.swing.*;
-
 import java.awt.event.*;
+import java.awt.Frame;
+import java.awt.Component;
+import java.awt.Container;
 import java.util.List;
 import java.util.ArrayList;
 import java.util.ListIterator;
 import edu.uoregon.tau.perfdmf.*;
+import edu.uoregon.tau.common.Utility;
 import constants.Constants;
+import common.AnalysisType;
 import common.RMIView;
+import common.RMISortableIntervalEvent;
 import common.RMIPerfExplorerModel;
+import common.TransformationType;
+import common.PerfExplorerOutput;
 
 public class PerfExplorerActionListener implements ActionListener {
 
 	public final static String QUIT = "Quit PerfExplorer";
 	public final static String QUIT_SERVER = "Quit PerfExplorer (Shutdown Server)";
+	public final static String LOOK_AND_FEEL = "Set Look and Feel: ";
 	public final static String ABOUT = "About PerfExplorer";
 	public final static String SEARCH = "Search for help on...";
 	// clustering menu items
@@ -33,6 +41,7 @@ public class PerfExplorerActionListener implements ActionListener {
 	public final static String SET_TIMESTEPS = "Set Total Number of Timesteps";
 	public final static String SET_EVENTNAME = "Set Event of Interest";
 	public final static String TIMESTEPS_CHART = "Timesteps Per Second";
+	public final static String TOTAL_TIME_CHART = "Total Execution Time";
 	public final static String EFFICIENCY_CHART = "Relative Efficiency";
 	public final static String EFFICIENCY_EVENTS_CHART = "Relative Efficiency by Event";
 	public final static String EFFICIENCY_ONE_EVENT_CHART = "Relative Efficiency for One Event";
@@ -51,6 +60,7 @@ public class PerfExplorerActionListener implements ActionListener {
 	public final static String CREATE_NEW_SUB_VIEW = "Create New Sub-view";
 	public final static String DO_IQR_BOXCHART = "Create BoxChart";
 	public final static String DO_HISTOGRAM = "Create Histograms";
+	public final static String DO_PROBABILITY_PLOT = "Create Normal Probability Plot";
 
 	private PerfExplorerClient mainFrame;
 
@@ -70,6 +80,26 @@ public class PerfExplorerActionListener implements ActionListener {
 					//PerfExplorerConnection server = PerfExplorerConnection.getConnection();
 					//server.stopServer();
 					System.exit(0);
+				} else if (arg.startsWith(LOOK_AND_FEEL)) {
+					UIManager.LookAndFeelInfo[] info = UIManager.getInstalledLookAndFeels();
+					for (int i = 0 ; i < info.length ; i++) {
+						if (arg.endsWith(info[i].getName())) {
+							try {
+								PerfExplorerClient frame = PerfExplorerClient.getMainFrame();
+								UIManager.setLookAndFeel(info[i].getClassName());
+								Frame[] frames = frame.getFrames();
+								for (int x = 0 ; x < frames.length ; x++) {
+									Component[] comps = frame.getComponents();
+									for (int y = 0 ; y < comps.length ; y++) {
+										updateAll((Container)comps[y]);
+									}
+									frames[x].repaint();
+								}
+							} catch (UnsupportedLookAndFeelException e) {
+								PerfExplorerOutput.println(e.getMessage());
+							}
+						}
+					}
 			// help menu items
 				} else if (arg.equals(ABOUT)) {
 					createAboutWindow();
@@ -90,6 +120,7 @@ public class PerfExplorerActionListener implements ActionListener {
 				} else if (arg.equals(DO_CORRELATION_ANALYSIS)) {
 					if (validCorrelationSelection())
 						createDoCorrelationWindow();
+			// data display items
 				} else if (arg.equals(DO_CORRELATION_CUBE)) {
 					if (valid3DSelection())
 						PerfExplorerCube.doCorrelationCube();
@@ -97,11 +128,14 @@ public class PerfExplorerActionListener implements ActionListener {
 					if (valid3DSelection())
 						PerfExplorerBoxChart.doIQRBoxChart();
 				} else if (arg.equals(DO_HISTOGRAM)) {
-					if (valid3DSelection())
+					if (validDistributionSelection())
 						PerfExplorerHistogramChart.doHistogram();
 				} else if (arg.equals(DO_VARIATION_ANALYSIS)) {
 					if (valid3DSelection())
 						PerfExplorerVariation.doVariationAnalysis();
+				} else if (arg.equals(DO_PROBABILITY_PLOT)) {
+					if (validDistributionSelection())
+						PerfExplorerProbabilityPlot.doProbabilityPlot();
 			// chart items
 				} else if (arg.equals(SET_PROBLEM_SIZE)) {
 					checkAndSetProblemSize(true);
@@ -117,6 +151,9 @@ public class PerfExplorerActionListener implements ActionListener {
 					if (checkAndSetMetricName(false))
 						if (checkAndSetTimesteps(false))
 							PerfExplorerChart.doTimestepsChart();
+				} else if (arg.equals(TOTAL_TIME_CHART)) {
+					if (checkAndSetMetricName(false))
+						PerfExplorerChart.doTotalTimeChart();
 				} else if (arg.equals(EFFICIENCY_CHART)) {
 					if (checkAndSetMetricName(false))
 						if (checkAndSetProblemSize(false))
@@ -152,7 +189,7 @@ public class PerfExplorerActionListener implements ActionListener {
 						PerfExplorerChart.doFractionChart();
 				} else if (arg.equals(CORRELATION_CHART)) {
 					if (checkAndSetMetricName(false))
-						//if (checkAndSetProblemSize(false))
+						if (checkAndSetProblemSize(false))
 							PerfExplorerChart.doCorrelationChart();
 				} else if (arg.equals(EFFICIENCY_PHASE_CHART)) {
 					if (checkAndSetMetricName(false))
@@ -181,6 +218,24 @@ public class PerfExplorerActionListener implements ActionListener {
 		} 
 	}
 
+	private void updateAll (Container container) {
+		Component[] comps = container.getComponents();
+		for (int y = 0 ; y < comps.length ; y++) {
+			if (comps[y] instanceof Container) {
+				updateAll((Container)comps[y]);
+			}
+			if (comps[y] instanceof JComponent) {
+				JComponent comp = (JComponent)comps[y];
+				comp.updateUI();
+			}
+			if (comps[y] instanceof JTree) {
+				JTree tree = (JTree)comps[y];
+				tree.updateUI();
+			}
+		}
+		container.repaint();
+	}
+
     public static String getVersionString() {
 		return new String(Constants.VERSION);
 	}
@@ -192,56 +247,69 @@ public class PerfExplorerActionListener implements ActionListener {
 		String message = new String("PerfExplorer 1.0\n" +
 					getVersionString() + "\nJVM Heap Size: " + memUsage
 					+ "kb\n");
+		ImageIcon icon = createImageIcon(Utility.getResource("tau-large.png"));
 		JOptionPane.showMessageDialog(mainFrame, message, 
-			"About PerfExplorer", JOptionPane.PLAIN_MESSAGE);
+			"About PerfExplorer", JOptionPane.INFORMATION_MESSAGE, icon);
 	}
 
+    /** Returns an ImageIcon, or null if the path was invalid. */
+    protected static ImageIcon createImageIcon(java.net.URL imgURL) {
+        if (imgURL != null) {
+            return new ImageIcon(imgURL);
+        } else {
+            System.err.println("Couldn't find file: " + imgURL);
+            return null;
+        }
+    }
+
 	public void createHelpWindow() {
+		ImageIcon icon = createImageIcon(Utility.getResource("tau-large.png"));
 		JOptionPane.showMessageDialog(mainFrame, 
-			"Help not implemented.\nFor the most up-to-date documentation, please see\n<html><a href='http://www.cs.uoregon.edu/research/tau/'>http://www.cs.uoregon.edu/research/tau/</a></html>",
-			"PerfExplorer Help", JOptionPane.PLAIN_MESSAGE);
+			"Internal help not implemented.\nFor the most up-to-date documentation, please see\n<html><a href='http://www.cs.uoregon.edu/research/tau/'>http://www.cs.uoregon.edu/research/tau/</a></html>",
+			"PerfExplorer Help", JOptionPane.INFORMATION_MESSAGE, icon);
 	}
 
 	public void createMethodWindow() {
-		Object[] options = PerfExplorerModel.getClusterMethods();
+		Object[] options = AnalysisType.getClusterMethods();
 		String reply = (String)JOptionPane.showInputDialog (mainFrame,
 			"Select a cluster method:",
 			"Cluster Method",
 			JOptionPane.PLAIN_MESSAGE,
 			null,
 			options,
-			PerfExplorerModel.K_MEANS);
+			AnalysisType.K_MEANS);
 		PerfExplorerModel.getModel().setClusterMethod(reply);
 	}
 
 	public void createDimensionWindow() {
-		Object[] options = PerfExplorerModel.getDimensionReductions();
-		String reply = (String)JOptionPane.showInputDialog (mainFrame,
+		Object[] options = TransformationType.getDimensionReductions();
+		Object obj = JOptionPane.showInputDialog (mainFrame,
 			"Select a dimension reduction method:",
 			"Dimension Reduction",
 			JOptionPane.PLAIN_MESSAGE,
 			null,
 			options,
-			PerfExplorerModel.NONE);
+			TransformationType.NONE);
+        TransformationType reply = (TransformationType)obj;
 		PerfExplorerModel.getModel().setDimensionReduction(reply);
 		if (PerfExplorerModel.getModel().getDimensionReduction().equals(reply)) {
-			reply = (String)JOptionPane.showInputDialog (mainFrame,
+			String reply2 = (String)JOptionPane.showInputDialog (mainFrame,
 				"Only select events with exclusive time % greater than X:\n(where 0 <= X < 100)",
 				"Minimum Percentage", JOptionPane.PLAIN_MESSAGE);
 			if (reply != null && !reply.equals(""))
-				PerfExplorerModel.getModel().setXPercent(reply);
+				PerfExplorerModel.getModel().setXPercent(reply2);
 		}
 	}
 
 	public void createNormalizationWindow() {
-		Object[] options = PerfExplorerModel.getNormalizations();
-		String reply = (String)JOptionPane.showInputDialog (mainFrame,
+		Object[] options = TransformationType.getNormalizations();
+		TransformationType reply = (TransformationType)JOptionPane.showInputDialog (mainFrame,
 			"Select a normalization method:",
 			"Normalization",
 			JOptionPane.PLAIN_MESSAGE,
 			null,
 			options,
-			PerfExplorerModel.NONE);
+			TransformationType.NONE);
 		PerfExplorerModel.getModel().setNormalization(reply);
 	}
 
@@ -386,7 +454,7 @@ public class PerfExplorerActionListener implements ActionListener {
 		if ((selection instanceof Trial) || (selection instanceof Metric)) {
 //			String tmp = theModel.getClusterMethod();
 			RMIPerfExplorerModel modelCopy = theModel.copy();
-			modelCopy.setClusterMethod(PerfExplorerModel.CORRELATION_ANALYSIS);
+			modelCopy.setClusterMethod(AnalysisType.CORRELATION_ANALYSIS);
 			int reply = getConfirmation(modelCopy);
 			if (reply == 1) {
 				// request some analysis!
@@ -409,10 +477,10 @@ public class PerfExplorerActionListener implements ActionListener {
 		StringBuffer buf = new StringBuffer();
 		buf.append("Analysis method: " + theModel.getClusterMethod());
 		buf.append("\nDimension Reduction: " + theModel.getDimensionReduction());
-		if (theModel.getDimensionReduction().equals(PerfExplorerModel.OVER_X_PERCENT)) 
+		if (theModel.getDimensionReduction().equals(TransformationType.OVER_X_PERCENT)) 
 			buf.append("\n\t\t Minimum percentage: " + theModel.getXPercent());
 		buf.append("\nNormalization: " + theModel.getNormalization());
-		if (!theModel.getClusterMethod().equals(PerfExplorerModel.CORRELATION_ANALYSIS))
+		if (!theModel.getClusterMethod().equals(AnalysisType.CORRELATION_ANALYSIS))
 			buf.append("\nMax Clusters: " + theModel.getNumberOfClusters());
 		buf.append("\nTrial: " + theModel.toString());
 		buf.append("\n\nPerform clustering with the these options?");
@@ -454,12 +522,20 @@ public class PerfExplorerActionListener implements ActionListener {
 
 	private boolean validSelection (PerfExplorerModel theModel) {
 		Object selection = theModel.getCurrentSelection();
-		// allow Experiments or Trials or 1 view
+/*		// allow Experiments or Trials or 1 view
 		if (!(selection instanceof Experiment) && !(selection instanceof Trial) && !(selection instanceof RMIView)) {
 			JOptionPane.showMessageDialog(mainFrame, "Please select one or more Experiments or Trials.",
 				"Selection Error", JOptionPane.ERROR_MESSAGE);
 			return false;
 		}
+*/		// allow Experiments or 1 view
+		if (!(selection instanceof Experiment) && !(selection instanceof RMIView)) {
+		// allow Experiments or 1 view
+		// if (!(selection instanceof Experiment) && !(selection instanceof RMIView)) {
+			JOptionPane.showMessageDialog(mainFrame, "Please select one or more Experiments or one View.",
+				"Selection Error", JOptionPane.ERROR_MESSAGE);
+			return false;
+		}
 		// check multi-selections, to make sure they are homogeneous
 		return true;
 	}
@@ -476,6 +552,21 @@ public class PerfExplorerActionListener implements ActionListener {
 		return true;
 	}
 
+	private boolean validDistributionSelection () {
+		PerfExplorerModel theModel = PerfExplorerModel.getModel();
+		Object selection = theModel.getCurrentSelection();
+		// allow only Metrics or IntervalEvents
+		if ((selection == null) || 
+			(!(selection instanceof Metric) && 
+			 !(selection instanceof RMISortableIntervalEvent))) {
+			JOptionPane.showMessageDialog(mainFrame, 
+				"Please select an Metrics or one or more Events.",
+				"Selection Error", JOptionPane.ERROR_MESSAGE);
+			return false;
+		}
+		return true;
+	}
+
 	private boolean checkAndSetMetricName (boolean forceIt) {
 		//TODO - MAKE SURE THE METRIC EXISTS IN THE SELECTED TRIALS!
 		PerfExplorerModel theModel = PerfExplorerModel.getModel();
@@ -564,14 +655,14 @@ public class PerfExplorerActionListener implements ActionListener {
 		Boolean constantProblem = theModel.getConstantProblem();
 		if (forceIt || constantProblem == null) {
 			List answers = new ArrayList();
-			answers.add("Yes, the problem size remains constant.");
-			answers.add("No, the problem increases as the processor count increases.");
+			answers.add("The problem size remains constant. (strong scaling)");
+			answers.add("The problem size increases as the processor count increases. (weak scaling)");
 			Object[] options = answers.toArray();
 			String response = (String)JOptionPane.showInputDialog (mainFrame,
-				"Does the problem size remain constant, or does it scale with the number of processors?",
+				"Please select the problem scaling:",
 				"Problem Scaling", JOptionPane.PLAIN_MESSAGE,
 				null, options, options[0]);
-			theModel.setConstantProblem(response.startsWith("Y") ? true : false);
+			theModel.setConstantProblem(response.startsWith("The problem size remains") ? true : false);
 			constantProblem = theModel.getConstantProblem();
 		}
 		return (!forceIt && constantProblem == null) ? false : true;
