@@ -1,6 +1,8 @@
 package client;
 
 import common.RMIChartData;
+import common.RMIGeneralChartData;
+import common.ChartDataType;
 import java.awt.*;
 import java.awt.event.*;
 import java.awt.BasicStroke;
@@ -16,12 +18,17 @@ import org.jfree.data.xy.DefaultTableXYDataset;
 import org.jfree.data.xy.XYSeriesCollection;
 import org.jfree.data.xy.XYSeries;
 import org.jfree.data.xy.XYDataset;
+import org.jfree.data.category.DefaultCategoryDataset;
+import org.jfree.data.Range;
 import org.jfree.chart.axis.NumberAxis;
+import org.jfree.chart.axis.LogarithmicAxis;
 import org.jfree.chart.renderer.xy.StandardXYItemRenderer;
 import org.jfree.chart.renderer.xy.XYItemRenderer;
 import org.jfree.chart.plot.XYPlot;
+import org.jfree.chart.plot.CategoryPlot;
 import org.jfree.chart.StandardLegend;
 import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
+import org.jfree.chart.renderer.category.LineAndShapeRenderer;
 import org.jfree.chart.labels.StandardXYToolTipGenerator;
 import java.text.DecimalFormat;
 import edu.uoregon.tau.common.ImageExport;
@@ -39,7 +46,7 @@ public class PerfExplorerChart extends PerfExplorerChartWindow {
 		// get the data
 		RMIChartData rawData = server.requestChartData(
 			PerfExplorerModel.getModel(), 
-			RMIChartData.FRACTION_OF_TOTAL);
+			ChartDataType.FRACTION_OF_TOTAL);
 
         DefaultTableXYDataset dataset = new DefaultTableXYDataset();
 		List rowLabels = rawData.getRowLabels();
@@ -64,6 +71,10 @@ public class PerfExplorerChart extends PerfExplorerChartWindow {
             true,                            // tooltips
             false                            // urls
         );
+        XYPlot plot = chart.getXYPlot();
+		NumberAxis axis = new NumberAxis("Percentage of Total Runtime");
+        axis.setRange(new Range(0,100));
+        plot.setRangeAxis(0, axis);
 
 		return new PerfExplorerChart(chart, "Total Runtime Breakdown");
 	}
@@ -73,7 +84,7 @@ public class PerfExplorerChart extends PerfExplorerChartWindow {
 		PerfExplorerConnection server = PerfExplorerConnection.getConnection();
 		// get the data
 		RMIChartData rawData = server.requestChartData(
-			PerfExplorerModel.getModel(), RMIChartData.CORRELATION_DATA);
+			PerfExplorerModel.getModel(), ChartDataType.CORRELATION_DATA);
 
         XYDataset dataset = new CorrelationPlotDataset(rawData, false);
         //JFreeChart chart = ChartFactory.createScatterPlot(
@@ -91,9 +102,20 @@ public class PerfExplorerChart extends PerfExplorerChartWindow {
 
 		customizeChart(chart, rawData.getRows(), true);
         XYPlot plot = chart.getXYPlot();
-        NumberAxis axis2 = new NumberAxis("Inclusive Time for " + (String)rawData.getRowLabels().get(0));
-        axis2.setAutoRangeIncludesZero(false);
-        plot.setRangeAxis(1, axis2);
+
+		if (PerfExplorerModel.getModel().getConstantProblem().booleanValue()) {
+			// log axis, to make the chart more readable
+        	LogarithmicAxis axis = new LogarithmicAxis("Exclusive Time for Event");
+        	axis.setAutoRangeIncludesZero(true);
+        	axis.setAllowNegativesFlag(true);
+        	axis.setLog10TickLabelsFlag(true);
+        	plot.setRangeAxis(0, axis);
+ 		}else {
+			// otherwise, give the inclusive time its own axis
+        	NumberAxis axis2 = new NumberAxis("Inclusive Time for " + (String)rawData.getRowLabels().get(0));
+        	axis2.setAutoRangeIncludesZero(false);
+        	plot.setRangeAxis(1, axis2);
+		}
         XYDataset dataset2 = new CorrelationPlotDataset(rawData, true);
         plot.setDataset(1, dataset2);
         plot.mapDatasetToRangeAxis(1, 1);
@@ -121,7 +143,7 @@ public class PerfExplorerChart extends PerfExplorerChartWindow {
 		// get the data
 		RMIChartData rawData = server.requestChartData(
 			PerfExplorerModel.getModel(), 
-			RMIChartData.RELATIVE_EFFICIENCY);
+			ChartDataType.RELATIVE_EFFICIENCY);
 
         XYSeriesCollection dataset = new XYSeriesCollection();
 		List rowLabels = rawData.getRowLabels();
@@ -158,13 +180,129 @@ public class PerfExplorerChart extends PerfExplorerChartWindow {
 		return new PerfExplorerChart(chart, "Relative Efficiency");
 	}
 
+	/**
+	 * This method will produce a general line chart, with 
+	 * one or more series of data, with anything you want on
+	 * the x-axis, and some measurement on the y-axis.
+	 *
+	 */
+	public static PerfExplorerChart doGeneralChart () {
+		// get the server
+		PerfExplorerConnection server = PerfExplorerConnection.getConnection();
+		// get the data
+		PerfExplorerModel model = PerfExplorerModel.getModel();
+		RMIGeneralChartData rawData = server.requestGeneralChartData(
+			model, ChartDataType.PARAMETRIC_STUDY_DATA);
+
+        PECategoryDataset dataset = new PECategoryDataset();
+		if (rawData.getCategoryType() == Integer.class) {
+			if (model.getChartScalability()) {
+
+				// create an "ideal" line.
+        		dataset.addValue(1.0, "Ideal", new Integer(rawData.getMinimum()));
+        		dataset.addValue(rawData.getMaximum()/rawData.getMinimum(), "Ideal", 
+					new Integer(rawData.getMaximum()));
+
+				// get the baseline values
+				common.RMIGeneralChartData.CategoryDataRow baseline = rawData.getRowData(0);
+
+				// iterate through the values
+				for (int i = 0 ; i < rawData.getRows() ; i++) {
+					common.RMIGeneralChartData.CategoryDataRow row = rawData.getRowData(i);
+					if (!row.series.equals(baseline.series)) {
+						//System.out.println(row.series);
+						baseline = row;
+					}
+					if (model.getConstantProblem().booleanValue()) {
+						//System.out.println("value: " + row.value);
+						
+						double ratio = baseline.categoryInteger.doubleValue() / row.categoryInteger.doubleValue();
+						//System.out.println("ratio: " + ratio);
+						double efficiency = baseline.value/row.value;
+						//System.out.println("efficiency: " + efficiency);
+        				dataset.addValue(efficiency / ratio, row.series, row.categoryInteger);
+					} else {
+        				dataset.addValue(baseline.value / row.value, row.series, row.categoryInteger);
+					}
+				}
+
+				// create an "ideal" line.
+				List keys = dataset.getColumnKeys();
+				for (int i = 0 ; i < keys.size() ; i++) {
+					Integer key = (Integer)keys.get(i);
+        			dataset.addValue(key.doubleValue()/rawData.getMinimum(), "Ideal", key);
+				}
+
+			} else {
+				// iterate through the values
+				for (int i = 0 ; i < rawData.getRows() ; i++) {
+					common.RMIGeneralChartData.CategoryDataRow row = rawData.getRowData(i);
+        			dataset.addValue(row.value / 1000000, row.series, row.categoryInteger);
+				}
+			}
+		} else {
+			// iterate through the values
+			for (int i = 0 ; i < rawData.getRows() ; i++) {
+				common.RMIGeneralChartData.CategoryDataRow row = rawData.getRowData(i);
+        		dataset.addValue(row.value / 1000000, row.series, row.categoryString);
+			}
+		}
+
+        JFreeChart chart = ChartFactory.createLineChart(
+            model.getChartTitle(),  // chart title
+            model.getChartXAxisLabel(),  // domain axis label
+            model.getChartYAxisLabel(),  // range axis label
+            dataset,                         // data
+            PlotOrientation.VERTICAL,        // the plot orientation
+            true,                            // legend
+            true,                            // tooltips
+            false                            // urls
+        );
+		// customize the chart!
+        StandardLegend legend = (StandardLegend) chart.getLegend();
+        legend.setDisplaySeriesShapes(true);
+        
+        // get a reference to the plot for further customisation...
+        CategoryPlot plot = (CategoryPlot)chart.getPlot();
+     
+        //StandardXYItemRenderer renderer = (StandardXYItemRenderer) plot.getRenderer();
+		LineAndShapeRenderer renderer = (LineAndShapeRenderer)plot.getRenderer();
+        renderer.setDefaultShapesFilled(true);
+        renderer.setDrawShapes(true);
+        renderer.setDrawLines(true);
+        renderer.setItemLabelsVisible(true);
+		if (model.getChartScalability()) {
+			//renderer.setDrawShapes(false);
+		}
+
+		for (int i = 0 ; i < rawData.getRows() ; i++) {
+			renderer.setSeriesStroke(i, new BasicStroke(2.0f));
+		}
+
+        // change the auto tick unit selection to integer units only...
+        NumberAxis rangeAxis = (NumberAxis) plot.getRangeAxis();
+        rangeAxis.setStandardTickUnits(NumberAxis.createIntegerTickUnits());
+		rangeAxis.setAutoRangeIncludesZero(true);
+
+		if (model.getChartLogYAxis()) {
+        	LogarithmicAxis axis = new LogarithmicAxis(
+				PerfExplorerModel.getModel().getChartYAxisLabel());
+        	axis.setAutoRangeIncludesZero(true);
+        	axis.setAllowNegativesFlag(true);
+        	axis.setLog10TickLabelsFlag(true);
+        	plot.setRangeAxis(0, axis);
+ 		}
+
+		return new PerfExplorerChart(chart, "General Chart");
+	}
+
 	public static PerfExplorerChart doEfficiencyEventsChart () {
 		// get the server
 		PerfExplorerConnection server = PerfExplorerConnection.getConnection();
 		// get the data
 		RMIChartData rawData = server.requestChartData(
 			PerfExplorerModel.getModel(), 
-			RMIChartData.RELATIVE_EFFICIENCY_EVENTS);
+			ChartDataType.RELATIVE_EFFICIENCY_EVENTS);
 
         XYSeriesCollection dataset = new XYSeriesCollection();
 		List rowLabels = rawData.getRowLabels();
@@ -207,7 +345,7 @@ public class PerfExplorerChart extends PerfExplorerChartWindow {
 		// get the data
 		RMIChartData rawData = server.requestChartData(
 			PerfExplorerModel.getModel(), 
-			RMIChartData.RELATIVE_EFFICIENCY_ONE_EVENT);
+			ChartDataType.RELATIVE_EFFICIENCY_ONE_EVENT);
 
         XYSeriesCollection dataset = new XYSeriesCollection();
 		List rowLabels = rawData.getRowLabels();
@@ -250,7 +388,7 @@ public class PerfExplorerChart extends PerfExplorerChartWindow {
 		// get the data
 		RMIChartData rawData = server.requestChartData(
 			PerfExplorerModel.getModel(), 
-			RMIChartData.RELATIVE_EFFICIENCY);
+			ChartDataType.RELATIVE_EFFICIENCY);
 
         XYSeriesCollection dataset = new XYSeriesCollection();
 
@@ -305,7 +443,7 @@ public class PerfExplorerChart extends PerfExplorerChartWindow {
 		// get the data
 		RMIChartData rawData = server.requestChartData(
 			PerfExplorerModel.getModel(), 
-			RMIChartData.RELATIVE_EFFICIENCY_EVENTS);
+			ChartDataType.RELATIVE_EFFICIENCY_EVENTS);
 
         XYSeriesCollection dataset = new XYSeriesCollection();
 		List rowLabels = rawData.getRowLabels();
@@ -358,7 +496,7 @@ public class PerfExplorerChart extends PerfExplorerChartWindow {
 		// get the data
 		RMIChartData rawData = server.requestChartData(
 			PerfExplorerModel.getModel(), 
-			RMIChartData.RELATIVE_EFFICIENCY_ONE_EVENT);
+			ChartDataType.RELATIVE_EFFICIENCY_ONE_EVENT);
 
         XYSeriesCollection dataset = new XYSeriesCollection();
 		List rowLabels = rawData.getRowLabels();
@@ -412,7 +550,7 @@ public class PerfExplorerChart extends PerfExplorerChartWindow {
 		// get the data
 		RMIChartData rawData = server.requestChartData(
 			PerfExplorerModel.getModel(), 
-			RMIChartData.RELATIVE_EFFICIENCY);
+			ChartDataType.RELATIVE_EFFICIENCY);
 
 		int timesteps = Integer.parseInt(PerfExplorerModel.getModel().getTotalTimesteps());
         XYSeriesCollection dataset = new XYSeriesCollection();
@@ -443,17 +581,71 @@ public class PerfExplorerChart extends PerfExplorerChartWindow {
 		return new PerfExplorerChart(chart, "Timesteps per Second");
 	}
 
+	public static PerfExplorerChart doTotalTimeChart () {
+		// get the server
+		PerfExplorerConnection server = PerfExplorerConnection.getConnection();
+		// get the data
+		RMIChartData rawData = server.requestChartData(
+			PerfExplorerModel.getModel(), 
+			ChartDataType.RELATIVE_EFFICIENCY);
+
+		int decreasing = 0, total = 0;
+		double lastValue = 0.0;
+        XYSeriesCollection dataset = new XYSeriesCollection();
+		List rowLabels = rawData.getRowLabels();
+		for (int y = 0 ; y < rawData.getRows() ; y++) {
+			List row = rawData.getRowData(y);
+        	XYSeries s = new XYSeries((String)rowLabels.get(y), true, false);
+			total = total + row.size();
+			for (int x = 0 ; x < row.size() ; x++) {
+				double[] values = (double[])(row.get(x));
+        		s.add(values[0], values[1]);
+				if (lastValue > values[1])
+					decreasing++;
+				lastValue = values[1];
+			}
+        	dataset.addSeries(s);
+		}
+
+        JFreeChart chart = ChartFactory.createXYLineChart(
+            "Total Execution:" +
+			PerfExplorerModel.getModel().getMetricName(),  // chart title
+            "Number of Processors",          // domain axis label
+            PerfExplorerModel.getModel().getMetricNameUnits(),     // range axis label
+            dataset,                         // data
+            PlotOrientation.VERTICAL,        // the plot orientation
+            true,                            // legend
+            true,                            // tooltips
+            false                            // urls
+        );
+
+		customizeChart(chart, rawData.getRows(), false);
+
+		// if decreasing, assume strong scaling, and do Log axis for range.
+		if (decreasing > total/2) {
+        	XYPlot plot = chart.getXYPlot();
+        	LogarithmicAxis axis = new LogarithmicAxis(
+				PerfExplorerModel.getModel().getMetricName());
+        	axis.setAutoRangeIncludesZero(true);
+        	axis.setAllowNegativesFlag(true);
+        	axis.setLog10TickLabelsFlag(true);
+        	plot.setRangeAxis(0, axis);
+ 		}
+
+		return new PerfExplorerChart(chart, "Total Execution Time");
+	}
+
 	public static PerfExplorerChart doCommunicationChart () {
 		// get the server
 		PerfExplorerConnection server = PerfExplorerConnection.getConnection();
 		// get the data
 		RMIChartData rawData1 = server.requestChartData(
 			PerfExplorerModel.getModel(), 
-			RMIChartData.TOTAL_FOR_GROUP);
+			ChartDataType.TOTAL_FOR_GROUP);
 
 		RMIChartData rawData2 = server.requestChartData(
 			PerfExplorerModel.getModel(), 
-			RMIChartData.RELATIVE_EFFICIENCY);
+			ChartDataType.RELATIVE_EFFICIENCY);
 
         XYSeriesCollection dataset = new XYSeriesCollection();
 		List rowLabels = rawData1.getRowLabels();
@@ -493,7 +685,7 @@ public class PerfExplorerChart extends PerfExplorerChartWindow {
 		// get the data
 		RMIChartData rawData = server.requestChartData(
 			PerfExplorerModel.getModel(), 
-			RMIChartData.RELATIVE_EFFICIENCY_PHASES);
+			ChartDataType.RELATIVE_EFFICIENCY_PHASES);
 
         XYSeriesCollection dataset = new XYSeriesCollection();
 		List rowLabels = rawData.getRowLabels();
@@ -533,7 +725,7 @@ public class PerfExplorerChart extends PerfExplorerChartWindow {
 		// get the data
 		RMIChartData rawData = server.requestChartData(
 			PerfExplorerModel.getModel(), 
-			RMIChartData.RELATIVE_EFFICIENCY_PHASES);
+			ChartDataType.RELATIVE_EFFICIENCY_PHASES);
 
         XYSeriesCollection dataset = new XYSeriesCollection();
 		List rowLabels = rawData.getRowLabels();
@@ -584,7 +776,7 @@ public class PerfExplorerChart extends PerfExplorerChartWindow {
 		// get the data
 		RMIChartData rawData = server.requestChartData(
 			PerfExplorerModel.getModel(), 
-			RMIChartData.FRACTION_OF_TOTAL_PHASES);
+			ChartDataType.FRACTION_OF_TOTAL_PHASES);
 
         DefaultTableXYDataset dataset = new DefaultTableXYDataset();
 		List rowLabels = rawData.getRowLabels();
