@@ -1,11 +1,13 @@
 package client;
 
-import server.AnalysisTaskWrapper;
+import common.EngineType;
 import common.PerfExplorerOutput;
+import edu.uoregon.tau.common.Utility;
 import jargs.gnu.CmdLineParser;
 import javax.swing.*;
 import java.awt.*;
 import java.awt.event.*;
+import java.net.URL;
 
 public class PerfExplorerClient extends JFrame {
 	private static String USAGE = "Usage: PerfExplorerClient [{-h,--help}] {-c,--configfile}=<config_file> [{-s,--standalone}] [{-e,--engine}=<analysis_engine>]\n  where analysis_engine = R or Weka";
@@ -21,10 +23,10 @@ public class PerfExplorerClient extends JFrame {
 		return listener;
 	}
 
-	private PerfExplorerClient (boolean standalone, String configFile,
-	int analysisEngine, boolean quiet) {
-		super("PerfExplorer Client");
-		PerfExplorerOutput.initialize(quiet);
+	PerfExplorerClient (boolean standalone, String configFile,
+	EngineType analysisEngine, boolean quiet) {
+		super("TAU: PerfExplorer Client");
+		PerfExplorerOutput.setQuiet(quiet);
 		PerfExplorerConnection.setStandalone(standalone);
 		PerfExplorerConnection.setConfigFile(configFile);
 		PerfExplorerConnection.setAnalysisEngine(analysisEngine);
@@ -84,6 +86,9 @@ public class PerfExplorerClient extends JFrame {
         } else {
             this.setLocation(savedPosition);
         }
+    	URL url = Utility.getResource("tau32x32.gif");
+    	if (url != null)
+    		setIconImage(Toolkit.getDefaultToolkit().getImage(url));
 		mainFrame = this;
 	}
 
@@ -129,7 +134,7 @@ public class PerfExplorerClient extends JFrame {
 		String engine = (String) parser.getOptionValue(engineOpt);
 		Boolean quiet = (Boolean) parser.getOptionValue(quietOpt);
 
-		int analysisEngine = AnalysisTaskWrapper.WEKA_ENGINE;
+		EngineType analysisEngine = EngineType.WEKA;
 
 		if (help != null && help.booleanValue()) {
 			System.err.println(USAGE);
@@ -148,27 +153,21 @@ public class PerfExplorerClient extends JFrame {
 				System.err.println(USAGE);
 				System.exit(-1);
 			}
-			if (engine == null) {
+			try {
+				analysisEngine = EngineType.getType(engine);
+			} catch (Exception e) {
 				System.err.println("Please enter a valid engine type.");
 				System.err.println(USAGE);
 				System.exit(-1);
-			} else if (engine.equalsIgnoreCase("R")) {
-				analysisEngine = AnalysisTaskWrapper.RPROJECT_ENGINE;
-			} else if (engine.equalsIgnoreCase("weka")) {
-				analysisEngine = AnalysisTaskWrapper.WEKA_ENGINE;
-			} else {
-				System.err.println(USAGE);
-				System.exit(-1);
 			}
 		}
 
-
-	/*
+/*
 		try {
 			UIManager.setLookAndFeel(
 				UIManager.getCrossPlatformLookAndFeelClassName());
 		} catch (Exception e) { }
-	*/
+*/
 
 		JFrame frame = new PerfExplorerClient(standalone.booleanValue(),
 			configFile, analysisEngine, quiet.booleanValue());
