@@ -2,7 +2,7 @@ package client;
 
 import java.rmi.*;
 import common.*;
-import server.*;
+import server.PerfExplorerServer;
 import java.util.*;
 import javax.swing.*;
 
@@ -13,7 +13,7 @@ public class PerfExplorerConnection {
     private RMIPerfExplorer server = null;
     private static boolean standalone = false;
     private static String configFile = null;
-    private static int analysisEngine = 0;
+    private static EngineType analysisEngine = EngineType.WEKA;
 
     private PerfExplorerConnection () {
 		makeConnection();
@@ -49,7 +49,7 @@ public class PerfExplorerConnection {
 	PerfExplorerConnection.configFile = configFile;
     }
 	
-    public static void setAnalysisEngine(int analysisEngine) {
+    public static void setAnalysisEngine(EngineType analysisEngine) {
 	PerfExplorerConnection.analysisEngine = analysisEngine;
     }
 
@@ -165,7 +165,7 @@ public class PerfExplorerConnection {
 	}
     }
 
-    public RMIChartData requestChartData(PerfExplorerModel model, int dataType) {
+    public RMIChartData requestChartData(PerfExplorerModel model, ChartDataType dataType) {
 	RMIChartData data = null;
 	try {
 	    data = server.requestChartData(model, dataType);
@@ -175,6 +175,16 @@ public class PerfExplorerConnection {
 	return data;
     }
 
+    public RMIGeneralChartData requestGeneralChartData(PerfExplorerModel model, ChartDataType dataType) {
+	RMIGeneralChartData data = null;
+	try {
+	    data = server.requestGeneralChartData(model, dataType);
+	} catch (RemoteException e) {
+	    handleError(e, "requestGeneralChartData(" + model.toString() + ")");
+	}
+	return data;
+    }
+
     public List getPotentialGroups(PerfExplorerModel model) {
 	List groups = null;
 	try {
@@ -275,4 +285,33 @@ public class PerfExplorerConnection {
 	return results;
     }
 
+	public String getConnectionString() {
+		String conn = null;
+		try {
+	    	conn = server.getConnectionString();
+		} catch (RemoteException e) {
+	    	handleError(e, "getConnectionString()");
+		}
+		return conn;
+	}
+
+    public ListIterator getEventList(int trialID, int metricIndex) {
+	ListIterator tmpIterator = null;
+	try {
+	    tmpIterator = server.getEventList(trialID, metricIndex).listIterator();
+	} catch (RemoteException e) {
+	    handleError(e, "getEventList(" + trialID + ")");
+	}
+	return tmpIterator;
+    }
+
+	public List getTrialList(String criteria) {
+		List list = null;
+		try {
+			list = server.getTrialList(criteria);
+		} catch (RemoteException e) {
+	    	handleError(e, "getTrialList(" + criteria + ")");
+		}
+		return list;
+	}
 }
