@@ -12,6 +12,8 @@ import java.io.IOException;
 import java.util.List;
 import javax.swing.JFrame;
 import java.lang.Math;
+import org.jfree.data.Range;
+import org.jfree.chart.axis.NumberAxis;
 import org.jfree.chart.ChartFactory;
 import org.jfree.chart.ChartPanel;
 import org.jfree.chart.JFreeChart;
@@ -37,10 +39,16 @@ public class PerfExplorerHistogramChart extends PerfExplorerChartWindow {
 		// for each event, get the variation across all threads.
 		PerfExplorerConnection server = PerfExplorerConnection.getConnection();
 		// get the data
-		RMIChartData data = server.requestChartData(PerfExplorerModel.getModel(), RMIChartData.IQR_DATA);
+		PerfExplorerModel model = PerfExplorerModel.getModel();
+        Object selection = model.getCurrentSelection();
+		RMIChartData data = null;
+        if (selection instanceof RMISortableIntervalEvent) {
+			data = server.requestChartData(model, ChartDataType.DISTRIBUTION_DATA);
+		} else {
+			data = server.requestChartData(model, ChartDataType.IQR_DATA);
+		}
 
 		// build the chart
-		//BoxAndWhiskerCategoryDataset dataset = createDataset(data);
         IntervalXYDataset dataset = createDataset(data);
         JFreeChart chart = createChart(dataset);
         
@@ -111,15 +119,18 @@ public class PerfExplorerHistogramChart extends PerfExplorerChartWindow {
      */
     private static JFreeChart createChart(IntervalXYDataset dataset) {
         JFreeChart chart = ChartFactory.createHistogram(
-            "Significant (>2.0% of runtime) Event Histograms", 
-            null, 
-            null, 
+            "TAU/PerfExplorer: Significant (>2.0% of runtime) Event Histograms", 
+            "Percentiles", 
+            "Count", 
             dataset, 
             PlotOrientation.VERTICAL, 
             true, 
             false, 
             false
         );
+		NumberAxis axis = new NumberAxis("Percentiles");
+		axis.setRange(new Range(0,1.0));
+        chart.getXYPlot().setDomainAxis(0, axis);
         chart.getXYPlot().setForegroundAlpha(0.75f);
         return chart;
     }
