@@ -1,6 +1,7 @@
 package client;
 
 import javax.swing.*;
+
 import java.awt.event.*;
 
 public class PerfExplorerMainJMenuBar extends JMenuBar {
@@ -36,6 +37,20 @@ public class PerfExplorerMainJMenuBar extends JMenuBar {
 		quitServerItem.addActionListener(listener);
 		fileMenu.add(quitServerItem);
 
+/*
+		try {
+			UIManager.LookAndFeelInfo[] info = UIManager.getInstalledLookAndFeels();
+			JMenuItem item[] = new JMenuItem[info.length];
+			for (int i = 0 ; i < info.length ; i++) {
+				//System.out.println(info[i].getClassName() + ": " + info[i].getName());
+				//Add a menu item.
+				item[i] = new JMenuItem(PerfExplorerActionListener.LOOK_AND_FEEL + info[i].getName());
+				item[i].addActionListener(listener);
+				fileMenu.add(item[i]);
+			}
+		} catch (Exception e) { }
+*/
+
 		this.add(fileMenu);
 	}
 
@@ -90,6 +105,11 @@ public class PerfExplorerMainJMenuBar extends JMenuBar {
 		histogramItem.addActionListener(listener);
 		visualizationMenu.add(histogramItem);
 
+		//Add a menu item.
+		JMenuItem probabilityPlotItem = new JMenuItem(
+				PerfExplorerActionListener.DO_PROBABILITY_PLOT);
+		probabilityPlotItem.addActionListener(listener);
+		visualizationMenu.add(probabilityPlotItem);
 		
 		this.add(visualizationMenu);
 	}
@@ -179,6 +199,12 @@ public class PerfExplorerMainJMenuBar extends JMenuBar {
 		problemItem.addActionListener(listener);
 		chartMenu.add(problemItem);
 
+		//Add a menu item.
+		JMenuItem totalTime = new JMenuItem(
+			PerfExplorerActionListener.TOTAL_TIME_CHART);
+		totalTime.addActionListener(listener);
+		chartMenu.add(totalTime);
+
 		//Add a menu item.
 		JMenuItem timesteps = new JMenuItem(
 			PerfExplorerActionListener.TIMESTEPS_CHART);
