@@ -9,9 +9,12 @@ package client;
 
 import java.awt.Point;
 import java.awt.Dimension;
+import java.awt.Toolkit;
+
 import javax.swing.JFrame;
 import javax.swing.JTable;
 import javax.swing.JScrollPane;
+import javax.swing.JScrollBar;
 import common.RMIVarianceData;
 
 /**
@@ -34,7 +37,7 @@ public class PerfExplorerVariation {
 
 	private static void displayResults(RMIVarianceData data) {
         // Create and set up the window.
-        JFrame frame = new JFrame("Summarization Analysis");
+        JFrame frame = new JFrame("TAU/PerfExplorer: Data Summary");
         //frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
 
         // arrange the data
@@ -47,27 +50,33 @@ public class PerfExplorerVariation {
         TableSorter sorter = new TableSorter(new MyTableModel(columns, mydata)); //ADDED THIS
         JTable table = new JTable(sorter);             //NEW
         sorter.setTableHeader(table.getTableHeader()); //ADDED THIS
-        
+       
         // Make the table vertically scrollable
         JScrollPane scrollPane = new JScrollPane(table);
+		JScrollBar jScrollBar = scrollPane.getVerticalScrollBar();
+		jScrollBar.setUnitIncrement(35);
         
         //Window Stuff.
         int windowWidth = 800;
         int windowHeight = 400;
+        int xPosition = 0;
+        int yPosition = 0;
         
         //Grab paraProfManager position and size.
-        Point parentPosition = PerfExplorerClient.getMainFrame().getLocationOnScreen();
-        Dimension parentSize = PerfExplorerClient.getMainFrame().getSize();
-        int parentWidth = parentSize.width;
-        int parentHeight = parentSize.height;
-        
-        //Set the window to come up in the center of the screen.
-        int xPosition = (parentWidth - windowWidth) / 2;
-        int yPosition = (parentHeight - windowHeight) / 2;
-
-        xPosition = (int) parentPosition.getX() + xPosition;
-        yPosition = (int) parentPosition.getY() + yPosition;
-
+        JFrame main = PerfExplorerClient.getMainFrame();
+        if (main != null) {
+	        Point parentPosition = PerfExplorerClient.getMainFrame().getLocationOnScreen();
+	        Dimension parentSize = PerfExplorerClient.getMainFrame().getSize();
+	        int parentWidth = parentSize.width;
+	        int parentHeight = parentSize.height;
+	        
+	        //Set the window to come up in the center of the screen.
+	        xPosition = (parentWidth - windowWidth) / 2;
+	        yPosition = (parentHeight - windowHeight) / 2;
+	
+	        xPosition = (int) parentPosition.getX() + xPosition;
+	        yPosition = (int) parentPosition.getY() + yPosition;
+        }
         frame.setLocation(xPosition, yPosition);
         frame.setSize(new java.awt.Dimension(windowWidth, windowHeight));
         scrollPane.setPreferredSize(new java.awt.Dimension(windowWidth, windowHeight));
