@@ -11,12 +11,15 @@ declare -i group_C=3
 declare -i disablePdtStep=$FALSE
 declare -i hasAnOutputFile=$FALSE
 declare -i fortranParserDefined=$FALSE
+declare -i gfparseUsed=$FALSE
+declare -i pdtUsed=$FALSE
 declare -i isForCompilation=$FALSE
 declare -i hasAnObjectOutputFile=$FALSE
 declare -i hasMpi=$TRUE
 declare -i needToCleanPdbInstFiles=$TRUE
 declare -i pdbFileSpecified=$FALSE
 declare -i optResetUsed=$FALSE
+declare -i optDetectMemoryLeaks=$FALSE
 
 declare -i isVerbose=$FALSE
 declare -i isDebug=$FALSE
@@ -35,35 +38,55 @@ declare -i counterForOutput=-10
 declare -i counterForOptions=0
 declare -i temp=0
 
+declare -i preprocess=$FALSE
+declare -i revertOnError=$TRUE
+
+preprocessorOpts="-P  -traditional-cpp"
+
 
 printUsage () {
 	echo -e "Usage: tau_compiler.sh"
 	echo -e "  -optVerbose\t\t\tTurn on verbose debugging message"
+	echo -e "  -optDetectMemoryLeaks\t\tTrack mallocs/frees using TAU's memory wrapper"
 	echo -e "  -optPdtDir=\"\"\t\t\tPDT architecture directory. Typically \$(PDTDIR)/\$(PDTARCHDIR)"
 	echo -e "  -optPdtF95Opts=\"\"\t\tOptions for Fortran parser in PDT (f95parse)"
-	echo -e "  -optPdtF95Reset=\"\"\tReset options to the Fortran parser to the given list"
+	echo -e "  -optPdtF95Reset=\"\"\t\tReset options to the Fortran parser to the given list"
 	echo -e "  -optPdtCOpts=\"\"\t\tOptions for C parser in PDT (cparse). Typically \$(TAU_MPI_INCLUDE) \$(TAU_INCLUDE) \$(TAU_DEFS)"
 	echo -e "  -optPdtCReset=\"\"\t\tReset options to the C parser to the given list"
 	echo -e "  -optPdtCxxOpts=\"\"\t\tOptions for C++ parser in PDT (cxxparse). Typically \$(TAU_MPI_INCLUDE) \$(TAU_INCLUDE) \$(TAU_DEFS)"
 	echo -e "  -optPdtCReset=\"\"\t\tReset options to the C++ parser to the given list"
 	echo -e "  -optPdtF90Parser=\"\"\t\tSpecify a different Fortran parser. For e.g., f90parse instead of f95parse"
+	echo -e "  -optPdtGnuFortranParser\tSpecify the GNU gfortran PDT parser gfparse instead of f95parse"
 	echo -e "  -optPdtUser=\"\"\t\tOptional arguments for parsing source code"
 	echo -e "  -optTauInstr=\"\"\t\tSpecify location of tau_instrumentor. Typically \$(TAUROOT)/\$(CONFIG_ARCH)/bin/tau_instrumentor"
+	echo -e "  -optPreProcess\t\tPreprocess the source code before parsing. Uses /usr/bin/cpp -P by default."
+	echo -e "  -optCPP=\"\"\t\t\tSpecify an alternative preprocessor and pre-process the sources."
+	echo -e "  -optCPPOpts=\"\"\t\tSpecify additional options to the C pre-processor."
+	echo -e "  -optCPPReset=\"\"\t\tReset C preprocessor options to the specified list."
 	echo -e "  -optTauSelectFile=\"\"\t\tSpecify selective instrumentation file for tau_instrumentor"
 	echo -e "  -optPDBFile=\"\"\t\tSpecify PDB file for tau_instrumentor. Skips parsing stage."
 	echo -e "  -optTau=\"\"\t\t\tSpecify options for tau_instrumentor"
-	echo -e "  -optCompile=\"\"\t\tOptions passed to the compiler. Typically \$(TAU_MPI_INCLUDE) \$(TAU_INCLUDE) \$(TAU_DEFS)"
+	echo -e "  -optCompile=\"\"\t\tOptions passed to the compiler by the user."
+	echo -e "  -optTauDefs=\"\"\t\tOptions passed to the compiler by TAU. Typically \$(TAU_DEFS)"
+	echo -e "  -optTauIncludes=\"\"\t\tOptions passed to the compiler by TAU. Typically \$(TAU_MPI_INCLUDE) \$(TAU_INCLUDE)"
+	echo -e "  -optIncludeMemory=\"\"\t\tFlags for replacement of malloc/free. Typically -I\$(TAU_DIR)/include/Memory"
 	echo -e "  -optReset=\"\"\t\t\tReset options to the compiler to the given list"
 	echo -e "  -optLinking=\"\"\t\tOptions passed to the linker. Typically \$(TAU_MPI_FLIBS) \$(TAU_LIBS) \$(TAU_CXXLIBS)"
 	echo -e "  -optLinkReset=\"\"\t\tReset options to the linker to the given list"
 	echo -e "  -optTauCC=\"<cc>\"\t\tSpecifies the C compiler used by TAU"
 	echo -e "  -optOpariTool=\"<path/opari>\"\tSpecifies the location of the Opari tool"
-	echo -e "  -optOpariDir=\"<path>\"\tSpecifies the location of the Opari directory"
+	echo -e "  -optOpariDir=\"<path>\"\t\tSpecifies the location of the Opari directory"
 	echo -e "  -optOpariOpts=\"\"\t\tSpecifies optional arguments to the Opari tool"
 	echo -e "  -optOpariReset=\"\"\t\tResets options passed to the Opari tool"
 	echo -e "  -optNoMpi\t\t\tRemoves -l*mpi* libraries during linking (default)"
 	echo -e "  -optMpi\t\t\tDoes not remove -l*mpi* libraries during linking"
+	echo -e "  -optNoRevert\t\t\tExit on error. Does not revert to the original compilation rule on error."
+	echo -e "  -optRevert\t\t\tRevert to the original compilation rule on error (default)."
 	echo -e "  -optKeepFiles\t\t\tDoes not remove intermediate .pdb and .inst.* files" 
+	echo -e "  -optAppCC=\"<cc>\"\t\tSpecifies the fallback C compiler."
+	echo -e "  -optAppCXX=\"<cxx>\"\t\tSpecifies the fallback C++ compiler."
+	echo -e "  -optAppF90=\"<f90>\"\t\tSpecifies the fallback F90 compiler."
+
 	if [ $1 == 0 ]; then #Means there are no other option passed with the myscript. It is better to exit then.
 		exit
 	fi
@@ -101,7 +124,7 @@ printError() {
 evalWithDebugMessage() {
 	echoIfVerbose "\n\nDebug: $2"
 	echoIfVerbose "Executing>  $1"
-	$1
+	eval $1
 # NEVER add additional statements below $1, users of this function need the return code ($?)
 #	echoIfVerbose "....."
 }
@@ -113,36 +136,49 @@ fi
 
 
 
+# We grab the first argument and remember it as the compiler
+# if the user doesn't specify a fallback compiler, we use it
+foundFirstArg=0
+compilerSpecified=""
+
 #All the script options must be passed as -opt*
 #including verbose option [as -optVerbose]. The reason being
 #script assumes that all any tokens passed after -opt* sequenece
 #constitute the regular command, with the first command (immediately) 
 #after the sequence, being the compiler.  In this "for" loops, the 
 #regular command is being read.
-for name in "$@"; do
+for arg in "$@"; do
 
-	case $name in
+	case $arg in
 
 	-opt*)
 		;;
 
 	*)
 		if [ $tempCounter == 0 ]; then
-			CMD=$name
+			CMD=$arg
 			#The first command (immediately) after the -opt sequence is the compiler.
 		fi
-		regularCmd="$regularCmd  $name"	
+
+                # Thanks to Bernd Mohr for the following that handles quotes and spaces (see configure for explanation)
+		modarg=`echo "x$arg" | sed -e 's/^x//' -e 's/"/\\\"/g' -e s,\',%@%\',g -e 's/%@%/\\\/g' -e 's/ /\\\ /g'`
+		THEARGS="$THEARGS $modarg"
+
+		if [ $foundFirstArg == 0 ]; then
+		    foundFirstArg=1
+		    compilerSpecified="$modarg"
+		else
+		    regularCmd="$regularCmd $modarg"	
+		fi
+
 		tempCounter=tempCounter+1
 		;;
-
 	esac
 done
 echoIfDebug "\nRegular command passed is --  $regularCmd "; 
 echoIfDebug "The compiler being read is $CMD \n"
 
 
-
-
 ####################################################################
 # Initialize optOpariOpts 
 ####################################################################
@@ -167,6 +203,38 @@ for arg in "$@"
 			counterForOptions=counterForOptions+1
 			case $arg in
 
+			-optPreProcess)
+				preprocess=$TRUE
+			# if a preprocessor has not been specified yet, use
+			# the default C preprocessor
+				if [ "x$preprocessor" == "x" ]; then
+				  preprocessor=/usr/bin/cpp
+				fi
+				if [ ! -x $preprocessor ]; then
+				  preprocessor=`which cpp`
+				fi
+				if [ ! -x $preprocessor ]; then
+ 				  echo "ERROR: No working cpp found in path. Please specify -optCPP=<full_path_to_cpp> and recompile"
+				fi
+				# Default options 	
+				echoIfDebug "\tPreprocessing turned on. preprocessor used is $preprocessor with options $preprocessorOpts"
+				;;
+
+			-optCPP=*)
+                                preprocessor=${arg#"-optCPP="}
+				preprocess=$TRUE
+				echoIfDebug "\tPreprocessing $preprocess. preprocessor used is $preprocessor with options $preprocessorOpts"
+				;;
+
+			-optCPPOpts=*)
+				preprocessorOpts="$preprocessorOpts ${arg#"-optCPPOpts="}"
+				echoIfDebug "\tPreprocessing $preprocess. preprocessor used is $preprocessor with options $preprocessorOpts"
+				;;
+			-optCPPReset=*)
+                                preprocessorOpts="${arg#"-optCPPReset="}"
+				echoIfDebug "\tPreprocessing $preprocess. preprocessor used is $preprocessor with options $preprocessorOpts"
+				;;
+
 			-optPdtF90Parser*)
 				#Assumption: This should be passed with complete path to the
 				#parser executable. However, if the path is defined in the
@@ -189,9 +257,17 @@ for arg in "$@"
 				if [ ! -d $optPdtDir ]; then
 				  disablePdtStep=$TRUE
 				  echoIfDebug "PDT is not configured."
+				else
+				  pdtUsed=$TRUE
 				fi 
 				;;
 
+			-optPdtGnuFortranParser*)
+				fortranParserDefined=$TRUE
+				pdtParserF="$optPdtDir""/gfparse"
+				gfparseUsed=$TRUE
+				;;
+
 			-optPdtF95Opts*)
 				#reads all the options needed for Parsing a Fortran file
 				#e.g ${FFLAGS}, ${FCPPFLAGS}. If one needs to pass any
@@ -252,9 +328,10 @@ for arg in "$@"
 					
 				;;
 
-			-optTau*)
-				optTau=${arg#"-optTau="}
+			-optTau=*)
+				optTauInstrOpts=${arg#"-optTau="}
 				echoIfDebug "\tTau Options are: $optTau"
+				optTau="$optTauInstrOpts $optTau"
 				;;
 
 			-optLinking*)
@@ -266,6 +343,30 @@ for arg in "$@"
 				optLinking=${arg#"-optLinkReset="}
 				echoIfDebug "\tLinking Options are: $optLinking"
 				;;
+			-optTauDefs=*)
+				optTauDefs="${arg#"-optTauDefs="}"
+				echoIfDebug "\tCompiling Defines Options from TAU are: $optTauDefs"
+				echoIfDebug "\tFrom optTauDefs: $optTauDefs"
+				optDefs="$optDefs $optTauDefs"
+				;;
+			-optTauIncludes=*)
+				optTauIncludes="${arg#"-optTauIncludes="}"
+				echoIfDebug "\tCompiling Include Options from TAU are: $optTauIncludes"
+				echoIfDebug "\tFrom optTauIncludes: $optTauIncludes"
+				optIncludes="$optIncludes $optTauIncludes"
+				;;
+			-optIncludeMemory=*)
+				optIncludeMemory="${arg#"-optIncludeMemory="}"
+				echoIfDebug "\tCompiling Include Memory Options from TAU are: $optIncludeMemory"
+				echoIfDebug "\tFrom optIncludeMemory: $optIncludeMemory"
+				;;
+			-optDetectMemoryLeaks)
+				optDetectMemoryLeaks=$TRUE
+				optIncludes="$optIncludes $optIncludeMemory"
+				optTau="-memory $optTau"
+				echoIfDebug "\Including Memory directory for malloc/free replacement and calling tau_instrumentor with -memory"
+				echoIfDebug "\tFrom optIncludes: $optIncludes"
+				;;
 			-optCompile*)
 				optCompile="${arg#"-optCompile="} $optCompile"
 				echoIfDebug "\tCompiling Options are: $optCompile"
@@ -299,6 +400,10 @@ for arg in "$@"
 				echoIfDebug "\tVerbose Option is being passed"
 				isVerbose=$TRUE
 				;;
+			-optQuiet)
+				echoIfDebug "\tQuiet Option is being passed"
+				isVerbose=$FALSE
+				;;
 
 
 			-optNoMpi*)
@@ -311,6 +416,14 @@ for arg in "$@"
 				hasMpi=$TRUE
 				;;
 
+			-optNoRevert*)
+				revertOnError=$FALSE
+				;;
+
+			-optRevert*)
+				revertOnError=$TRUE
+				;;
+
 			-optKeepFiles*)
 				#By default this is False. 
 				#removes *.inst.* and *.pdb
@@ -340,6 +453,20 @@ for arg in "$@"
 				echoIfDebug "\tOpari Tool used: $optOpariOpts"
 				;;
 
+			-optAppCC*)
+				optAppCC="${arg#"-optAppCC="}"
+				echoIfDebug "\tFallback C Compiler: $optAppCC"
+				;;
+
+			-optAppCXX*)
+				optAppCXX="${arg#"-optAppCXX="}"
+				echoIfDebug "\tFallback C++ Compiler: $optAppCXX"
+				;;
+
+			-optAppF90*)
+				optAppF90="${arg#"-optAppF90="}"
+				echoIfDebug "\tFallback Fortran Compiler: $optAppF90"
+				;;
 
 			esac #end case for parsing script Options
 			;;
@@ -373,26 +500,54 @@ for arg in "$@"
 			fi
 			echoIfDebug "Using Fortran Parser"
 			if [ $optResetUsed == $FALSE ]; then
-			  optCompile="`echo $optCompile | sed -e 's/ -D[^ ]*//g'`"
-			  echoIfDebug "Resetting optCompile (removing -D* ): $optCompile"
+			  #optCompile="`echo $optCompile | sed -e 's/ -D[^ ]*//g'`"
+			  #echoIfDebug "Resetting optCompile (removing -D* ): $optCompile"
+			  #optCompile="`echo $optCompile | sed -e 's/ -[^I][^ ]*//g'`"
+			  echoIfDebug "Keeping optCompile as it is: $optCompile"
+			  # Do not remove anything! Let TAU pass in -optCompile that we ignore 
 			fi
 			groupType=$group_f_F
 			;;
 
-		-I*|-D*)
-			optPdtCFlags="$arg $optPdtCFlags"
-			optPdtCxxFlags="$arg $optPdtCxxFlags"
-			optPdtF95="$arg $optPdtF95"
-			optCompile="$arg $optCompile"
-			optIncludeDefs="$arg $optIncludeDefs"
+		-WF,-D*)
+			theDefine=${arg#"-WF,"}
+ 		        theDefine=`echo "x$theDefine" | sed -e 's/^x//' -e 's/"/\\\"/g' -e 's/'\''/'\\\'\''/g' -e 's/ /\\\ /g'`
+			optPdtCFlags="$theDefine $optPdtCFlags"
+			optPdtCxxFlags="$theDefine $optPdtCxxFlags"
+			optPdtF95="$theDefine $optPdtF95"
+		        mod_arg=`echo "x$arg" | sed -e 's/^x//' -e 's/"/\\\"/g' -e 's/'\''/'\\\'\''/g' -e 's/ /\\\ /g'`
+
+			optCompile="$mod_arg $optCompile"
+			optIncludeDefs="$theDefine $optIncludeDefs"
 			;;
 
+		-I*|-D*)
+
+		        mod_arg=`echo "x$arg" | sed -e 's/^x//' -e 's/"/\\\"/g' -e 's/'\''/'\\\'\''/g' -e 's/ /\\\ /g'`
+			optPdtCFlags="$mod_arg $optPdtCFlags"
+			optPdtCxxFlags="$mod_arg $optPdtCxxFlags"
+			optPdtF95="$mod_arg $optPdtF95"
+			optCompile="$mod_arg $optCompile"
+			optIncludeDefs="$mod_arg $optIncludeDefs"
+			;;
+
+
+	        -qfixed*)
+		        optPdtF95="$optPdtF95 -R fixed"
+			argsRemaining="$argsRemaining $arg"
+		        ;;
+
+	        -qfree*)
+		        optPdtF95="$optPdtF95 -R free"
+			argsRemaining="$argsRemaining $arg"
+		        ;;
+
 		-c)
 			isForCompilation=$TRUE
 			argsRemaining="$argsRemaining $arg"
 			;;
 
-		-M*|-S|-E)
+		-M|-MM|-S|-E)
 # We ignore -M processing step for making dependencies, -S for assembly
 # and -E for preprocessing the source code. These are the options that are 
 # ignored by PDT. Add to this list if you need an additional option that should
@@ -404,6 +559,31 @@ for arg in "$@"
 			;;
 
 
+		-o)
+			hasAnOutputFile=$TRUE
+			counterForOutput=$tempCounter
+			echoIfDebug "\tHas an output file"
+			#With compilation, a new output file is created and is written with -o
+			#options, so no need to append it to argsRemaining. WIth
+			#others it is simply added to the command.
+			#-o is added later
+			;;
+
+		-o*)
+ 			if [ "x$arg" != "x-openmp" -a "x$arg" != "x-override_limits" ]; then
+			    hasAnOutputFile=$TRUE
+			    passedOutputFile="${arg#"-o"}"
+			    echoIfDebug "\tHas an output file = $passedOutputFile"
+			#With compilation, a new output file is created and is written with -o
+			#options, so no need to append it to argsRemaining. WIth
+			#others it is simply added to the command.
+			#-o is added later
+		        else
+				argsRemaining="$argsRemaining ""$arg"
+ 			fi
+			;;
+
+
 		*.o)
 			objectOutputFile="$arg"
 			hasAnObjectOutputFile=$TRUE
@@ -420,25 +600,7 @@ for arg in "$@"
 			fi
 			;;
 
-		-o)
-			hasAnOutputFile=$TRUE
-			counterForOutput=$tempCounter
-			echoIfDebug "\tHas an output file"
-			#With compilation, a new output file is created and is written with -o
-			#options, so no need to append it to argsRemaining. WIth
-			#others it is simply added to the command.
-			#-o is added later
-			;;
 
-		-o*)
-			hasAnOutputFile=$TRUE
-			passedOutputFile="${arg#"-o"}"
-			echoIfDebug "\tHas an output file = $passedOutputFile"
-			#With compilation, a new output file is created and is written with -o
-			#options, so no need to append it to argsRemaining. WIth
-			#others it is simply added to the command.
-			#-o is added later
-			;;
 
 		$CMD)
 			;;
@@ -479,9 +641,51 @@ while [ $tempCounter -lt $numFiles ]; do
 	#Here arrays holding sourcefiles, .inst. and .pdb files
 	#are created based on the baseName of the source file.
 	echoIfDebug "FileName: ${arrFileName[$tempCounter]}" 
-	base=`echo ${arrFileName[$tempCounter]} | sed -e 's/\.[^\.]*$//'`
+	base=`echo ${arrFileName[$tempCounter]} | sed -e 's/\.[^\.]*$//' -e's/.*\///'`
+	# this transforms /this/file\ name/has/spaces/ver1.0.2/foo.pp.F90 to foo.pp for base
 	suf=`echo ${arrFileName[$tempCounter]} | sed -e 's/.*\./\./' `
+	# suf gets .F90 in the example above.
 	#echoIfDebug "suffix here is -- $suf"
+	# If we need to pre-process the source code, we should do so here!
+	if [ $preprocess = $TRUE -a $groupType == $group_f_F ]; then
+	  base=${base}.pp
+	  cmdToExecute="${preprocessor} $preprocessorOpts $optTauIncludes $optIncludeDefs ${arrFileName[$tempCounter]} $base$suf"
+	  evalWithDebugMessage "$cmdToExecute" "Preprocessing"
+          if [ ! -f $base$suf ]; then
+            echoIfVerbose "ERROR: Did not generate .pp file"
+	    printError "$preprocessor" "$cmdToExecute"
+          fi
+	  arrFileName[$tempCounter]=$base$suf
+	  echoIfDebug "Completed Preprocessing\n"
+        fi
+	# Before we pass it to Opari for OpenMP instrumentation
+	# we should use tau_ompcheck to verify that OpenMP constructs are 
+	# used correctly.
+	if [ $opari == $TRUE -a $pdtUsed == $TRUE ]; then
+	
+	  case $groupType in
+	    $group_f_F)
+	      pdtParserCmd="$pdtParserF ${arrFileName[$tempCounter]} $optPdtUser ${optPdtF95} $optIncludes"
+	      ;;
+	    $group_c)
+	      pdtParserCmd="$optPdtDir/$pdtParserType ${arrFileName[$tempCounter]} $optPdtCFlags $optPdtUser $optDefines $optIncludes"
+	      ;;
+	    $group_C)
+	      pdtParserCmd="$optPdtDir/$pdtParserType ${arrFileName[$tempCounter]} $optPdtCxxFlags $optPdtUser $optDefines $optIncludes"
+	      ;;
+	  esac
+	  evalWithDebugMessage "$pdtParserCmd" "Parsing with PDT for OpenMP directives verification:" 
+	  pdbcommentCmd="$optPdtDir/pdbcomment -o ${base}.comment.pdb ${base}.pdb"
+	  
+	  evalWithDebugMessage "$pdbcommentCmd" "Using pdbcomment:" 
+
+	  ompcheck=`echo $optTauInstr | sed -e 's@tau_instrumentor@tau_ompcheck@'` 
+	  ompcheckCmd="$ompcheck ${base}.comment.pdb ${arrFileName[$tempCounter]} -o ${base}.chk${suf}"
+	  arrFileName[$tempCounter]=$base.chk$suf
+	  base=${base}.chk
+	  evalWithDebugMessage "$ompcheckCmd" "Using tau_ompcheck:" 
+	fi
+	# And then pass it on to opari for OpenMP programs.
 	if [ $opari = $TRUE ]; then
 	  base=${base}.pomp
 	  cmdToExecute="${optOpariTool} $optOpariOpts ${arrFileName[$tempCounter]} $base$suf"
@@ -530,15 +734,15 @@ if [ $numFiles == 0 ]; then
 
 	if [ $hasAnOutputFile == $FALSE ]; then
 		passedOutputFile="a.out"
-		linkCmd="$regularCmd  $optLinking -o $passedOutputFile"
+		linkCmd="$compilerSpecified $regularCmd $optLinking -o $passedOutputFile"
 	else
-		linkCmd="$regularCmd  $optLinking"
+		linkCmd="$compilerSpecified $regularCmd $optLinking"
 		#Do not add -o, since the regular command has it already.
     fi	
 
 	if [ $opari == $TRUE ]; then
 	  evalWithDebugMessage "/bin/rm -f opari.rc" "Removing opari.rc"
-	  cmdCompileOpariTab="${optTauCC} -c ${optIncludeDefs} opari.tab.c"
+	  cmdCompileOpariTab="${optTauCC} -c ${optIncludeDefs} ${optIncludes} ${optDefs} opari.tab.c"
 	  evalWithDebugMessage "$cmdCompileOpariTab" "Compiling opari.tab.c"
 	  linkCmd="$linkCmd opari.tab.o"
 	fi
@@ -553,6 +757,9 @@ if [ $numFiles == 0 ]; then
 	gotoNextStep=$FALSE
 	if [ $opari == $TRUE -a $needToCleanPdbInstFiles == $TRUE ]; then
 	  evalWithDebugMessage "/bin/rm -f opari.tab.c opari.tab.o *.opari.inc" "Removing opari.tab.c opari.tab.o *.opari.inc"
+	  if [ $pdtUsed == $TRUE ]; then
+	    evalWithDebugMessage "/bin/rm -f *.comment.pdb *.chk.* *.pomp.* *.pp.pdb *.comment.pdb" "Removing *.chk *.pomp.* *.comment.pdb *.pp.pdb"
+	  fi
 	fi
 fi
 
@@ -577,19 +784,22 @@ if [ $gotoNextStep == $TRUE ]; then
 				pdtCmd="$pdtParserF"
 				pdtCmd="$pdtCmd ${arrFileName[$tempCounter]}"
 				pdtCmd="$pdtCmd $optPdtUser"
-				pdtCmd="$pdtCmd ${optPdtF95} "
+				pdtCmd="$pdtCmd ${optPdtF95} $optIncludes"
+				optCompile="$optCompile $optIncludes"
 				;;
 
 			$group_c)
 				pdtCmd="$optPdtDir""/$pdtParserType"
 				pdtCmd="$pdtCmd ${arrFileName[$tempCounter]} "
 				pdtCmd="$pdtCmd $optPdtCFlags $optPdtUser "
+				optCompile="$optCompile $optDefs $optIncludes"
 				;;
 
 			$group_C)
 				pdtCmd="$optPdtDir""/$pdtParserType"
 				pdtCmd="$pdtCmd ${arrFileName[$tempCounter]} "
 				pdtCmd="$pdtCmd $optPdtCxxFlags $optPdtUser "
+				optCompile="$optCompile $optDefs $optIncludes"
 				;;
 
 		esac
@@ -678,12 +888,15 @@ if [ $gotoNextStep == $TRUE ]; then
 		#The number of files could be more than one. Check for creation of each .o file.
 		tempCounter=0
 		while [ $tempCounter -lt $numFiles ]; do
-			base=`echo ${arrFileName[$tempCounter]} | sed -e 's/\.[^\.]*$//'`
+			base=`echo ${arrFileName[$tempCounter]} | sed -e 's/\.[^\.]*$//'  -e's/.*\///'`
 			suf=`echo ${arrFileName[$tempCounter]} | sed -e 's/.*\./\./' `
 			outputFile=${base##*/}.o	#strip it off the directory
 			# Remove the .pomp from the name of the output file. 
-			if [ $opari == $TRUE ]; then
-			  outputFile=`echo $outputFile | sed -e 's/\.pomp//'`
+			if [ $preprocess == $TRUE ]; then
+			  outputFile=`echo $outputFile | sed -e 's/\.pp//'`
+			fi
+			if [ $opari == $TRUE -a $pdtUsed == $TRUE ]; then
+			  outputFile=`echo $outputFile | sed -e 's/\.chk\.pomp//'`
 			fi
 			
 				
@@ -693,7 +906,7 @@ if [ $gotoNextStep == $TRUE ]; then
 			tempTauFileName=${arrTau[$tempCounter]##*/}
 			instrumentedFileForCompilation="$tempTauFileName"
 #			newCmd="$CMD  $argsRemaining $instrumentedFileForCompilation  $OUTPUTARGSFORTAU $optCompile"
-			newCmd="$CMD -I ${arrFileNameDirectory[$tempCounter]} $argsRemaining $instrumentedFileForCompilation  $OUTPUTARGSFORTAU $optCompile"
+			newCmd="$CMD -I${arrFileNameDirectory[$tempCounter]} $argsRemaining $instrumentedFileForCompilation  $OUTPUTARGSFORTAU $optCompile"
 
 			#echoIfDebug "cmd before appending the .o file is $newCmd"
 			if [ $hasAnOutputFile == $TRUE ]; then
@@ -729,7 +942,7 @@ if [ $gotoNextStep == $TRUE ]; then
 		tempCounter=0
 		while [ $tempCounter -lt $numFiles ]; do
 
-			base=`echo ${arrFileName[$tempCounter]} | sed -e 's/\.[^\.]*$//'`
+			base=`echo ${arrFileName[$tempCounter]} | sed -e 's/\.[^\.]*$//'  -e's/.*\///'`
 			suf=`echo ${arrFileName[$tempCounter]} | sed -e 's/.*\./\./' `
 			outputFile=${base##*/}.o	#strip it off the directory
 
@@ -739,7 +952,7 @@ if [ $gotoNextStep == $TRUE ]; then
 			instrumentedFileForCompilation=" $tempTauFileName"
 
 #			newCmd="$CMD $argsRemaining  -c $instrumentedFileForCompilation  $OUTPUTARGSFORTAU $optCompile -o $outputFile"
-			newCmd="$CMD $argsRemaining  -I ${arrFileNameDirectory[$tempCounter]} -c $instrumentedFileForCompilation  $OUTPUTARGSFORTAU $optCompile -o $outputFile"
+			newCmd="$CMD $argsRemaining  -I${arrFileNameDirectory[$tempCounter]} -c $instrumentedFileForCompilation  $OUTPUTARGSFORTAU $optCompile -o $outputFile"
 
 			evalWithDebugMessage "$newCmd" "Compiling (Individually) with Instrumented Code"
 			if [  ! -e $outputFile ]; then
@@ -758,12 +971,13 @@ if [ $gotoNextStep == $TRUE ]; then
 	
 		if [ $opari == $TRUE ]; then
 	  	  evalWithDebugMessage "/bin/rm -f opari.rc" "Removing opari.rc"
-	  	  cmdCompileOpariTab="${optTauCC} -c ${optIncludeDefs} opari.tab.c"
+	          cmdCompileOpariTab="${optTauCC} -c ${optIncludeDefs} ${optIncludes} ${optDefs} opari.tab.c"
+
 		  evalWithDebugMessage "$cmdCompileOpariTab" "Compiling opari.tab.c"
 	  	  objectFilesForLinking="$objectFilesForLinking opari.tab.o"
 		fi
 
-		newCmd="$CMD  $argsRemaining $listOfObjectFiles $objectFilesForLinking $OUTPUTARGSFORTAU $optLinking -o $passedOutputFile"
+		newCmd="$CMD $listOfObjectFiles $objectFilesForLinking $argsRemaining $OUTPUTARGSFORTAU $optLinking -o $passedOutputFile"
 		evalWithDebugMessage "$newCmd" "Linking (Together) object files"
 
 		if [ ! -e $passedOutputFile ] ; then
@@ -781,9 +995,21 @@ fi
 if [ $needToCleanPdbInstFiles == $TRUE ]; then
 	tempCounter=0
 	while [ $tempCounter -lt $numFiles -a $disablePdtStep == $FALSE ]; do
-		eval "/bin/rm ${arrTau[$tempCounter]##*/}"
+		evalWithDebugMessage "/bin/rm -f ${arrTau[$tempCounter]##*/}" "cleaning inst file"
+		if [ $preprocess == $TRUE -a $groupType == $group_f_F ]; then
+		  if [ $opari == $TRUE ]; then
+		    secondSource=`echo ${arrTau[$tempCounter]##*/} | sed -e 's/\.chk\.pomp\.inst//'`
+		  else
+		    secondSource=`echo ${arrTau[$tempCounter]##*/} | sed -e 's/\.inst//'`
+		  fi
+		  evalWithDebugMessage "/bin/rm -f $secondSource" "cleaning pp file"
+		fi
 		if [ $pdbFileSpecified == $FALSE ]; then
 		  evalWithDebugMessage "/bin/rm -f ${arrPdb[$tempCounter]##*/}" "cleaning PDB file"
+		  if [ $preprocess == $TRUE -o $opari == $TRUE ]; then
+		    secondPDB=`echo $outputFile | sed -e 's/\.o/\.pdb/'`
+		    evalWithDebugMessage "/bin/rm -f $secondPDB" "cleaning PDB file"
+		  fi
 		fi
 		if [ $opari == $TRUE ]; then
 		  evalWithDebugMessage "/bin/rm -f ${arrFileName[$tempCounter]}" "cleaning opari file"
@@ -797,6 +1023,35 @@ fi
 #Regular Command: In case of an Intermediate Error. 
 ####################################################################
 if [ $errorStatus == $TRUE ] ; then
-	evalWithDebugMessage "$regularCmd" "Compiling with Non-Instrumented Regular Code"
+	if [ $revertOnError == $TRUE ]; then
+
+	    if [ $groupType == $group_f_F ]; then
+		if [ "x$optAppF90" == "x" ]; then
+		    regularCmd="$compilerSpecified $regularCmd"
+		else
+		    regularCmd="$optAppF90 $regularCmd"
+		fi
+	    elif [ $groupType == $group_c ]; then
+		if [ "x$optAppCC" == "x" ]; then
+		    regularCmd="$compilerSpecified $regularCmd"
+		else
+		    regularCmd="$optAppCC $regularCmd"
+		fi
+	    elif [ $groupType == $group_C ]; then
+		if [ "x$optAppCXX" == "x" ]; then
+		    regularCmd="$compilerSpecified $regularCmd"
+		else
+		    regularCmd="$optAppCXX $regularCmd"
+		fi
+	    else
+		    regularCmd="$compilerSpecified $regularCmd"
+	    fi
+
+	    evalWithDebugMessage "$regularCmd" "Compiling with Non-Instrumented Regular Code"
+
+	else
+		echo "Reverting to uninstrumented command disabled. To enable reverting pass -optRevert to tau_compiler.sh."
+		exit 1
+	fi
 fi
 echo -e ""
